package DAOTests;

import java.util.Arrays;
import java.util.List;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.transaction.annotation.Transactional;

import TestConfig.DAOTestConfiguration;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.Connection;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.ErrorMetadata;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.ConnectionsDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.ErrorDefinitionDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.ErrorMetadataDAO;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

@RunWith(SpringRunner.class)
@Transactional
@ContextConfiguration(classes = {DAOTestConfiguration.class})
@ActiveProfiles({"test"})
public class ErrorMetadataDAOTests {

    @Autowired
    private ErrorMetadataDAO errorMetadataDAO;

    @Autowired
    private ConnectionsDAO connectionsDAO;

    @Autowired
    private ErrorDefinitionDAO errorDefinitionDAO;

    private Long errorDefinitionId = null;
    private String workflowId = "WE1";
    private String executionId = "SE1";
    private String connectionId = "OracleTestDB";
    private String application = "Test Application";
    private Long connDbId = null;

    private final Long ONE = 1L;

    @Before
    public void before(){
        connDbId = connectionsDAO.getIdByData(new Connection(null, connectionId, executionId, null));
        errorDefinitionId = errorDefinitionDAO.getAll().get(0);
    }

    @Test
    public void save_addNewErrorMetadata_added(){
        ErrorMetadata errorMetadata = new ErrorMetadata(
            errorDefinitionId,
            workflowId,
            executionId,
            connDbId,
            null,
            application
        );

        Long insertedRows = errorMetadataDAO.save(errorMetadata);

        assertEquals(ONE, insertedRows);

        Long id = errorMetadata.getErrorId();
        ErrorMetadata foundErrorMetadata = errorMetadataDAO.find(id);

        assertEquals(errorMetadata.getWorkflowExecutionId(), foundErrorMetadata.getWorkflowExecutionId());
        assertEquals(errorMetadata.getScenarioExecutionId(), foundErrorMetadata.getScenarioExecutionId());
        assertEquals(errorMetadata.getConnDbId(), foundErrorMetadata.getConnDbId());
        assertEquals(errorMetadata.getApplication(), foundErrorMetadata.getApplication());
        assertEquals(errorMetadata.getPackageName(), foundErrorMetadata.getPackageName());
    }

    @Test
    public void delete_deleteExistingErrorMetadata_deleted(){
        ErrorMetadata errorMetadata = new ErrorMetadata(
                errorDefinitionId,
                workflowId,
                executionId,
                connDbId,
                null,
                application
        );

        Long dbId = errorMetadataDAO.getIdByData(errorMetadata);
        assertNotNull(dbId);

        Long deletedRows = errorDefinitionDAO.delete(Arrays.asList(dbId));
        assertEquals(ONE, deletedRows);

        ErrorMetadata fondErrorMetadata = errorMetadataDAO.find(dbId);
        assertNull(fondErrorMetadata);
    }

    @Test
    public void find_findExistingErrorMetadata_found(){
        ErrorMetadata errorMetadata = new ErrorMetadata(
                errorDefinitionId,
                workflowId,
                executionId,
                connDbId,
                null,
                application
        );

        Long dbId = errorMetadataDAO.getIdByData(errorMetadata);
        assertNotNull(dbId);

        ErrorMetadata foundErrorMetadata = errorMetadataDAO.find(dbId);

        assertEquals(errorMetadata.getWorkflowExecutionId(), foundErrorMetadata.getWorkflowExecutionId());
        assertEquals(errorMetadata.getScenarioExecutionId(), foundErrorMetadata.getScenarioExecutionId());
        assertEquals(errorMetadata.getConnDbId(), foundErrorMetadata.getConnDbId());
        assertEquals(errorMetadata.getApplication(), foundErrorMetadata.getApplication());
        assertEquals(errorMetadata.getPackageName(), foundErrorMetadata.getPackageName());
    }

    @Test
    public void getAll_retrieveAllErrorMetadata_oneFound(){
        ErrorMetadata errorMetadata = new ErrorMetadata(
                errorDefinitionId,
                workflowId,
                executionId,
                connDbId,
                null,
                application
        );

        List<Long> errorMetadataList = errorMetadataDAO.getAllIds();

        assertEquals(2, errorMetadataList.size());

        Long dbId = errorMetadataList.get(0);

        ErrorMetadata foundErrorMetadata = errorMetadataDAO.find(dbId);

        assertEquals(errorMetadata.getWorkflowExecutionId(), foundErrorMetadata.getWorkflowExecutionId());
        assertEquals(errorMetadata.getScenarioExecutionId(), foundErrorMetadata.getScenarioExecutionId());
        assertEquals(errorMetadata.getConnDbId(), foundErrorMetadata.getConnDbId());
        assertEquals(errorMetadata.getApplication(), foundErrorMetadata.getApplication());
        assertEquals(errorMetadata.getPackageName(), foundErrorMetadata.getPackageName());
    }

    @Test
    public void getDistinct_getDistinctErrorDefinitions_shouldReturnOneAfterInsertingASameErrorForDifferentApplication(){
        ErrorMetadata errorMetadata = new ErrorMetadata(
                errorDefinitionId,
                workflowId,
                executionId,
                connDbId,
                null,
                "Manta Server"
        );

        Long insertedRows = errorMetadataDAO.save(errorMetadata);
        assertEquals(ONE, insertedRows);

        List<Long> distinctErrorDefinitions = errorDefinitionDAO.getAll();
        assertEquals(ONE, new Long(distinctErrorDefinitions.size()));
    }


}
