package DAOTests;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.transaction.annotation.Transactional;

import TestConfig.DAOTestConfiguration;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.Connection;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.ErrorMetadata;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.filter.Filter;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.DebugLog;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.ErrorLog;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.Log;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.LogMetadata;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.ConnectionsDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.ErrorMetadataDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.LogDAO;

import static utils.Utils.checkDebugLogEquals;
import static utils.Utils.checkErrorLogEquals;
import static utils.Utils.checkErrorMetadataEquals;
import static utils.Utils.getDebugLog;
import static utils.Utils.getErrorLog;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

@RunWith(SpringRunner.class)
@Transactional
@ContextConfiguration(classes = {DAOTestConfiguration.class})
@ActiveProfiles({"test"})
public class LogDAOTests {

    @Autowired
    private LogDAO logDAO;

    @Autowired
    private ErrorMetadataDAO errorMetadataDAO;

    @Autowired
    private ConnectionsDAO connectionsDAO;

    private Long errorId = null;
    private Long connDbId = null;
    private String workflowId = "WE1";
    private String executionId = "SE1";
    private String connectionId = "OracleTestDB";
    private String application = "Test Application";

    private String errorCategory = "TEST_CATEGORY";
    private String errorType = "TEST_ERROR";
    private String errorLogType = "ERROR";

    private final Long ONE = 1L;

    @Before
    public void before(){
        connDbId = connectionsDAO.getIdByData(new Connection(null, connectionId, executionId, null));
        errorId = errorMetadataDAO.getAllIds().get(0);
    }

    @Test
    public void save_addNewErrorLogBelongingToConnection_added(){
        ErrorLog errorLog = getErrorLog(workflowId, executionId, connectionId);

        Long insertedRows = logDAO.save(errorLog, errorId, connDbId);
        assertEquals(ONE, insertedRows);

        List<? extends Log> allLogs = logDAO.getAll();

        assertEquals(ONE, new Long(allLogs.size()));

        ErrorLog foundLog = (ErrorLog) allLogs.get(0);

        checkErrorLogEquals(errorLog, foundLog);
    }

    @Test
    public void save_addNewDebugLogBelongingToConnection_added(){
        DebugLog debugLog = getDebugLog(workflowId, executionId, connectionId);
        Long insertedRows = logDAO.save(debugLog, connDbId);
        assertEquals(ONE, insertedRows);

        List<? extends Log> allLogs = logDAO.getAll();

        assertEquals(ONE, new Long(allLogs.size()));

        DebugLog foundLog = (DebugLog) allLogs.get(0);

        checkDebugLogEquals(debugLog, foundLog);
    }

    @Test
    public void save_addNewErrorWithoutConnection_added(){
        ErrorLog errorLog = getErrorLog(workflowId, executionId, null);

        Long insertedRows = logDAO.save(errorLog, errorId, null);
        assertEquals(ONE, insertedRows);

        List<? extends Log> allLogs = logDAO.getAll();

        assertEquals(ONE, new Long(allLogs.size()));

        ErrorLog foundLog = (ErrorLog) allLogs.get(0);

        checkErrorLogEquals(errorLog, foundLog);
    }

    @Test
    public void save_addNewDebugLogWithoutConnection_added(){
        DebugLog debugLog = getDebugLog(workflowId, executionId, null);
        Long insertedRows = logDAO.save(debugLog, null);
        assertEquals(ONE, insertedRows);

        List<? extends Log> allLogs = logDAO.getAll();

        assertEquals(ONE, new Long(allLogs.size()));

        DebugLog foundLog = (DebugLog) allLogs.get(0);

        checkDebugLogEquals(debugLog, foundLog);
    }

    @Test
    public void save_addNewErrorWithoutScenario_added(){
        ErrorLog errorLog = getErrorLog(workflowId, null, null);

        Long insertedRows = logDAO.save(errorLog, errorId, null);
        assertEquals(ONE, insertedRows);

        List<? extends Log> allLogs = logDAO.getAll();

        assertEquals(ONE, new Long(allLogs.size()));

        ErrorLog foundLog = (ErrorLog) allLogs.get(0);

        checkErrorLogEquals(errorLog, foundLog);
    }

    @Test
    public void save_addNewDebugWithoutScenario_added(){
        DebugLog debugLog = getDebugLog(workflowId, null, null);
        Long insertedRows = logDAO.save(debugLog, null);
        assertEquals(ONE, insertedRows);

        List<? extends Log> allLogs = logDAO.getAll();

        assertEquals(ONE, new Long(allLogs.size()));

        DebugLog foundLog = (DebugLog) allLogs.get(0);

        checkDebugLogEquals(debugLog, foundLog);
    }

    @Test(expected = DataIntegrityViolationException.class)
    public void save_addNewErrorLogNonExistingScenarioExecution_dataIntegrityExceptionThrown(){
        ErrorLog errorLog = getErrorLog(workflowId, "SE2", null);

        Long insertedRows = logDAO.save(errorLog, errorId, null);
        assertEquals(ONE, insertedRows);

        List<? extends Log> allLogs = logDAO.getAll();

        assertEquals(ONE, new Long(allLogs.size()));

        ErrorLog foundLog = (ErrorLog) allLogs.get(0);

        checkErrorLogEquals(errorLog, foundLog);
    }

    @Test
    public void fullText_searchErrorUsingRegex_errorFound(){
        String fullTextString = "T*";
        Filter filter = new Filter.FilterBuilder().buildFilter();

        List<ErrorMetadata> errors = logDAO.fullText(fullTextString, filter);

        assertEquals(ONE, new Long(errors.size()));
    }

    @Test
    public void getLogMetadata_findLogMetadata_found(){
        ErrorLog errorLog = getErrorLog(workflowId, executionId, connectionId);

        Long insertedRows = logDAO.save(errorLog, errorId, connDbId);
        assertEquals(ONE, insertedRows);

        Long id = errorLog.getLogId();
        assertNotNull(id);

        LogMetadata logMetadata = logDAO.getLogMetadata(id);

        assertEquals(errorLogType, logMetadata.getType());
        assertEquals(connDbId, logMetadata.getConnDbId());
        assertEquals(executionId, logMetadata.getScenarioExecutionId());
        assertNull(logMetadata.getPackageName());
        assertEquals(workflowId, logMetadata.getWorkflowExecutionId());
    }

    @Test
    public void find_findRootLogForContext_found(){
        ErrorLog errorLog = getErrorLog(workflowId, executionId, connectionId);

        Long insertedRows = logDAO.save(errorLog, errorId, connDbId);
        assertEquals(ONE, insertedRows);

        Long id = errorLog.getLogId();
        assertNotNull(id);

        LogMetadata logMetadata = logDAO.getLogMetadata(id);

        ErrorLog foundLog = (ErrorLog) logDAO.find(id, logMetadata);

        checkErrorLogEquals(errorLog, foundLog);
    }

    @Test
    public void loadEarlier_findLogsThatOccurredBeforeTheRootLog_found(){
        List<Long> insertedIds = new ArrayList<>();

        for (int i = 0; i < 5; ++i){
            DebugLog debugLog = getDebugLog(workflowId, executionId, connectionId);
            Long insertedRows = logDAO.save(debugLog, connDbId);
            assertEquals(ONE, insertedRows);
            insertedIds.add(debugLog.getLogId());
        }

        ErrorLog errorLog = getErrorLog(workflowId, executionId, connectionId);
        Long insertedRows = logDAO.save(errorLog, errorId, connDbId);
        assertEquals(ONE, insertedRows);

        LogMetadata logMetadata = logDAO.getLogMetadata(errorLog.getLogId());

        List<Log> logs = logDAO.loadEarlier(errorLog, errorLog.getLogId(), 0, 5, logMetadata);
        assertEquals(5, logs.size());

        logs.forEach(log -> {
            assertTrue(insertedIds.contains(log.getLogId()));
        });
    }

    @Test
    public void loadLater_findLogsThatOccurredBeforeTheRootLog_found(){
        ErrorLog errorLog = getErrorLog(workflowId, executionId, connectionId);
        Long insertedRows = logDAO.save(errorLog, errorId, connDbId);
        assertEquals(ONE, insertedRows);

        List<Long> insertedIds = new ArrayList<>();

        for (int i = 0; i < 5; ++i){
            DebugLog debugLog = getDebugLog(workflowId, executionId, connectionId);
            Long insertedRowsDebug = logDAO.save(debugLog, connDbId);
            assertEquals(ONE, insertedRowsDebug);
            insertedIds.add(debugLog.getLogId());
        }

        LogMetadata logMetadata = logDAO.getLogMetadata(errorLog.getLogId());

        List<Log> logs = logDAO.loadLater(errorLog, errorLog.getLogId(), 0, 5, logMetadata);
        assertEquals(5, logs.size());

        logs.forEach(log -> {
            assertTrue(insertedIds.contains(log.getLogId()));
        });
    }

    @Test
    public void queryLogs_queryLogsWithoutFilter_found(){
        ErrorLog errorLog = getErrorLog(workflowId, executionId, connectionId);
        Long insertedRows = logDAO.save(errorLog, errorId, connDbId);
        assertEquals(ONE, insertedRows);
        Filter filter = new Filter.FilterBuilder().buildFilter();

        List<ErrorMetadata> errors = logDAO.queryLogs(filter);
        assertEquals(ONE, new Long(errors.size()));

        checkErrorMetadataEquals(errorLog, errors.get(0));
    }

    @Test
    public void queryLogs_queryLogsWantParticularError_noneFound(){
        ErrorLog errorLog = getErrorLog(workflowId, executionId, connectionId);
        Long insertedRows = logDAO.save(errorLog, errorId, connDbId);
        assertEquals(ONE, insertedRows);
        Filter.FilterBuilder builder = new Filter.FilterBuilder();
        builder.setErrorTypes(Arrays.asList("ERROR_TYPE_THAT_IS_NOT_THERE"));
        Filter filter = builder.buildFilter();

        List<ErrorMetadata> errors = logDAO.queryLogs(filter);
        assertEquals(0, errors.size());
    }

    @Test
    public void queryLogs_queryLogsWantParticularConnectionId_oneFound(){
        ErrorLog errorLog = getErrorLog(workflowId, executionId, connectionId);
        Long insertedRows = logDAO.save(errorLog, errorId, connDbId);
        assertEquals(ONE, insertedRows);
        Filter.FilterBuilder builder = new Filter.FilterBuilder();
        builder.setWorkflowExecutionId(workflowId);
        builder.setScenarioExecutionId(executionId);
        builder.setConnectionId(connectionId);
        Filter filter = builder.buildFilter();

        List<ErrorMetadata> errors = logDAO.queryLogs(filter);
        assertEquals(ONE, new Long(errors.size()));

        checkErrorMetadataEquals(errorLog, errors.get(0));
    }

    @Test
    public void queryLogs_queryLogsWrongExeuction_noneFound(){
        ErrorLog errorLog = getErrorLog(workflowId, executionId, connectionId);
        Long insertedRows = logDAO.save(errorLog, errorId, connDbId);
        assertEquals(ONE, insertedRows);
        Filter.FilterBuilder builder = new Filter.FilterBuilder();
        builder.setWorkflowExecutionId(workflowId);
        builder.setScenarioExecutionId("EXECUTION THAT DOES NOT EXIST");
        builder.setConnectionId(connectionId);
        Filter filter = builder.buildFilter();

        List<ErrorMetadata> errors = logDAO.queryLogs(filter);
        assertEquals(0, errors.size());
    }

    @Test
    public void expandLogs_getConcreteLogsForAnErrorType_fiveFound(){
        for (int i = 0; i < 5; ++i){
            ErrorLog errorLog = getErrorLog(workflowId, executionId, connectionId);
            Long insertedRows = logDAO.save(errorLog, errorId, connDbId);
            assertEquals(ONE, insertedRows);
        }

        Filter.FilterBuilder builder = new Filter.FilterBuilder();
        Filter filter = builder.buildFilter();

        List<ErrorLog> errors = logDAO.expandLogs(errorCategory, errorType, application, filter);
        assertEquals(5, errors.size());
    }

    @Test
    public void expandLogs_getConcreteLogsForErrorTypeDoesNotExist_noneFound(){
        for (int i = 0; i < 5; ++i){
            ErrorLog errorLog = getErrorLog(workflowId, executionId, connectionId);
            Long insertedRows = logDAO.save(errorLog, errorId, connDbId);
            assertEquals(ONE, insertedRows);
        }

        Filter.FilterBuilder builder = new Filter.FilterBuilder();
        Filter filter = builder.buildFilter();

        List<ErrorLog> errors = logDAO.expandLogs(errorCategory, "SOME TYPE THAT DOES NOT EXIST", application, filter);
        assertEquals(0, errors.size());
    }

    @Test
    public void expandLogs_getConcreteLogsForConnectionWhileFilteringFromOther_oneFound(){
        ErrorLog errorLog = getErrorLog(workflowId, executionId, connectionId);
        Long insertedRows = logDAO.save(errorLog, errorId, connDbId);
        assertEquals(ONE, insertedRows);

        ErrorLog otherLog = getErrorLog(workflowId, executionId, "CONNECTION THAT DOES NOT EXIST");
        Long insertedRows2 = logDAO.save(otherLog, errorId, null);
        assertEquals(ONE, insertedRows2);

        Filter.FilterBuilder builder = new Filter.FilterBuilder();
        builder.setWorkflowExecutionId(workflowId);
        builder.setScenarioExecutionId(executionId);
        builder.setConnectionId(connectionId);
        Filter filter = builder.buildFilter();

        List<ErrorLog> errors = logDAO.expandLogs(errorCategory, errorType, application, filter);
        assertEquals(ONE, new Long(errors.size()));
    }

    @Test
    public void pruneLogsFromApplication_pruneLogsFromAParticularApplication_oneDeleted(){
        ErrorLog errorLog = getErrorLog(workflowId, executionId, connectionId);
        Long insertedRows = logDAO.save(errorLog, errorId, connDbId);
        assertEquals(ONE, insertedRows);

        ErrorLog otherLog = getErrorLog(workflowId, executionId, connectionId, "Some other application");
        Long insertedRows2 = logDAO.save(otherLog, errorId, connDbId);
        assertEquals(ONE, insertedRows2);

        Integer deleted = logDAO.pruneLogsFromApplication(application);
        assertEquals(ONE, new Long(deleted));
    }

    @Test
    public void pruneNonExpiredExecutionLogs_pruneExpiredLogsThatDoNotBelongToWorkflow_oneDeleted(){
        ErrorLog errorLog = getErrorLog(workflowId, executionId, connectionId);
        Long insertedRows = logDAO.save(errorLog, errorId, connDbId);
        assertEquals(ONE, insertedRows);

        ErrorLog otherLog = getErrorLog(null, null, null);
        Long insertedRows2 = logDAO.save(otherLog, errorId, connDbId);
        assertEquals(ONE, insertedRows2);

        Integer deleted = logDAO.pruneExpiredNonExecutionLogs(0); // 0 means automatically deleted
        assertEquals(ONE, new Long(deleted));
    }

}
