package DAOTests;

import java.util.List;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.transaction.annotation.Transactional;

import TestConfig.DAOTestConfiguration;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.Connection;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.ErrorDefinition;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.PackageMetadata;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.ErrorLog;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.UploadedLog;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.ConnectionsDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.ErrorMetadataDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.LogDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.PackageCreateDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.PackageUploadDAO;

import static utils.Utils.getErrorLog;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;

@RunWith(SpringRunner.class)
@Transactional
@ContextConfiguration(classes = {DAOTestConfiguration.class})
@ActiveProfiles({"test"})
public class PackageCreateDAOTests {

    @Autowired
    private PackageCreateDAO packageCreateDAO;

    @Autowired
    private ConnectionsDAO connectionsDAO;

    @Autowired
    private PackageUploadDAO packageUploadDAO;

    @Autowired
    private LogDAO logDAO;

    @Autowired
    private ErrorMetadataDAO errorMetadataDAO;

    private String workflowId = "WE1";
    private String executionId = "SE1";
    private String connectionId = "OracleTestDB";

    private String technology = "Oracle";
    private String phase = "Analysis";
    private String scenarioName = "Oracle DDL Dataflow Analysis";

    private String errorCategory = "TEST_CATEGORY";
    private String errorType = "TEST_ERROR";
    private String userMessage = "Test user message.";
    private String technicalMessage = "Test technical message.";
    private String solution = "Test solution.";
    private String impact = "Test impact.";
    private String logLevel = "ERROR";

    private String application = "Test Application";

    private Long connDbId;
    private Long errorId;

    private final Long ONE = new Long(1);

    @Before
    public void before(){
        connDbId = connectionsDAO.getIdByData(new Connection(null, connectionId, executionId, null));
        errorId = errorMetadataDAO.getAllIds().get(0);
    }

    @Test
    public void getPackageMetadata_createMetadataForAPotentialPackage_success(){
        Connection connection = new Connection(
                null,
                connectionId,
                executionId,
                null
        );

        PackageMetadata packageMetadata = packageCreateDAO.getPackageMetadata(connection);
        assertEquals(connDbId, packageMetadata.getConnDbId());
        assertEquals(technology, packageMetadata.getTechnology());
        assertEquals(phase, packageMetadata.getPhase());
        assertEquals(scenarioName, packageMetadata.getScenarioName());
        assertEquals(connectionId, packageMetadata.getConnectionId());
    }

    @Test
    public void getPackageMetadata_createMetadataForAnUnexistendConnection_null(){
        Connection connection = new Connection(
                null,
                "SOME CONNECTION THAT DOES NOT EXIST",
                executionId,
                null
        );

        PackageMetadata packageMetadata = packageCreateDAO.getPackageMetadata(connection);
        assertNull(packageMetadata);
    }

    @Test
    public void createErrorDefinitionsDumpForConnection_createFileThenReadItAndCheckIfSame_equal(){
        String path = "./target/test-data/errorsConnection.csv";

        packageCreateDAO.createErrorDefinitionsDumpForConnection(connDbId, path);
        List<ErrorDefinition> errorDefinitions = packageUploadDAO.readErrorDefinitions(path);

        assertEquals(1, errorDefinitions.size());

        ErrorDefinition errorDefinition = errorDefinitions.get(0);

        assertEquals(errorCategory, errorDefinition.getErrorCategory());
        assertEquals(errorType, errorDefinition.getErrorType());
        assertEquals(userMessage, errorDefinition.getUserMessage());
        assertEquals(technicalMessage, errorDefinition.getTechnicalMessage());
        assertEquals(solution, errorDefinition.getSolution());
        assertEquals(impact, errorDefinition.getImpact());
        assertEquals(logLevel, errorDefinition.getLogLevel());
    }

    @Test
    public void createErrorDefinitionsDumpForApplication_createFileThenReadItAndCheckIfSame_equal(){
        String path = "./target/test-data/errorsApplication.csv";

        packageCreateDAO.createErrorDefinitionsDumpForApplication(application, path);
        List<ErrorDefinition> errorDefinitions = packageUploadDAO.readErrorDefinitions(path);

        assertEquals(1, errorDefinitions.size());

        ErrorDefinition errorDefinition = errorDefinitions.get(0);

        assertEquals(errorCategory, errorDefinition.getErrorCategory());
        assertEquals(errorType, errorDefinition.getErrorType());
        assertEquals(userMessage, errorDefinition.getUserMessage());
        assertEquals(technicalMessage, errorDefinition.getTechnicalMessage());
        assertEquals(solution, errorDefinition.getSolution());
        assertEquals(impact, errorDefinition.getImpact());
        assertEquals(logLevel, errorDefinition.getLogLevel());
    }

    @Test
    public void createLogDumpForConnection_createFileThenReadItAndCheckIfSame_equal(){
        String path = "./target/test-data/logsConnection.csv";

        ErrorLog errorLog = getErrorLog(workflowId, executionId, connectionId);
        Long inserted = logDAO.save(errorLog, errorId, connDbId);
        assertEquals(ONE, inserted);

        packageCreateDAO.createLogDumpForConnection(connDbId, path);
        List<UploadedLog> logs = packageUploadDAO.readLogs(path);

        assertEquals(1, logs.size());

        UploadedLog uploadedLog = logs.get(0);

        assertEquals(errorLog.getType(), uploadedLog.getType());
        assertEquals(errorLog.getMessage(), uploadedLog.getMessage());
        assertEquals(errorLog.getApplication(), uploadedLog.getApplication());
        assertEquals(errorLog.getContext(), uploadedLog.getContext());
        assertEquals(errorLog.getEpochSecond(), uploadedLog.getEpochSecond());
        assertEquals(errorLog.getExceptionName(), uploadedLog.getExceptionName());
        assertEquals(errorLog.getExtendedStacktrace(), uploadedLog.getExtendedStacktrace());
        assertEquals(errorLog.getLoggerName(), uploadedLog.getLoggerName());
        assertEquals(errorLog.getLevel(), uploadedLog.getLogLevel());
        assertEquals(errorLog.getNanoSecond(), uploadedLog.getNanoOfSecond());
        assertEquals(errorLog.getThreadId(), uploadedLog.getThreadId());
        assertEquals(errorLog.getWorkerNumber(), uploadedLog.getWorkerNumber());
    }

    @Test
    public void createLogDumpForApplication_createFileThenReadItAndCheckIfSame_equal(){
        String path = "./target/test-data/logsApplication.csv";

        ErrorLog errorLog = getErrorLog(workflowId, executionId, connectionId);
        Long inserted = logDAO.save(errorLog, errorId, connDbId);
        assertEquals(ONE, inserted);

        packageCreateDAO.createLogDumpForApplication(application, path);
        List<UploadedLog> logs = packageUploadDAO.readLogs(path);

        assertEquals(1, logs.size());

        UploadedLog uploadedLog = logs.get(0);

        assertEquals(errorLog.getType(), uploadedLog.getType());
        assertEquals(errorLog.getMessage(), uploadedLog.getMessage());
        assertEquals(errorLog.getApplication(), uploadedLog.getApplication());
        assertEquals(errorLog.getContext(), uploadedLog.getContext());
        assertEquals(errorLog.getEpochSecond(), uploadedLog.getEpochSecond());
        assertEquals(errorLog.getExceptionName(), uploadedLog.getExceptionName());
        assertEquals(errorLog.getExtendedStacktrace(), uploadedLog.getExtendedStacktrace());
        assertEquals(errorLog.getLoggerName(), uploadedLog.getLoggerName());
        assertEquals(errorLog.getLevel(), uploadedLog.getLogLevel());
        assertEquals(errorLog.getNanoSecond(), uploadedLog.getNanoOfSecond());
        assertEquals(errorLog.getThreadId(), uploadedLog.getThreadId());
        assertEquals(errorLog.getWorkerNumber(), uploadedLog.getWorkerNumber());
    }

}
