package DAOTests;

import java.util.List;
import java.util.Optional;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.transaction.annotation.Transactional;

import TestConfig.DAOTestConfiguration;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.ScenarioExecution;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.ScenarioExecutionDAO;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

@RunWith(SpringRunner.class)
@Transactional
@ContextConfiguration(classes = {DAOTestConfiguration.class})
@ActiveProfiles({"test"})
public class ScenarioExecutionDAOTests {

    @Autowired
    private ScenarioExecutionDAO scenarioExecutionDAO;

    private String workflowId = "WE1";
    private String executionId = "SE1";
    private String technology = "Oracle";
    private String phase = "Analysis";
    private String scenarioName = "Oracle DDL Dataflow Analysis";

    private final Long ONE = 1L;

    @Test
    public void save_addNewScenarioExecution_added(){
        String newExecutionId = "SE2";

        ScenarioExecution scenarioExecution = new ScenarioExecution(
                newExecutionId,
                workflowId,
                technology,
                phase,
                scenarioName
        );

        Long insertedRows = scenarioExecutionDAO.save(scenarioExecution);
        assertEquals(ONE, insertedRows);

        Optional<ScenarioExecution> foundScenarioExecutionOpt = scenarioExecutionDAO.find(newExecutionId);
        assertTrue(foundScenarioExecutionOpt.isPresent());

        ScenarioExecution foundScenarioExecution = foundScenarioExecutionOpt.get();

        assertEquals(scenarioExecution.getScenarioExecutionId(), foundScenarioExecution.getScenarioExecutionId());
        assertEquals(scenarioExecution.getWorkflowExecutionId(), foundScenarioExecution.getWorkflowExecutionId());
        assertEquals(scenarioExecution.getPhase(), foundScenarioExecution.getPhase());
        assertEquals(scenarioExecution.getScenarioName(), foundScenarioExecution.getScenarioName());
        assertEquals(scenarioExecution.getTechnology(), foundScenarioExecution.getTechnology());
    }

    @Test(expected = DataIntegrityViolationException.class)
    public void save_addNewScenarioExecutionWorkflowDoesNotExist_shouldThrowConstraintViolation(){
        String newExecutionId = "SE2";

        ScenarioExecution scenarioExecution = new ScenarioExecution(
                newExecutionId,
                "WE2",
                technology,
                phase,
                scenarioName
        );

        scenarioExecutionDAO.save(scenarioExecution);
    }

    @Test
    public void delete_deleteScenarioExecution_deleted(){
        Long deletedRows = scenarioExecutionDAO.delete(executionId);
        assertEquals(ONE, deletedRows);

        Optional<ScenarioExecution> nullScenarioExecution = scenarioExecutionDAO.find(executionId);
        assertFalse(nullScenarioExecution.isPresent());
    }

    @Test
    public void getAllFor_getAllScenarioExecutionsForWorkflowExecution_oneFound(){
        ScenarioExecution scenarioExecution = new ScenarioExecution(
                executionId,
                workflowId,
                technology,
                phase,
                scenarioName
        );

        List<ScenarioExecution> scenarioExecutionList = scenarioExecutionDAO.getAllFor(workflowId);
        assertEquals(ONE, new Long(scenarioExecutionList.size()));

        ScenarioExecution foundScenarioExecution = scenarioExecutionList.get(0);

        assertEquals(scenarioExecution.getScenarioExecutionId(), foundScenarioExecution.getScenarioExecutionId());
        assertEquals(scenarioExecution.getWorkflowExecutionId(), foundScenarioExecution.getWorkflowExecutionId());
        assertEquals(scenarioExecution.getPhase(), foundScenarioExecution.getPhase());
        assertEquals(scenarioExecution.getScenarioName(), foundScenarioExecution.getScenarioName());
        assertEquals(scenarioExecution.getTechnology(), foundScenarioExecution.getTechnology());
    }

    @Test
    public void find_getExistingExecution_found(){
        ScenarioExecution scenarioExecution = new ScenarioExecution(
                executionId,
                workflowId,
                technology,
                phase,
                scenarioName
        );

        Optional<ScenarioExecution> foundScenarioExecutionOpt = scenarioExecutionDAO.find(executionId);
        assertTrue(foundScenarioExecutionOpt.isPresent());

        ScenarioExecution foundScenarioExecution = foundScenarioExecutionOpt.get();

        assertEquals(scenarioExecution.getScenarioExecutionId(), foundScenarioExecution.getScenarioExecutionId());
        assertEquals(scenarioExecution.getWorkflowExecutionId(), foundScenarioExecution.getWorkflowExecutionId());
        assertEquals(scenarioExecution.getPhase(), foundScenarioExecution.getPhase());
        assertEquals(scenarioExecution.getScenarioName(), foundScenarioExecution.getScenarioName());
        assertEquals(scenarioExecution.getTechnology(), foundScenarioExecution.getTechnology());
    }

}
