package Integration;

import java.io.File;
import java.io.FileInputStream;
import java.util.Properties;

import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.junit.MockitoJUnit;
import org.mockito.junit.MockitoRule;
import org.mockito.quality.Strictness;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;
import org.springframework.test.context.web.WebAppConfiguration;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.context.WebApplicationContext;

import com.fasterxml.jackson.databind.ObjectMapper;

import TestConfig.IntegrationTestsConfig;
import eu.profinit.manta.bachelor.thesis.log.viewer.controllers.dto.SetExpiryCheckResponseDTO;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.ErrorLog;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.WorkflowExecutionDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.LogPersistenceService;

import static org.junit.Assert.assertEquals;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;
import static utils.Utils.getErrorLog;
import static utils.Utils.getNonWorkflowErrorLog;

@RunWith(SpringJUnit4ClassRunner.class)
@ContextConfiguration(classes = {IntegrationTestsConfig.class})
@ActiveProfiles({"test"})
@WebAppConfiguration
@Transactional
public class ChangeSettingsControllerTests {

    @Autowired
    private WebApplicationContext webAppContext;

    @Autowired
    private LogPersistenceService logPersistenceService;

    @Rule
    public MockitoRule rule = MockitoJUnit.rule().strictness(Strictness.STRICT_STUBS);

    @Autowired
    private ObjectMapper objectMapper;

    @Value("${property.path}")
    private File propertiesFile;

    private MockMvc mockMvc;

    private String PRUNE_PROPERTY = "logviewer.pruneAfter";

    private String baseUrl = "/private/log-viewer/v1/";

    @Autowired
    WorkflowExecutionDAO workflowExecutionDAO;

    @Before
    public void setup() throws Exception {
        this.mockMvc = MockMvcBuilders.webAppContextSetup(this.webAppContext).build();
    }

    @Test
    public void setLogExpiry_setNewExpiryValueCheckPropertyChanged_propertyChanged() throws Exception {
        double updatedPruningValue = 1.5;

        MvcResult mvcResult = this.mockMvc
                .perform(post(baseUrl + "/settings/set-log-expiry")
                        .param("daysUntilExpiry", String.valueOf(updatedPruningValue)))
                .andExpect(status().isOk())
                .andReturn();

        Properties props = new Properties();

        FileInputStream fileInputStream = new FileInputStream(propertiesFile);
        props.load(fileInputStream);
        fileInputStream.close();

        String value = (String) props.get("logviewer.pruneAfter");
        double actualValue = Double.parseDouble(value);

        assertEquals(updatedPruningValue, actualValue, 0.0001);


        // change it back
        double initialValue = 2;

        MvcResult mvcResult1 = this.mockMvc
                .perform(post(baseUrl + "/settings/set-log-expiry")
                        .param("daysUntilExpiry", String.valueOf(initialValue)))
                .andExpect(status().isOk())
                .andReturn();

        Properties props2 = new Properties();

        FileInputStream fileInputStream2 = new FileInputStream(propertiesFile);
        props2.load(fileInputStream2);
        fileInputStream2.close();

        String value2 = (String) props2.get("logviewer.pruneAfter");
        double actualValue2 = Double.parseDouble(value2);

        assertEquals(initialValue, actualValue2, 0.0001);
    }

    @Test
    public void setLogExpiryCheck_addVariousLogCheckCorrectAmountReturned_theLogsShouldBeDeleted() throws Exception {
        ErrorLog log =  getErrorLog("WE1", "SE1", "OracleTestDB");
        logPersistenceService.persistErrorLog(log);

        MvcResult mvcResult = this.mockMvc
                .perform(get(baseUrl + "/settings/set-log-expiry-check")
                        .param("daysUntilExpiry", "0.00000000001")) // very small value
                .andExpect(status().isOk())
                .andReturn();

        SetExpiryCheckResponseDTO response = objectMapper.readValue(mvcResult.getResponse().getContentAsString(), SetExpiryCheckResponseDTO.class);

        // test data
        assertEquals(1, response.getWorkflowsPotentiallyPruned().size());
        assertEquals(1, response.getWorkflowLogsPotentiallyPruned());
        assertEquals(0, response.getNonWorkflowLogsPotentiallyPruned());

        logPersistenceService.persistErrorLog(getNonWorkflowErrorLog());

        MvcResult mvcResult2 = this.mockMvc
                .perform(get(baseUrl + "/settings/set-log-expiry-check")
                        .param("daysUntilExpiry", "0.00000000001"))
                .andExpect(status().isOk())
                .andReturn();

        SetExpiryCheckResponseDTO response2 = objectMapper.readValue(mvcResult2.getResponse().getContentAsString(), SetExpiryCheckResponseDTO.class);

        assertEquals(1, response2.getWorkflowsPotentiallyPruned().size());
        assertEquals(1, response2.getWorkflowLogsPotentiallyPruned());
        assertEquals(1, response2.getNonWorkflowLogsPotentiallyPruned()); // change
    }

}
