package Integration;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;
import org.springframework.test.context.web.WebAppConfiguration;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.context.WebApplicationContext;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;

import TestConfig.IntegrationTestsConfig;
import eu.profinit.manta.bachelor.thesis.log.viewer.controllers.dto.ErrorDTO;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.DebugLog;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.ErrorLog;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.Log;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.LogDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.LogPersistenceService;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;
import static utils.Utils.checkDebugLogEquals;
import static utils.Utils.checkErrorLogEquals;
import static utils.Utils.getDebugLog;
import static utils.Utils.getErrorLog;

@RunWith(SpringJUnit4ClassRunner.class)
@ContextConfiguration(classes = {IntegrationTestsConfig.class})
@ActiveProfiles({"test"})
@WebAppConfiguration
@Transactional
public class ContextControllerTests {

    @Autowired
    private WebApplicationContext webAppContext;

    @Autowired
    private LogPersistenceService logPersistenceService;

    @Autowired
    private ObjectMapper objectMapper;

    private MockMvc mockMvc;

    private String baseUrl = "/private/log-viewer/v1/";

    private String workflowId = "WE1";
    private String executionId = "SE1";
    private String connectionId = "OracleTestDB";
    private String application = "Test Application";

    @Before
    public void setup() throws Exception {
        this.mockMvc = MockMvcBuilders.webAppContextSetup(this.webAppContext).build();
    }

    @Test
    public void getContextForLog_insertSomeTestLogsShuffleThemCheckCorrectLogsReturnedInCorrectOrder_11logsReturnedCorrectOrder() throws Exception {
        DebugLog debugLog1 = getDebugLog(workflowId, executionId, connectionId, application, 1, 1);
        DebugLog debugLog2 = getDebugLog(workflowId, executionId, connectionId, application, 2, 1);
        DebugLog debugLog3 = getDebugLog(workflowId, executionId, connectionId, application, 3, 1);
        DebugLog debugLog4 = getDebugLog(workflowId, executionId, connectionId, application, 4, 1);
        DebugLog debugLog5 = getDebugLog(workflowId, executionId, connectionId, application, 5, 1);
        ErrorLog rootLog = getErrorLog(workflowId, executionId, connectionId, application, 5, 2);
        DebugLog debugLog6 = getDebugLog(workflowId, executionId, connectionId, application, 6, 1);
        DebugLog debugLog7 = getDebugLog(workflowId, executionId, connectionId, application, 7, 1);
        DebugLog debugLog8 = getDebugLog(workflowId, executionId, connectionId, application, 8, 1);
        DebugLog debugLog9 = getDebugLog(workflowId, executionId, connectionId, application, 9, 1);
        DebugLog debugLog10 = getDebugLog(workflowId, executionId, connectionId, application, 10, 1);

        List<DebugLog> logs = Arrays.asList(debugLog1, debugLog2, debugLog3, debugLog4, debugLog5, debugLog6,
                debugLog7, debugLog8, debugLog9, debugLog10);

        // so that test is not affected by insertion order
        Collections.shuffle(logs);

        logPersistenceService.persistErrorLog(rootLog);

        logs.forEach(log -> {
            logPersistenceService.persistDebugLog(log);
        });

        Long logId = rootLog.getLogId();

        MvcResult mvcResult = this.mockMvc
                .perform(get(baseUrl + "/logs/{logId}", logId.toString()))
                .andExpect(status().isOk())
                .andReturn();

        List<Map<String, Object>> response = objectMapper.readValue(mvcResult.getResponse().getContentAsString(), new TypeReference<List<Map<String, Object>>>(){});

        assertEquals(11, response.size());

        checkDebugLogEquals(debugLog1, response.get(0));
        checkDebugLogEquals(debugLog2, response.get(1));
        checkDebugLogEquals(debugLog3, response.get(2));
        checkDebugLogEquals(debugLog4, response.get(3));
        checkDebugLogEquals(debugLog5, response.get(4));
        checkErrorLogEquals(rootLog, response.get(5));
        checkDebugLogEquals(debugLog6, response.get(6));
        checkDebugLogEquals(debugLog7, response.get(7));
        checkDebugLogEquals(debugLog8, response.get(8));
        checkDebugLogEquals(debugLog9, response.get(9));
        checkDebugLogEquals(debugLog10, response.get(10));
    }

    @Test
    public void loadEarlier_insertSomeLogsOnlyLogsBeforeRootLogReturned_5logsReturned() throws Exception {
        DebugLog debugLog1 = getDebugLog(workflowId, executionId, connectionId, application, 1, 1);
        DebugLog debugLog2 = getDebugLog(workflowId, executionId, connectionId, application, 2, 1);
        DebugLog debugLog3 = getDebugLog(workflowId, executionId, connectionId, application, 3, 1);
        DebugLog debugLog4 = getDebugLog(workflowId, executionId, connectionId, application, 4, 1);
        DebugLog debugLog5 = getDebugLog(workflowId, executionId, connectionId, application, 5, 1);
        ErrorLog rootLog = getErrorLog(workflowId, executionId, connectionId, application, 5, 2);
        DebugLog debugLog6 = getDebugLog(workflowId, executionId, connectionId, application, 6, 1);
        DebugLog debugLog7 = getDebugLog(workflowId, executionId, connectionId, application, 7, 1);
        DebugLog debugLog8 = getDebugLog(workflowId, executionId, connectionId, application, 8, 1);
        DebugLog debugLog9 = getDebugLog(workflowId, executionId, connectionId, application, 9, 1);
        DebugLog debugLog10 = getDebugLog(workflowId, executionId, connectionId, application, 10, 1);

        List<DebugLog> logs = Arrays.asList(debugLog1, debugLog2, debugLog3, debugLog4, debugLog5, debugLog6,
                debugLog7, debugLog8, debugLog9, debugLog10);

        // so that test is not affected by insertion order
        Collections.shuffle(logs);

        logPersistenceService.persistErrorLog(rootLog);

        logs.forEach(log -> {
            logPersistenceService.persistDebugLog(log);
        });

        Long logId = rootLog.getLogId();

        MvcResult mvcResult = this.mockMvc
                .perform(get(baseUrl + "/logs/{logId}/before", logId.toString())
                .param("offset", "0"))
                .andExpect(status().isOk())
                .andReturn();

        List<Map<String, Object>> response = objectMapper.readValue(mvcResult.getResponse().getContentAsString(), new TypeReference<List<Map<String, Object>>>(){});

        assertEquals(5, response.size());

        checkDebugLogEquals(debugLog1, response.get(0));
        checkDebugLogEquals(debugLog2, response.get(1));
        checkDebugLogEquals(debugLog3, response.get(2));
        checkDebugLogEquals(debugLog4, response.get(3));
        checkDebugLogEquals(debugLog5, response.get(4));
    }

    @Test
    public void loadEarlier_biggerOffsetThanAmountOfLogs_noneReturned() throws Exception {
        DebugLog debugLog1 = getDebugLog(workflowId, executionId, connectionId, application, 1, 1);
        DebugLog debugLog2 = getDebugLog(workflowId, executionId, connectionId, application, 2, 1);
        DebugLog debugLog3 = getDebugLog(workflowId, executionId, connectionId, application, 3, 1);
        DebugLog debugLog4 = getDebugLog(workflowId, executionId, connectionId, application, 4, 1);
        DebugLog debugLog5 = getDebugLog(workflowId, executionId, connectionId, application, 5, 1);
        ErrorLog rootLog = getErrorLog(workflowId, executionId, connectionId, application, 5, 2);
        DebugLog debugLog6 = getDebugLog(workflowId, executionId, connectionId, application, 6, 1);
        DebugLog debugLog7 = getDebugLog(workflowId, executionId, connectionId, application, 7, 1);
        DebugLog debugLog8 = getDebugLog(workflowId, executionId, connectionId, application, 8, 1);
        DebugLog debugLog9 = getDebugLog(workflowId, executionId, connectionId, application, 9, 1);
        DebugLog debugLog10 = getDebugLog(workflowId, executionId, connectionId, application, 10, 1);

        List<DebugLog> logs = Arrays.asList(debugLog1, debugLog2, debugLog3, debugLog4, debugLog5, debugLog6,
                debugLog7, debugLog8, debugLog9, debugLog10);

        // so that test is not affected by insertion order
        Collections.shuffle(logs);

        logPersistenceService.persistErrorLog(rootLog);

        logs.forEach(log -> {
            logPersistenceService.persistDebugLog(log);
        });

        Long logId = rootLog.getLogId();

        MvcResult mvcResult = this.mockMvc
                .perform(get(baseUrl + "/logs/{logId}/before", logId.toString())
                .param("offset", "6")) // bigger offset
                .andExpect(status().isOk())
                .andReturn();

        List<Map<String, Object>> response = objectMapper.readValue(mvcResult.getResponse().getContentAsString(), new TypeReference<List<Map<String, Object>>>(){});

        assertTrue(response.isEmpty());
    }

    @Test
    public void loadEarlier_offsetZeroTestCorrectAmountReturned_oneLogReturned() throws Exception {
        DebugLog debugLog1 = getDebugLog(workflowId, executionId, connectionId, application, 1, 1);
        DebugLog debugLog2 = getDebugLog(workflowId, executionId, connectionId, application, 2, 1);
        DebugLog debugLog3 = getDebugLog(workflowId, executionId, connectionId, application, 3, 1);
        DebugLog debugLog4 = getDebugLog(workflowId, executionId, connectionId, application, 4, 1);
        DebugLog debugLog5 = getDebugLog(workflowId, executionId, connectionId, application, 5, 1);
        ErrorLog rootLog = getErrorLog(workflowId, executionId, connectionId, application, 5, 2);
        DebugLog debugLog6 = getDebugLog(workflowId, executionId, connectionId, application, 6, 1);
        DebugLog debugLog7 = getDebugLog(workflowId, executionId, connectionId, application, 7, 1);
        DebugLog debugLog8 = getDebugLog(workflowId, executionId, connectionId, application, 8, 1);
        DebugLog debugLog9 = getDebugLog(workflowId, executionId, connectionId, application, 9, 1);
        DebugLog debugLog10 = getDebugLog(workflowId, executionId, connectionId, application, 10, 1);

        List<DebugLog> logs = Arrays.asList(debugLog1, debugLog2, debugLog3, debugLog4, debugLog5, debugLog6,
                debugLog7, debugLog8, debugLog9, debugLog10);

        // so that test is not affected by insertion order
        Collections.shuffle(logs);

        logPersistenceService.persistErrorLog(rootLog);

        logs.forEach(log -> {
            logPersistenceService.persistDebugLog(log);
        });

        Long logId = rootLog.getLogId();

        MvcResult mvcResult = this.mockMvc
                .perform(get(baseUrl + "/logs/{logId}/before", logId.toString())
                        .param("offset", "0")
                        .param("amount", "1"))
                .andExpect(status().isOk())
                .andReturn();

        List<Map<String, Object>> response = objectMapper.readValue(mvcResult.getResponse().getContentAsString(), new TypeReference<List<Map<String, Object>>>(){});
        assertEquals(1, response.size());
        checkDebugLogEquals(debugLog5, response.get(0));
    }

    @Test
    public void loadLater_offsetZeroTestCorrectAmountReturned_fiveReturned() throws Exception {
        DebugLog debugLog1 = getDebugLog(workflowId, executionId, connectionId, application, 1, 1);
        DebugLog debugLog2 = getDebugLog(workflowId, executionId, connectionId, application, 2, 1);
        DebugLog debugLog3 = getDebugLog(workflowId, executionId, connectionId, application, 3, 1);
        DebugLog debugLog4 = getDebugLog(workflowId, executionId, connectionId, application, 4, 1);
        DebugLog debugLog5 = getDebugLog(workflowId, executionId, connectionId, application, 5, 1);
        ErrorLog rootLog = getErrorLog(workflowId, executionId, connectionId, application, 5, 2);
        DebugLog debugLog6 = getDebugLog(workflowId, executionId, connectionId, application, 6, 1);
        DebugLog debugLog7 = getDebugLog(workflowId, executionId, connectionId, application, 7, 1);
        DebugLog debugLog8 = getDebugLog(workflowId, executionId, connectionId, application, 8, 1);
        DebugLog debugLog9 = getDebugLog(workflowId, executionId, connectionId, application, 9, 1);
        DebugLog debugLog10 = getDebugLog(workflowId, executionId, connectionId, application, 10, 1);

        List<DebugLog> logs = Arrays.asList(debugLog1, debugLog2, debugLog3, debugLog4, debugLog5, debugLog6,
                debugLog7, debugLog8, debugLog9, debugLog10);

        // so that test is not affected by insertion order
        Collections.shuffle(logs);

        logPersistenceService.persistErrorLog(rootLog);

        logs.forEach(log -> {
            logPersistenceService.persistDebugLog(log);
        });

        Long logId = rootLog.getLogId();

        MvcResult mvcResult = this.mockMvc
                .perform(get(baseUrl + "/logs/{logId}/after", logId.toString())
                        .param("offset", "0"))
                .andExpect(status().isOk())
                .andReturn();

        List<Map<String, Object>> response = objectMapper.readValue(mvcResult.getResponse().getContentAsString(), new TypeReference<List<Map<String, Object>>>(){});
        assertEquals(5, response.size());
        checkDebugLogEquals(debugLog6, response.get(0));
        checkDebugLogEquals(debugLog7, response.get(1));
        checkDebugLogEquals(debugLog8, response.get(2));
        checkDebugLogEquals(debugLog9, response.get(3));
        checkDebugLogEquals(debugLog10, response.get(4));
    }

    @Test
    public void loadLater_biggerOffsetThanAmountOfLogs_noneReturned() throws Exception {
        DebugLog debugLog1 = getDebugLog(workflowId, executionId, connectionId, application, 1, 1);
        DebugLog debugLog2 = getDebugLog(workflowId, executionId, connectionId, application, 2, 1);
        DebugLog debugLog3 = getDebugLog(workflowId, executionId, connectionId, application, 3, 1);
        DebugLog debugLog4 = getDebugLog(workflowId, executionId, connectionId, application, 4, 1);
        DebugLog debugLog5 = getDebugLog(workflowId, executionId, connectionId, application, 5, 1);
        ErrorLog rootLog = getErrorLog(workflowId, executionId, connectionId, application, 5, 2);
        DebugLog debugLog6 = getDebugLog(workflowId, executionId, connectionId, application, 6, 1);
        DebugLog debugLog7 = getDebugLog(workflowId, executionId, connectionId, application, 7, 1);
        DebugLog debugLog8 = getDebugLog(workflowId, executionId, connectionId, application, 8, 1);
        DebugLog debugLog9 = getDebugLog(workflowId, executionId, connectionId, application, 9, 1);
        DebugLog debugLog10 = getDebugLog(workflowId, executionId, connectionId, application, 10, 1);

        List<DebugLog> logs = Arrays.asList(debugLog1, debugLog2, debugLog3, debugLog4, debugLog5, debugLog6,
                debugLog7, debugLog8, debugLog9, debugLog10);

        // so that test is not affected by insertion order
        Collections.shuffle(logs);

        logPersistenceService.persistErrorLog(rootLog);

        logs.forEach(log -> {
            logPersistenceService.persistDebugLog(log);
        });

        Long logId = rootLog.getLogId();

        MvcResult mvcResult = this.mockMvc
                .perform(get(baseUrl + "/logs/{logId}/after", logId.toString())
                        .param("offset", "6"))
                .andExpect(status().isOk())
                .andReturn();

        List<Map<String, Object>> response = objectMapper.readValue(mvcResult.getResponse().getContentAsString(), new TypeReference<List<Map<String, Object>>>(){});
        assertTrue(response.isEmpty());
    }

    @Test
    public void loadLater_offsetZeroWantOnlyOneLog_oneLogReturned() throws Exception {
        DebugLog debugLog1 = getDebugLog(workflowId, executionId, connectionId, application, 1, 1);
        DebugLog debugLog2 = getDebugLog(workflowId, executionId, connectionId, application, 2, 1);
        DebugLog debugLog3 = getDebugLog(workflowId, executionId, connectionId, application, 3, 1);
        DebugLog debugLog4 = getDebugLog(workflowId, executionId, connectionId, application, 4, 1);
        DebugLog debugLog5 = getDebugLog(workflowId, executionId, connectionId, application, 5, 1);
        ErrorLog rootLog = getErrorLog(workflowId, executionId, connectionId, application, 5, 2);
        DebugLog debugLog6 = getDebugLog(workflowId, executionId, connectionId, application, 6, 1);
        DebugLog debugLog7 = getDebugLog(workflowId, executionId, connectionId, application, 7, 1);
        DebugLog debugLog8 = getDebugLog(workflowId, executionId, connectionId, application, 8, 1);
        DebugLog debugLog9 = getDebugLog(workflowId, executionId, connectionId, application, 9, 1);
        DebugLog debugLog10 = getDebugLog(workflowId, executionId, connectionId, application, 10, 1);

        List<DebugLog> logs = Arrays.asList(debugLog1, debugLog2, debugLog3, debugLog4, debugLog5, debugLog6,
                debugLog7, debugLog8, debugLog9, debugLog10);

        // so that test is not affected by insertion order
        Collections.shuffle(logs);

        logPersistenceService.persistErrorLog(rootLog);

        logs.forEach(log -> {
            logPersistenceService.persistDebugLog(log);
        });

        Long logId = rootLog.getLogId();

        MvcResult mvcResult = this.mockMvc
                .perform(get(baseUrl + "/logs/{logId}/after", logId.toString())
                        .param("offset", "0")
                        .param("amount", "1"))
                .andExpect(status().isOk())
                .andReturn();

        List<Map<String, Object>> response = objectMapper.readValue(mvcResult.getResponse().getContentAsString(), new TypeReference<List<Map<String, Object>>>(){});
        assertEquals(1, response.size());

        checkDebugLogEquals(debugLog6, response.get(0));
    }

    @Test
    public void loadLater_passInvalidParams_get400response() throws Exception {
        this.mockMvc
                .perform(get(baseUrl + "/logs/{logId}/after", "0") // doesn't matter, should throw error
                        .param("offset", "-1")
                        .param("amount", "-1"))
                .andExpect(status().is(400));
    }

    @Test
    public void loadEarlier_passInvalidParams_get400response() throws Exception {
        this.mockMvc
                .perform(get(baseUrl + "/logs/{logId}/after", "0") // doesn't matter, should throw error
                        .param("offset", "-1")
                        .param("amount", "-1"))
                .andExpect(status().is(400));
    }

}
