package Integration;

import java.util.List;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;
import org.springframework.test.context.web.WebAppConfiguration;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.context.WebApplicationContext;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;

import TestConfig.IntegrationTestsConfig;
import eu.profinit.manta.bachelor.thesis.log.viewer.controllers.dto.ErrorDTO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.LogPersistenceService;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@RunWith(SpringJUnit4ClassRunner.class)
@ContextConfiguration(classes = {IntegrationTestsConfig.class})
@ActiveProfiles({"test"})
@WebAppConfiguration
@Transactional
public class FullTextControllerTests {

    @Autowired
    private WebApplicationContext webAppContext;

    @Autowired
    private ObjectMapper objectMapper;

    private MockMvc mockMvc;

    private String baseUrl = "/private/log-viewer/v1/";

    @Before
    public void setup() throws Exception {
        this.mockMvc = MockMvcBuilders.webAppContextSetup(this.webAppContext).build();
    }

    @Test
    public void fulltext_searchBasePath_oneErrorReturned() throws Exception {
        MvcResult mvcResult = this.mockMvc
                .perform(get(baseUrl + "/search")
                        .param("query", "technical") // from test data
                        .param("offset", "0"))
                .andExpect(status().isOk())
                .andReturn();

        List<ErrorDTO> response = objectMapper.readValue(mvcResult.getResponse().getContentAsString(), new TypeReference<List<ErrorDTO>>(){});

        assertEquals(1, response.size());

        ErrorDTO errorDTO = response.get(0);

        assertEquals("TEST_CATEGORY", errorDTO.getErrorCategory());
        assertEquals("TEST_ERROR", errorDTO.getErrorType());
        assertEquals("Test user message.", errorDTO.getUserMessage());
        assertEquals("Test technical message.", errorDTO.getTechnicalMessage());
        assertEquals("Test solution.", errorDTO.getSolution());
        assertEquals("Test impact.", errorDTO.getImpact());
        assertEquals("ERROR", errorDTO.getLogLevel());
        assertEquals("Test Application", errorDTO.getApplication());
    }

    @Test
    public void fullText_passEmptyString_get400() throws Exception {
        this.mockMvc
                .perform(get(baseUrl + "/search")
                        .param("query", "") // from test data
                        .param("offset", "0"))
                .andExpect(status().is(400));
    }

    @Test
    public void fullText_basePathPassRegexForExistingTechnicalMessage_getOneError() throws Exception {
        MvcResult mvcResult = this.mockMvc
                .perform(get(baseUrl + "/search")
                        .param("query", "tech*") // from test data
                        .param("offset", "0"))
                .andExpect(status().isOk())
                .andReturn();

        List<ErrorDTO> response = objectMapper.readValue(mvcResult.getResponse().getContentAsString(), new TypeReference<List<ErrorDTO>>(){});

        assertEquals(1, response.size());

        ErrorDTO errorDTO = response.get(0);

        assertEquals("TEST_CATEGORY", errorDTO.getErrorCategory());
        assertEquals("TEST_ERROR", errorDTO.getErrorType());
        assertEquals("Test user message.", errorDTO.getUserMessage());
        assertEquals("Test technical message.", errorDTO.getTechnicalMessage());
        assertEquals("Test solution.", errorDTO.getSolution());
        assertEquals("Test impact.", errorDTO.getImpact());
        assertEquals("ERROR", errorDTO.getLogLevel());
        assertEquals("Test Application", errorDTO.getApplication());
    }

    @Test
    public void fullText_passOffsetBiggerThanAmountOfLogs_noneReturned() throws Exception {
        MvcResult mvcResult = this.mockMvc
                .perform(get(baseUrl + "/search")
                        .param("query", "tech*") // from test data
                        .param("offset", "5"))
                .andExpect(status().isOk())
                .andReturn();

        List<ErrorDTO> response = objectMapper.readValue(mvcResult.getResponse().getContentAsString(), new TypeReference<List<ErrorDTO>>(){});

        assertTrue(response.isEmpty());
    }

    @Test
    public void fullText_workflowExecutionPath_oneLogReturned() throws Exception {
        MvcResult mvcResult = this.mockMvc
                .perform(get(baseUrl + "workflow-executions/{workflow-execution}/search", "WE1") // test data
                        .param("query", "tech*") // from test data
                        .param("offset", "0"))
                .andExpect(status().isOk())
                .andReturn();

        List<ErrorDTO> response = objectMapper.readValue(mvcResult.getResponse().getContentAsString(), new TypeReference<List<ErrorDTO>>(){});

        assertEquals(1, response.size());

        ErrorDTO errorDTO = response.get(0);

        assertEquals("TEST_CATEGORY", errorDTO.getErrorCategory());
        assertEquals("TEST_ERROR", errorDTO.getErrorType());
        assertEquals("Test user message.", errorDTO.getUserMessage());
        assertEquals("Test technical message.", errorDTO.getTechnicalMessage());
        assertEquals("Test solution.", errorDTO.getSolution());
        assertEquals("Test impact.", errorDTO.getImpact());
        assertEquals("ERROR", errorDTO.getLogLevel());
        assertEquals("Test Application", errorDTO.getApplication());
    }

    @Test
    public void fullText_scenarioExecutionPath_oneLogReturned() throws Exception {
        MvcResult mvcResult = this.mockMvc
                .perform(get(baseUrl + "workflow-executions/{workflow-execution}/scenario-executions/{scenario-execution}/search", "WE1", "SE1") // test data
                        .param("query", "tech*") // from test data
                        .param("offset", "0"))
                .andExpect(status().isOk())
                .andReturn();

        List<ErrorDTO> response = objectMapper.readValue(mvcResult.getResponse().getContentAsString(), new TypeReference<List<ErrorDTO>>(){});

        assertEquals(1, response.size());

        ErrorDTO errorDTO = response.get(0);

        assertEquals("TEST_CATEGORY", errorDTO.getErrorCategory());
        assertEquals("TEST_ERROR", errorDTO.getErrorType());
        assertEquals("Test user message.", errorDTO.getUserMessage());
        assertEquals("Test technical message.", errorDTO.getTechnicalMessage());
        assertEquals("Test solution.", errorDTO.getSolution());
        assertEquals("Test impact.", errorDTO.getImpact());
        assertEquals("ERROR", errorDTO.getLogLevel());
        assertEquals("Test Application", errorDTO.getApplication());
    }

    @Test
    public void fullText_connectionIdPath_oneLogReturned() throws Exception {
        MvcResult mvcResult = this.mockMvc
                .perform(get(baseUrl + "workflow-executions/{workflow-execution}/scenario-executions/{scenario-execution}/connections/{connectionId}/search", "WE1", "SE1", "OracleTestDB") // test data
                        .param("query", "tech*") // from test data
                        .param("offset", "0"))
                .andExpect(status().isOk())
                .andReturn();

        List<ErrorDTO> response = objectMapper.readValue(mvcResult.getResponse().getContentAsString(), new TypeReference<List<ErrorDTO>>(){});

        assertEquals(1, response.size());

        ErrorDTO errorDTO = response.get(0);

        assertEquals("TEST_CATEGORY", errorDTO.getErrorCategory());
        assertEquals("TEST_ERROR", errorDTO.getErrorType());
        assertEquals("Test user message.", errorDTO.getUserMessage());
        assertEquals("Test technical message.", errorDTO.getTechnicalMessage());
        assertEquals("Test solution.", errorDTO.getSolution());
        assertEquals("Test impact.", errorDTO.getImpact());
        assertEquals("ERROR", errorDTO.getLogLevel());
        assertEquals("Test Application", errorDTO.getApplication());
    }

    @Test
    public void fullText_packagePath_oneLogReturned() throws Exception {
        MvcResult mvcResult = this.mockMvc
                .perform(get(baseUrl + "/packages/{packageName}/search", "My Package 1") // test data
                        .param("query", "tech*") // from test data
                        .param("offset", "0"))
                .andExpect(status().isOk())
                .andReturn();

        List<ErrorDTO> response = objectMapper.readValue(mvcResult.getResponse().getContentAsString(), new TypeReference<List<ErrorDTO>>(){});

        assertEquals(1, response.size());

        ErrorDTO errorDTO = response.get(0);

        assertEquals("TEST_CATEGORY", errorDTO.getErrorCategory());
        assertEquals("TEST_ERROR", errorDTO.getErrorType());
        assertEquals("Test user message.", errorDTO.getUserMessage());
        assertEquals("Test technical message.", errorDTO.getTechnicalMessage());
        assertEquals("Test solution.", errorDTO.getSolution());
        assertEquals("Test impact.", errorDTO.getImpact());
        assertEquals("ERROR", errorDTO.getLogLevel());
        assertEquals("Test Application", errorDTO.getApplication());
    }

}
