package Integration;

import java.util.List;
import java.util.Map;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;
import org.springframework.test.context.web.WebAppConfiguration;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.context.WebApplicationContext;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;

import TestConfig.IntegrationTestsConfig;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.ErrorMetadata;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.ErrorLog;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.UploadedLog;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.ErrorDefinitionDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.ErrorMetadataDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.PackageUploadDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.LogPersistenceService;

import static org.jgroups.util.Util.assertEquals;
import static org.jgroups.util.Util.assertTrue;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;
import static utils.Utils.checkErrorLogEquals;
import static utils.Utils.getErrorLog;

@RunWith(SpringJUnit4ClassRunner.class)
@ContextConfiguration(classes = {IntegrationTestsConfig.class})
@ActiveProfiles({"test"})
@WebAppConfiguration
@Transactional
public class LogExpandControllerTests {

    @Autowired
    private WebApplicationContext webAppContext;

    @Autowired
    private LogPersistenceService logPersistenceService;

    @Autowired
    private ObjectMapper objectMapper;

    @Autowired
    private PackageUploadDAO packageUploadDAO;

    @Autowired
    private ErrorMetadataDAO errorMetadataDAO;

    @Autowired
    private ErrorDefinitionDAO errorDefinitionDAO;

    private MockMvc mockMvc;

    private String baseUrl = "/private/log-viewer/v1/";

    private String workflowId = "WE1";
    private String executionId = "SE1";
    private String connectionId = "OracleTestDB";
    private String application = "Test Application";
    private String errorCategory = "TEST_CATEGORY";
    private String errorType = "TEST_ERROR";


    private Long errorId;

    @Before
    public void setup() throws Exception {
        this.mockMvc = MockMvcBuilders.webAppContextSetup(this.webAppContext).build();
        Long errorDefinitionId = errorDefinitionDAO.getAll().get(0);
        errorId = errorMetadataDAO.getIdByData(new ErrorMetadata(errorDefinitionId, "My Package 1", "Test Application")); // test data.sql
    }

    @Test
    public void expandLogs_basePathPassTestErrorGetCorrectAmountOfLogs_5errorLogsReturned() throws Exception {
        // insert 5 logs, the static method uses metadata from test data
        for (int i = 0; i < 5; ++i){
            logPersistenceService.persistErrorLog(getErrorLog(workflowId, executionId, connectionId, application));
        }

        MvcResult mvcResult = this.mockMvc
                .perform(get(baseUrl + "/expand")
                            .param("offset", "0")
                            .param("errorCategory", errorCategory)
                            .param("collapsedElement", errorType)
                            .param("application", application)) // test data
                .andExpect(status().isOk())
                .andReturn();

        List<Map<String, Object>> response = objectMapper.readValue(mvcResult.getResponse().getContentAsString(), new TypeReference<List<Map<String, Object>>>(){});

        assertEquals(5, response.size());

        response.forEach(log -> {
            checkErrorLogEquals(getErrorLog(workflowId, executionId, connectionId, application), log);
        });
    }

    @Test
    public void expandLogs_passOffsetBiggerThanAmountOfLogs_noneReturned() throws Exception {
        // insert 5 logs, the static method uses metadata from test data
        for (int i = 0; i < 5; ++i){
            logPersistenceService.persistErrorLog(getErrorLog(workflowId, executionId, connectionId, application));
        }

        MvcResult mvcResult = this.mockMvc
                .perform(get(baseUrl + "/expand")
                        .param("offset", "6")
                        .param("errorCategory", errorCategory)
                        .param("collapsedElement", errorType)
                        .param("application", application)) // test data
                .andExpect(status().isOk())
                .andReturn();

        List<Map<String, Object>> response = objectMapper.readValue(mvcResult.getResponse().getContentAsString(), new TypeReference<List<Map<String, Object>>>(){});

        assertTrue(response.isEmpty());
    }

    @Test
    public void expandLogs_passADifferentError_noneReturned() throws Exception {
        // insert 5 logs, the static method uses metadata from test data
        for (int i = 0; i < 5; ++i){
            logPersistenceService.persistErrorLog(getErrorLog(workflowId, executionId, connectionId, application));
        }

        MvcResult mvcResult = this.mockMvc
                .perform(get(baseUrl + "/expand")
                        .param("offset", "0")
                        .param("errorCategory", errorCategory)
                        .param("collapsedElement", "SOME ERROR THAT DOES NOT EXIST")
                        .param("application", application)) // test data
                .andExpect(status().isOk())
                .andReturn();

        List<Map<String, Object>> response = objectMapper.readValue(mvcResult.getResponse().getContentAsString(), new TypeReference<List<Map<String, Object>>>(){});

        assertTrue(response.isEmpty());
    }

    @Test
    public void expandLogs_passNegativeOffset_400fromValidation() throws Exception {
        // insert 5 logs, the static method uses metadata from test data
        for (int i = 0; i < 5; ++i){
            logPersistenceService.persistErrorLog(getErrorLog(workflowId, executionId, connectionId, application));
        }

        MvcResult mvcResult = this.mockMvc
                .perform(get(baseUrl + "/expand")
                        .param("offset", "-1")
                        .param("errorCategory", errorCategory)
                        .param("collapsedElement", errorType)
                        .param("application", application)) // test data
                .andExpect(status().is(400))
                .andReturn();
    }

    @Test
    public void expandLogs_workflowExecutionPath_5errorLogsReturned() throws Exception {
        // insert 5 logs, the static method uses metadata from test data
        for (int i = 0; i < 5; ++i){
            logPersistenceService.persistErrorLog(getErrorLog(workflowId, executionId, connectionId, application));
        }

        logPersistenceService.persistErrorLog(getErrorLog("SOME DIFFERENT WORKFLOW EXECUTION", "SE5", connectionId, application));

        MvcResult mvcResult = this.mockMvc
                .perform(get(baseUrl + "/workflow-executions/{workflowExecutionId}/expand", workflowId)
                        .param("offset", "0")
                        .param("errorCategory", errorCategory)
                        .param("collapsedElement", errorType)
                        .param("application", application)) // test data
                .andExpect(status().isOk())
                .andReturn();

        List<Map<String, Object>> response = objectMapper.readValue(mvcResult.getResponse().getContentAsString(), new TypeReference<List<Map<String, Object>>>(){});

        assertEquals(5, response.size());

        response.forEach(log -> {
            checkErrorLogEquals(getErrorLog(workflowId, executionId, connectionId, application), log);
        });
    }

    @Test
    public void expandLogs_scenarioExecutionPath_5errorLogsReturned() throws Exception {
        // insert 5 logs, the static method uses metadata from test data
        for (int i = 0; i < 5; ++i){
            logPersistenceService.persistErrorLog(getErrorLog(workflowId, executionId, connectionId, application));
        }

        logPersistenceService.persistErrorLog(getErrorLog(workflowId, "SOME DIFFERENT SC. EXECUTION", connectionId, application));

        MvcResult mvcResult = this.mockMvc
                .perform(get(baseUrl + "/workflow-executions/{workflowExecutionId}/scenario-executions/{scenarioExecutionId}/expand", workflowId, executionId)
                        .param("offset", "0")
                        .param("errorCategory", errorCategory)
                        .param("collapsedElement", errorType)
                        .param("application", application)) // test data
                .andExpect(status().isOk())
                .andReturn();

        List<Map<String, Object>> response = objectMapper.readValue(mvcResult.getResponse().getContentAsString(), new TypeReference<List<Map<String, Object>>>(){});

        assertEquals(5, response.size());

        response.forEach(log -> {
            checkErrorLogEquals(getErrorLog(workflowId, executionId, connectionId, application), log);
        });
    }

    @Test
    public void expandLogs_connectionPath_5errorLogsReturned() throws Exception {
        // insert 5 logs, the static method uses metadata from test data
        for (int i = 0; i < 5; ++i){
            logPersistenceService.persistErrorLog(getErrorLog(workflowId, executionId, connectionId, application));
        }

        logPersistenceService.persistErrorLog(getErrorLog(workflowId, executionId, "DIFFERENT CONNECTION", application));

        MvcResult mvcResult = this.mockMvc
                .perform(get(baseUrl + "/workflow-executions/{workflowExecutionId}/scenario-executions/{scenarioExecutionId}/connections/{connectionId}/expand",
                        workflowId, executionId, connectionId)
                        .param("offset", "0")
                        .param("errorCategory", errorCategory)
                        .param("collapsedElement", errorType)
                        .param("application", application)) // test data
                .andExpect(status().isOk())
                .andReturn();

        List<Map<String, Object>> response = objectMapper.readValue(mvcResult.getResponse().getContentAsString(), new TypeReference<List<Map<String, Object>>>(){});

        assertEquals(5, response.size());

        response.forEach(log -> {
            checkErrorLogEquals(getErrorLog(workflowId, executionId, connectionId, application), log);
        });
    }
}
