package Integration;

import java.util.List;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;
import org.springframework.test.context.web.WebAppConfiguration;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.context.WebApplicationContext;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;

import static org.jgroups.util.Util.assertEquals;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;
import static utils.Utils.getErrorLog;
import static utils.Utils.getPackageErrorLog;

import TestConfig.IntegrationTestsConfig;
import eu.profinit.manta.bachelor.thesis.log.viewer.controllers.dto.ErrorDTO;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.ErrorLog;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.LogPersistenceService;

@RunWith(SpringJUnit4ClassRunner.class)
@ContextConfiguration(classes = {IntegrationTestsConfig.class})
@ActiveProfiles({"test"})
@WebAppConfiguration
@Transactional
public class LogQueryControllerTests {

    @Autowired
    private WebApplicationContext webAppContext;

    @Autowired
    private LogPersistenceService logPersistenceService;

    @Autowired
    private ObjectMapper objectMapper;

    private MockMvc mockMvc;

    private String baseUrl = "/private/log-viewer/v1/";

    @Before
    public void setup() throws Exception {
        this.mockMvc = MockMvcBuilders.webAppContextSetup(this.webAppContext).build();
    }

    @Test
    public void queryLog_addSendRequestForLogCheckErrorDataMatches_oneLogInResponse() throws Exception {
        ErrorLog log =  getErrorLog("WE1", "SE1", "OracleTestDB");
        logPersistenceService.persistErrorLog(log);

        MvcResult mvcResult = this.mockMvc
                .perform(get(baseUrl)
                .param("offset", "0"))
                .andExpect(status().isOk())
                .andReturn();

        List<ErrorDTO> response = objectMapper.readValue(mvcResult.getResponse().getContentAsString(), new TypeReference<List<ErrorDTO>>(){});

        assertEquals(1, response.size());

        ErrorDTO errorDTO = response.get(0);

        assertEquals(log.getCategory(), errorDTO.getErrorCategory());
        assertEquals(log.getErrorType(), errorDTO.getErrorType());
        assertEquals(log.getUserMessage(), errorDTO.getUserMessage());
        assertEquals(log.getTechnicalMessage(), errorDTO.getTechnicalMessage());
        assertEquals(log.getSolution(), errorDTO.getSolution());
        assertEquals(log.getLineageImpact(), errorDTO.getImpact());
        assertEquals(log.getLevel(), errorDTO.getLogLevel());
        assertEquals(log.getApplication(), errorDTO.getApplication());
    }

    @Test
    public void queryLog_addSendRequestForLogFromWECheckErrorDataMatches_oneLogInResponse() throws Exception {
        ErrorLog log = getErrorLog("WE1", "SE1", "OracleTestDB");
        ErrorLog log2 = getErrorLog("WE2", "SE2", "OracleTestDB");
        logPersistenceService.persistErrorLog(log);
        logPersistenceService.persistErrorLog(log2);

        MvcResult mvcResult = this.mockMvc
                .perform(get(baseUrl + "workflow-executions/{workflow-execution}", "WE1")
                        .param("offset", "0"))
                .andExpect(status().isOk())
                .andReturn();

        List<ErrorDTO> response = objectMapper.readValue(mvcResult.getResponse().getContentAsString(), new TypeReference<List<ErrorDTO>>(){});

        assertEquals(1, response.size());

        ErrorDTO errorDTO = response.get(0);

        assertEquals(log.getCategory(), errorDTO.getErrorCategory());
        assertEquals(log.getErrorType(), errorDTO.getErrorType());
        assertEquals(log.getUserMessage(), errorDTO.getUserMessage());
        assertEquals(log.getTechnicalMessage(), errorDTO.getTechnicalMessage());
        assertEquals(log.getSolution(), errorDTO.getSolution());
        assertEquals(log.getLineageImpact(), errorDTO.getImpact());
        assertEquals(log.getLevel(), errorDTO.getLogLevel());
        assertEquals(log.getApplication(), errorDTO.getApplication());
    }

    @Test
    public void queryLog_addSendRequestForLogFromSECheckErrorDataMatches_oneLogInResponse() throws Exception {
        ErrorLog log = getErrorLog("WE1", "SE1", "OracleTestDB");
        ErrorLog log2 = getErrorLog("WE1", "SE2", "OracleTestDB");
        logPersistenceService.persistErrorLog(log);
        logPersistenceService.persistErrorLog(log2);

        MvcResult mvcResult = this.mockMvc
                .perform(get(baseUrl + "workflow-executions/{workflow-execution}/scenario-executions/{scenario-execution}", "WE1", "SE1")
                        .param("offset", "0"))
                .andExpect(status().isOk())
                .andReturn();

        List<ErrorDTO> response = objectMapper.readValue(mvcResult.getResponse().getContentAsString(), new TypeReference<List<ErrorDTO>>(){});

        assertEquals(1, response.size());

        ErrorDTO errorDTO = response.get(0);

        assertEquals(log.getCategory(), errorDTO.getErrorCategory());
        assertEquals(log.getErrorType(), errorDTO.getErrorType());
        assertEquals(log.getUserMessage(), errorDTO.getUserMessage());
        assertEquals(log.getTechnicalMessage(), errorDTO.getTechnicalMessage());
        assertEquals(log.getSolution(), errorDTO.getSolution());
        assertEquals(log.getLineageImpact(), errorDTO.getImpact());
        assertEquals(log.getLevel(), errorDTO.getLogLevel());
        assertEquals(log.getApplication(), errorDTO.getApplication());
    }

    @Test
    public void queryLog_addSendRequestForLogFromConnectionCheckErrorDataMatches_oneLogInResponse() throws Exception {
        ErrorLog log = getErrorLog("WE1", "SE1", "OracleTestDB");
        ErrorLog log2 = getErrorLog("WE1", "SE1", "OracleProductionDB");
        logPersistenceService.persistErrorLog(log);
        logPersistenceService.persistErrorLog(log2);

        MvcResult mvcResult = this.mockMvc
                .perform(get(baseUrl + "workflow-executions/{workflow-execution}/scenario-executions/{scenario-execution}/connections/{connectionId}", "WE1", "SE1", "OracleTestDB")
                        .param("offset", "0"))
                .andExpect(status().isOk())
                .andReturn();

        List<ErrorDTO> response = objectMapper.readValue(mvcResult.getResponse().getContentAsString(), new TypeReference<List<ErrorDTO>>(){});

        assertEquals(1, response.size());

        ErrorDTO errorDTO = response.get(0);

        assertEquals(log.getCategory(), errorDTO.getErrorCategory());
        assertEquals(log.getErrorType(), errorDTO.getErrorType());
        assertEquals(log.getUserMessage(), errorDTO.getUserMessage());
        assertEquals(log.getTechnicalMessage(), errorDTO.getTechnicalMessage());
        assertEquals(log.getSolution(), errorDTO.getSolution());
        assertEquals(log.getLineageImpact(), errorDTO.getImpact());
        assertEquals(log.getLevel(), errorDTO.getLogLevel());
        assertEquals(log.getApplication(), errorDTO.getApplication());
    }

    @Test
    public void queryLog_addSendRequestForLogFromPackageCheckErrorDataMatches_oneLogInResponse() throws Exception {
        ErrorLog log = getPackageErrorLog("My Package 1");
        ErrorLog log2 = getPackageErrorLog("My Package 2");
        logPersistenceService.persistErrorLog(log);
        logPersistenceService.persistErrorLog(log2);

        MvcResult mvcResult = this.mockMvc
                .perform(get(baseUrl + "/packages/{packageName}", "My Package 1")
                        .param("offset", "0"))
                .andExpect(status().isOk())
                .andReturn();

        List<ErrorDTO> response = objectMapper.readValue(mvcResult.getResponse().getContentAsString(), new TypeReference<List<ErrorDTO>>(){});

        assertEquals(1, response.size());

        ErrorDTO errorDTO = response.get(0);

        assertEquals(log.getCategory(), errorDTO.getErrorCategory());
        assertEquals(log.getErrorType(), errorDTO.getErrorType());
        assertEquals(log.getUserMessage(), errorDTO.getUserMessage());
        assertEquals(log.getTechnicalMessage(), errorDTO.getTechnicalMessage());
        assertEquals(log.getSolution(), errorDTO.getSolution());
        assertEquals(log.getLineageImpact(), errorDTO.getImpact());
        assertEquals(log.getLevel(), errorDTO.getLogLevel());
        assertEquals(log.getApplication(), errorDTO.getApplication());
    }

}
