package Integration;

import java.util.List;
import java.util.stream.Collectors;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;
import org.springframework.test.context.web.WebAppConfiguration;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.context.WebApplicationContext;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;

import TestConfig.IntegrationTestsConfig;
import eu.profinit.manta.bachelor.thesis.log.viewer.controllers.dto.executions.ScenarioExecutionDTO;
import eu.profinit.manta.bachelor.thesis.log.viewer.controllers.dto.executions.WorkflowExecutionDTO;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.DistinctErrorType;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.ScenarioExecution;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.UploadedPackage;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.WorkflowExecution;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.LogPersistenceService;

import static org.jgroups.util.Util.assertEquals;
import static org.jgroups.util.Util.assertTrue;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;
import static utils.Utils.getErrorLog;

@RunWith(SpringJUnit4ClassRunner.class)
@ContextConfiguration(classes = {IntegrationTestsConfig.class})
@ActiveProfiles({"test"})
@WebAppConfiguration
@Transactional
public class MetadataControllerTests {

    @Autowired
    private WebApplicationContext webAppContext;

    @Autowired
    private ObjectMapper objectMapper;

    private MockMvc mockMvc;

    private String baseUrl = "/private/log-viewer/v1/";

    @Autowired
    private LogPersistenceService logPersistenceService;

    @Before
    public void setup() throws Exception {
        this.mockMvc = MockMvcBuilders.webAppContextSetup(this.webAppContext).build();
    }

    @Test
    public void listExecutions_insertSomeExecutionsAndRetrieveThem_correctlyRetrieved() throws Exception {
        logPersistenceService.persistErrorLog(getErrorLog("WE1", "SE11", "OracleTestDB1"));
        logPersistenceService.persistErrorLog(getErrorLog("WE1", "SE12", "OracleTestDB2"));
        logPersistenceService.persistErrorLog(getErrorLog("WE2", "SE21", "OracleTestDB1"));
        logPersistenceService.persistErrorLog(getErrorLog("WE2", "SE22", "OracleTestDB2"));
        logPersistenceService.persistErrorLog(getErrorLog("WE3", "SE31", "OracleTestDB1"));
        logPersistenceService.persistErrorLog(getErrorLog("WE3", "SE32", "OracleTestDB2"));
        logPersistenceService.persistErrorLog(getErrorLog("WE3", "SE33", "OracleTestDB3"));

        MvcResult mvcResult = this.mockMvc
                .perform(get(baseUrl + "/executions"))
                .andExpect(status().isOk())
                .andReturn();

        List<WorkflowExecutionDTO> response = objectMapper.readValue(mvcResult.getResponse().getContentAsString(), new TypeReference<List<WorkflowExecutionDTO>>(){});

        assertEquals(3, response.size());

        WorkflowExecutionDTO workflowExecution1 = response.get(0);
        assertEquals("WE1", workflowExecution1.getWorkflowExecutionId());
        assertEquals(3, workflowExecution1.getScenarioExecutions().size());

        List<ScenarioExecutionDTO> scenarioExecutions1 = workflowExecution1.getScenarioExecutions();
        List<String> scenarioExecutionIds1 = scenarioExecutions1.stream().map(ScenarioExecutionDTO::getScenarioExecutionId).collect(Collectors.toList());
        scenarioExecutionIds1.contains("SE1"); // test data
        scenarioExecutionIds1.contains("SE11");
        scenarioExecutionIds1.contains("SE12");

        WorkflowExecutionDTO workflowExecution2 = response.get(1);
        assertEquals("WE2", workflowExecution2.getWorkflowExecutionId());
        assertEquals(2, workflowExecution2.getScenarioExecutions().size());

        List<ScenarioExecutionDTO> scenarioExecutions2 = workflowExecution2.getScenarioExecutions();
        List<String> scenarioExecutionIds2 = scenarioExecutions2.stream().map(ScenarioExecutionDTO::getScenarioExecutionId).collect(Collectors.toList());
        scenarioExecutionIds2.contains("SE21");
        scenarioExecutionIds2.contains("SE22");

        WorkflowExecutionDTO workflowExecution3 = response.get(2);
        assertEquals("WE3", workflowExecution3.getWorkflowExecutionId());
        assertEquals(3, workflowExecution3.getScenarioExecutions().size());

        List<ScenarioExecutionDTO> scenarioExecutions3 = workflowExecution3.getScenarioExecutions();
        List<String> scenarioExecutionIds3 = scenarioExecutions3.stream().map(ScenarioExecutionDTO::getScenarioExecutionId).collect(Collectors.toList());
        scenarioExecutionIds3.contains("SE31");
        scenarioExecutionIds3.contains("SE32");
        scenarioExecutionIds3.contains("SE33");
    }

    @Test
    public void listApplications_insertThreeLogsWithTwoApplications_twoReturned() throws Exception {
        logPersistenceService.persistErrorLog(getErrorLog("WE1", "SE1", "OracleTestDB", "Application1"));
        logPersistenceService.persistErrorLog(getErrorLog("WE1", "SE1", "OracleTestDB", "Application1"));
        logPersistenceService.persistErrorLog(getErrorLog("WE1", "SE1", "OracleTestDB", "Application2"));

        MvcResult mvcResult = this.mockMvc
                .perform(get(baseUrl + "/applications"))
                .andExpect(status().isOk())
                .andReturn();

        List<String> response = objectMapper.readValue(mvcResult.getResponse().getContentAsString(), new TypeReference<List<String>>(){});

        assertEquals(2, response.size());
        assertTrue(response.contains("Application1"));
        assertTrue(response.contains("Application2"));
    }

    @Test
    public void listPackages_retrievePackagesFromTestData_twoReturned() throws Exception {
        MvcResult mvcResult = this.mockMvc
                .perform(get(baseUrl + "/packages"))
                .andExpect(status().isOk())
                .andReturn();

        List<UploadedPackage> response = objectMapper.readValue(mvcResult.getResponse().getContentAsString(), new TypeReference<List<UploadedPackage>>(){});

        List<String> names = response.stream().map(UploadedPackage::getPackageName).collect(Collectors.toList());

        assertEquals(2, response.size());
        assertTrue(names.contains("My Package 1"));
        assertTrue(names.contains("My Package 2"));
    }


    @Test
    public void getDistinctCategories_getDistinctCategoriesFromTestData_oneReturned() throws Exception {
        MvcResult mvcResult = this.mockMvc
                .perform(get(baseUrl + "/categories"))
                .andExpect(status().isOk())
                .andReturn();

        List<String> response = objectMapper.readValue(mvcResult.getResponse().getContentAsString(), new TypeReference<List<String>>(){});

        assertEquals(1, response.size());
        assertTrue(response.contains("TEST_CATEGORY"));
    }

    @Test
    public void getDistinctCategories_getDistinctCategoriesForWorkflowExecutionFromTestData_oneReturned() throws Exception {
        logPersistenceService.persistErrorLog(getErrorLog("WE1", "SE1", "OracleTestDB", "Application1"));
        logPersistenceService.persistErrorLog(getErrorLog("WE2", "SE2", "OracleTestDB", "Application1"));
        logPersistenceService.persistErrorLog(getErrorLog("WE2", "SE2", "OracleTestDB", "Application2"));

        MvcResult mvcResult = this.mockMvc
                .perform(get(baseUrl + "/workflow-executions/{workflowExecutionId}/categories", "WE1"))
                .andExpect(status().isOk())
                .andReturn();

        List<String> response = objectMapper.readValue(mvcResult.getResponse().getContentAsString(), new TypeReference<List<String>>(){});

        assertEquals(1, response.size());
        assertTrue(response.contains("TEST_CATEGORY"));
    }

    @Test
    public void getDistinctCategories_getDistinctCategoriesForScenarioExecutionFromTestData_oneReturned() throws Exception {
        logPersistenceService.persistErrorLog(getErrorLog("WE1", "SE1", "OracleTestDB", "Application1"));
        logPersistenceService.persistErrorLog(getErrorLog("WE1", "SE2", "OracleTestDB", "Application1"));
        logPersistenceService.persistErrorLog(getErrorLog("WE1", "SE2", "OracleTestDB", "Application2"));

        MvcResult mvcResult = this.mockMvc
                .perform(get(baseUrl + "/workflow-executions/{workflowExecutionId}/scenario-executions/{scenarioExecutionId}/categories", "WE1", "SE1"))
                .andExpect(status().isOk())
                .andReturn();

        List<String> response = objectMapper.readValue(mvcResult.getResponse().getContentAsString(), new TypeReference<List<String>>(){});

        assertEquals(1, response.size());
        assertTrue(response.contains("TEST_CATEGORY"));
    }

    @Test
    public void getDistinctCategories_getDistinctCategoriesForConnectionFromTestData_oneReturned() throws Exception {
        logPersistenceService.persistErrorLog(getErrorLog("WE1", "SE1", "OracleTestDB", "Application1"));
        logPersistenceService.persistErrorLog(getErrorLog("WE1", "SE1", "PostgreSQLDB", "Application1"));
        logPersistenceService.persistErrorLog(getErrorLog("WE1", "SE1", "OracleTestDB", "Application2"));

        MvcResult mvcResult = this.mockMvc
                .perform(get(baseUrl + "/workflow-executions/{workflowExecutionId}/scenario-executions/{scenarioExecutionId}/connections/{connectionId}/categories",
                        "WE1", "SE1", "PostgreSQLDB"))
                .andExpect(status().isOk())
                .andReturn();

        List<String> response = objectMapper.readValue(mvcResult.getResponse().getContentAsString(), new TypeReference<List<String>>(){});

        assertEquals(1, response.size());
        assertTrue(response.contains("TEST_CATEGORY"));
    }

    @Test
    public void getDistinctErrorTypes_getDistinctErrorTypesFromTestData_oneReturned() throws Exception {
        MvcResult mvcResult = this.mockMvc
                .perform(get(baseUrl + "/errortypes"))
                .andExpect(status().isOk())
                .andReturn();

        List<DistinctErrorType> response = objectMapper.readValue(mvcResult.getResponse().getContentAsString(), new TypeReference<List<DistinctErrorType>>(){});

        List<String> errors = response.stream().map(DistinctErrorType::getErrorType).collect(Collectors.toList());

        assertEquals(1, response.size());
        assertTrue(errors.contains("TEST_ERROR"));
    }

    @Test
    public void getDistinctErrorTypes_getDistinctErrorTypesForWorkflowExecutionFromTestData_oneReturned() throws Exception {
        logPersistenceService.persistErrorLog(getErrorLog("WE1", "SE1", "OracleTestDB", "Application1"));
        logPersistenceService.persistErrorLog(getErrorLog("WE2", "SE2", "OracleTestDB", "Application1"));
        logPersistenceService.persistErrorLog(getErrorLog("WE2", "SE2", "OracleTestDB", "Application2"));

        MvcResult mvcResult = this.mockMvc
                .perform(get(baseUrl + "/workflow-executions/{workflowExecutionId}/errortypes", "WE1"))
                .andExpect(status().isOk())
                .andReturn();

        List<DistinctErrorType> response = objectMapper.readValue(mvcResult.getResponse().getContentAsString(), new TypeReference<List<DistinctErrorType>>(){});

        List<String> errors = response.stream().map(DistinctErrorType::getErrorType).collect(Collectors.toList());

        assertEquals(1, response.size());
        assertTrue(errors.contains("TEST_ERROR"));
    }

    @Test
    public void getDistinctErrorTypes_getDistinctErrorTypesForScenarioExecutionFromTestData_oneReturned() throws Exception {
        logPersistenceService.persistErrorLog(getErrorLog("WE1", "SE1", "OracleTestDB", "Application1"));
        logPersistenceService.persistErrorLog(getErrorLog("WE1", "SE2", "OracleTestDB", "Application1"));
        logPersistenceService.persistErrorLog(getErrorLog("WE1", "SE2", "OracleTestDB", "Application2"));

        MvcResult mvcResult = this.mockMvc
                .perform(get(baseUrl + "/workflow-executions/{workflowExecutionId}/scenario-executions/{scenarioExecutionId}/errortypes", "WE1", "SE1"))
                .andExpect(status().isOk())
                .andReturn();

        List<DistinctErrorType> response = objectMapper.readValue(mvcResult.getResponse().getContentAsString(), new TypeReference<List<DistinctErrorType>>(){});

        List<String> errors = response.stream().map(DistinctErrorType::getErrorType).collect(Collectors.toList());

        assertEquals(1, response.size());
        assertTrue(errors.contains("TEST_ERROR"));
    }

    @Test
    public void getDistinctErrorTypes_getDistinctErrorTypesForConnectionFromTestData_oneReturned() throws Exception {
        logPersistenceService.persistErrorLog(getErrorLog("WE1", "SE1", "OracleTestDB", "Application1"));
        logPersistenceService.persistErrorLog(getErrorLog("WE1", "SE1", "PostgreSQLDB", "Application1"));
        logPersistenceService.persistErrorLog(getErrorLog("WE1", "SE1", "OracleTestDB", "Application2"));

        MvcResult mvcResult = this.mockMvc
                .perform(get(baseUrl + "/workflow-executions/{workflowExecutionId}/scenario-executions/{scenarioExecutionId}/connections/{connectionId}/errortypes",
                        "WE1", "SE1", "PostgreSQLDB"))
                .andExpect(status().isOk())
                .andReturn();

        List<DistinctErrorType> response = objectMapper.readValue(mvcResult.getResponse().getContentAsString(), new TypeReference<List<DistinctErrorType>>(){});

        List<String> errors = response.stream().map(DistinctErrorType::getErrorType).collect(Collectors.toList());

        assertEquals(1, response.size());
        assertTrue(errors.contains("TEST_ERROR"));
    }

}
