package Integration;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.List;
import java.util.stream.Collectors;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import org.apache.commons.io.IOUtils;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;
import org.springframework.test.context.web.WebAppConfiguration;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.context.WebApplicationContext;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;

import TestConfig.IntegrationTestsConfig;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.UploadedPackage;

import static org.jgroups.util.Util.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.multipart;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@RunWith(SpringJUnit4ClassRunner.class)
@ContextConfiguration(classes = {IntegrationTestsConfig.class})
@ActiveProfiles({"test"})
@WebAppConfiguration
@Transactional
public class PackageControllerTests {

    @Autowired
    private WebApplicationContext webAppContext;

    @Autowired
    private ObjectMapper objectMapper;

    private MockMvc mockMvc;

    private String baseUrl = "/private/log-viewer/v1/";

    @Value("classpath:applicationPackage.zip")
    private File applicationPackage;

    @Value("classpath:connectionPackage.zip")
    private File connectionPackage;

    @Before
    public void setup() throws Exception {
        this.mockMvc = MockMvcBuilders.webAppContextSetup(this.webAppContext).build();
    }

    @Test
    public void createPackageForConnection_getPackageCheckItContainsNecessaryFiles_threeFilesForConnectionPackage() throws Exception {
        MvcResult mvcResult = this.mockMvc
                .perform(get(baseUrl + "/workflow-executions/{workflowExecutionId}/scenario-executions/{scenarioExecutionId}/connection/{connectionId}/package", "WE1", "SE1", "OracleTestDB"))
                .andExpect(status().isOk())
                .andReturn();

        ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream(mvcResult.getResponse().getContentAsByteArray());
        IOUtils.copy(byteArrayInputStream, new FileOutputStream("./target/test-data/packageConnection.zip"));

        ZipFile file = new ZipFile("./target/test-data/packageConnection.zip");

        List<String> files = file.stream()
                .map(ZipEntry::getName)
                .collect(Collectors.toList());

        assertTrue(files.contains("OracleTestDB_metadata.json"));
        assertTrue(files.contains("OracleTestDB_logs.csv"));
        assertTrue(files.contains("OracleTestDB_errors.csv"));
    }

    @Test
    public void createPackageForApplication_getPackageCheckItContainsNecessaryFiles_threeFilesForConnectionPackage() throws Exception {
        MvcResult mvcResult = this.mockMvc
                .perform(get(baseUrl + "/applications/{application}/package",  "Test Application"))
                .andExpect(status().isOk())
                .andReturn();

        ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream(mvcResult.getResponse().getContentAsByteArray());
        IOUtils.copy(byteArrayInputStream, new FileOutputStream("./target/test-data/packageApplication.zip"));

        ZipFile file = new ZipFile("./target/test-data/packageApplication.zip");

        List<String> files = file.stream()
                .map(ZipEntry::getName)
                .collect(Collectors.toList());

        assertTrue(files.contains("Test Application_logs.csv"));
        assertTrue(files.contains("Test Application_errors.csv"));
        assertTrue(files.contains("application.json"));
    }

    @Test
    public void uploadPackage_checkUploaded_uploaded() throws Exception {
        FileInputStream fileInputStream = new FileInputStream(applicationPackage);
        MockMultipartFile multiPartFile = new MockMultipartFile("uploadedPackage", applicationPackage.getName(), "application/zip", IOUtils.toByteArray(fileInputStream));

        String packageName = "New Package";

        this.mockMvc
                .perform(multipart(baseUrl + "/upload")
                        .file(multiPartFile)
                        .param("packageName", packageName))
                .andExpect(status().isOk());

        MvcResult mvcResult = this.mockMvc
                .perform(get(baseUrl + "/packages"))
                .andExpect(status().isOk())
                .andReturn();

        List<UploadedPackage> response = objectMapper.readValue(mvcResult.getResponse().getContentAsString(), new TypeReference<List<UploadedPackage>>(){});

        List<String> names = response.stream().map(UploadedPackage::getPackageName).collect(Collectors.toList());

        assertEquals(3, response.size());
        assertTrue(names.contains("My Package 1")); // test data
        assertTrue(names.contains("My Package 2")); // test data
        assertTrue(names.contains(packageName));
    }
}
