package Integration;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;
import org.springframework.test.context.web.WebAppConfiguration;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.context.WebApplicationContext;

import com.fasterxml.jackson.databind.ObjectMapper;

import TestConfig.IntegrationTestsConfig;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.ErrorLog;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.LogDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.LogPersistenceService;

import static org.junit.Assert.assertEquals;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.delete;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;
import static utils.Utils.getErrorLog;
import static utils.Utils.getPackageErrorLog;

@RunWith(SpringJUnit4ClassRunner.class)
@ContextConfiguration(classes = {IntegrationTestsConfig.class})
@ActiveProfiles({"test"})
@WebAppConfiguration
@Transactional
public class PruningControllerTests {

    @Autowired
    private WebApplicationContext webAppContext;

    private MockMvc mockMvc;

    @Autowired
    private LogDAO logDAO;

    @Autowired
    private LogPersistenceService logPersistenceService;

    private String baseUrl = "/private/log-viewer/v1/";

    @Before
    public void setup() throws Exception {
        this.mockMvc = MockMvcBuilders.webAppContextSetup(this.webAppContext).build();
    }

    @Test
    public void pruneWorkflowExecution_insertTwoLogsFromDifferentWorkflowExecutionsAndThenDeleteOne_oneStayedInDB() throws Exception {
        ErrorLog errorLog1 = getErrorLog("WE1", "SE1", "OracleTestDB");
        ErrorLog errorLog2 = getErrorLog("WE2", "SE2", "OracleTestDB");

        logPersistenceService.persistErrorLog(errorLog1);
        logPersistenceService.persistErrorLog(errorLog2);

        assertEquals(2, logDAO.getAll().size());

        this.mockMvc
                .perform(delete(baseUrl + "/workflow-executions/{workflowExecutionId}", "WE1"))
                .andExpect(status().isOk());

        assertEquals(1, logDAO.getAll().size());
    }

    @Test
    public void pruneApplication_insertTwoLogsFromDifferentApplicationsAndThenDeleteOne_oneStayedInDB() throws Exception {
        ErrorLog errorLog1 = getErrorLog("WE1", "SE1", "OracleTestDB", "APPLICATION1");
        ErrorLog errorLog2 = getErrorLog("WE1", "SE1", "OracleTestDB", "APPLICATION2");

        logPersistenceService.persistErrorLog(errorLog1);
        logPersistenceService.persistErrorLog(errorLog2);

        assertEquals(2, logDAO.getAll().size());

        this.mockMvc
                .perform(delete(baseUrl + "/applications/{application}", "APPLICATION1"))
                .andExpect(status().isOk());

        assertEquals(1, logDAO.getAll().size());
    }

    @Test
    public void prunePackage_insertTwoLogsFromDifferentApplicationsAndThenDeleteOne_oneStayedInDB() throws Exception {
        ErrorLog errorLog1 = getPackageErrorLog("My Package 1");
        ErrorLog errorLog2 = getPackageErrorLog("My Package 2");

        logPersistenceService.persistErrorLog(errorLog1);
        logPersistenceService.persistErrorLog(errorLog2);

        assertEquals(2, logDAO.getAll().size());

        this.mockMvc
                .perform(delete(baseUrl + "/packages/{packageName}", "My Package 1"))
                .andExpect(status().isOk());

        assertEquals(1, logDAO.getAll().size());
    }

    @Test
    public void pruneScenarioExecution_insertTwoLogsFromDifferentScenarioExecutionsAndThenDeleteOne_oneStayedInDB() throws Exception {
        ErrorLog errorLog1 = getErrorLog("WE1", "SE1", "OracleTestDB", "APPLICATION1");
        ErrorLog errorLog2 = getErrorLog("WE1", "SE2", "OracleTestDB", "APPLICATION2");

        logPersistenceService.persistErrorLog(errorLog1);
        logPersistenceService.persistErrorLog(errorLog2);

        assertEquals(2, logDAO.getAll().size());

        this.mockMvc
                .perform(delete(baseUrl + "/workflow-executions/{workflowExecutionId}/scenario-executions/{scenarioExecutionId}", "WE1", "SE1"))
                .andExpect(status().isOk());

        assertEquals(1, logDAO.getAll().size());
    }

    @Test
    public void pruneConnection_insertTwoLogsFromDifferentConnectionsAndThenDeleteOne_oneStayedInDB() throws Exception {
        ErrorLog errorLog1 = getErrorLog("WE1", "SE1", "OracleTestDB", "APPLICATION1");
        ErrorLog errorLog2 = getErrorLog("WE1", "SE1", "PostgreSQLDB", "APPLICATION2");

        logPersistenceService.persistErrorLog(errorLog1);
        logPersistenceService.persistErrorLog(errorLog2);

        assertEquals(2, logDAO.getAll().size());

        this.mockMvc
                .perform(delete(baseUrl + "/workflow-executions/{workflowExecutionId}/scenario-executions/{scenarioExecutionId}/connections/{connectionId}", "WE1", "SE1", "OracleTestDB"))
                .andExpect(status().isOk());

        assertEquals(1, logDAO.getAll().size());
    }

}
