package ServiceTests;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.compress.archivers.zip.ZipArchiveEntry;
import org.apache.commons.compress.archivers.zip.ZipArchiveInputStream;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.Validate;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.MockitoJUnit;
import org.mockito.junit.MockitoRule;
import org.mockito.quality.Strictness;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Import;
import org.springframework.context.annotation.PropertySource;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringRunner;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;

import TestConfig.ServiceTestsConfiguration;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.Connection;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.PackageMetadata;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.PackageCreateDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.CreatePackageService;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.exceptions.UnableToCreateDumpException;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.impl.CreatePackageServiceImpl;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.anyLong;
import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.Mockito.doAnswer;
import static org.mockito.Mockito.when;

@RunWith(SpringRunner.class)
@ContextConfiguration(classes = {ServiceTestsConfiguration.class, CreatePackageServiceTests.CreatePackageServiceConfig.class})
@ActiveProfiles({"test"})
public class CreatePackageServiceTests {

    @Configuration
    @PropertySource("classpath:testrepository.properties")
    @Import(ServiceTestsConfiguration.class)
    static class CreatePackageServiceConfig {

        @Bean
        public CreatePackageService createPackageService(ObjectMapper objectMapper,
                                                         @Value("${packaging.base.zip.path}") String basePackagePath,
                                                         @Value("${packaging.base.csv.path}") String baseCsvPath,
                                                         @Value("${packaging.base.json.scenario.path}") String baseScenarioJsonPath,
                                                         @Value("${packaging.errors.csv.path}") String errorsCsvPath,
                                                         @Value("${packaging.base.json.application.path}") String applicationMetadataJsonPath){
            return new CreatePackageServiceImpl(packageCreateDAO, objectMapper, basePackagePath, baseCsvPath, baseScenarioJsonPath, errorsCsvPath, applicationMetadataJsonPath);
        }

    }

    @Rule
    public MockitoRule rule = MockitoJUnit.rule().strictness(Strictness.STRICT_STUBS);

    @Mock
    private static PackageCreateDAO packageCreateDAO;

    @Autowired
    @InjectMocks
    private CreatePackageService createPackageService;

    @Value("${packaging.base.zip.path}")
    private String basePackagePath;

    @Value("${packaging.base.csv.path}")
    private String baseCsvPath;

    @Value("${packaging.base.json.scenario.path}")
    private String baseJsonPath;

    @Value("${packaging.errors.csv.path}")
    private String errorsCsvPath;

    @Value("${packaging.base.json.application.path}")
    private String applicationJsonPath;

    @Autowired
    private ObjectMapper objectMapper;

    @Before
    public void before() throws IOException {
        File directory = new File("./target/test-data/packages");
        FileUtils.deleteDirectory(directory);
    }

    @Test
    public void createPackageForApplication_testCorrectFilesCreatedCorrectFormatForMetadataFile_filesCreatedCorrectFormat() throws IOException {
        doAnswer(invocation -> {
            File file = new File(String.format(errorsCsvPath, "APPLICATION"));

            File parentFile = file.getParentFile();
            if(!parentFile.exists()) assertTrue(parentFile.mkdir());

            Validate.isTrue(file.createNewFile());
            return null;
        }).when(packageCreateDAO).createErrorDefinitionsDumpForApplication(anyString(), anyString());

        doAnswer(invocation -> {
            File file = new File(String.format(baseCsvPath, "APPLICATION"));

            File parentFile = file.getParentFile();
            if(!parentFile.exists()) assertTrue(parentFile.mkdir());

            Validate.isTrue(file.createNewFile());
            return null;
        }).when(packageCreateDAO).createLogDumpForApplication(anyString(), anyString());

        createPackageService.createLogsPackageForApplication("APPLICATION");

        File packageFile = new File("./target/test-data/packages/APPLICATION_package.zip");

        assertTrue(packageFile.exists());

        File directory = new File("./target/test-data/packages");

        unzip(packageFile, directory);

        assertNotNull(directory.listFiles());

        assertEquals(4, directory.listFiles().length);

        File logsFile = new File("./target/test-data/packages/APPLICATION_logs.csv");
        File errorsFile = new File("./target/test-data/packages/APPLICATION_errors.csv");
        File metadataFile = new File("./target/test-data/packages/application.json");

        assertTrue(logsFile.exists());
        assertTrue(errorsFile.exists());
        assertTrue(metadataFile.exists());

        TypeReference<HashMap<String,String>> typeRef
                = new TypeReference<HashMap<String,String>>() {};

        Map<String, String> metadata = objectMapper.readValue(FileUtils.readFileToString(metadataFile), typeRef);

        assertEquals(1, metadata.size());
        assertTrue(metadata.containsKey("APPLICATION"));
        assertEquals("APPLICATION", metadata.get("APPLICATION"));
    }

    @Test
    public void createPackageForConnection_testCorrectFilesCreatedCorrectFormatForMetadataFile_filesCreatedCorrectFormat() throws IOException {
        String connectionId = "OracleTestDB";
        String scenarioName = "Oracle DDL Dataflow Analysis";
        String phase = "Analysis";
        String technology = "Oracle";

        doAnswer(invocation -> {
            File file = new File(String.format(errorsCsvPath, connectionId));

            File parentFile = file.getParentFile();
            if(!parentFile.exists()) assertTrue(parentFile.mkdir());

            Validate.isTrue(file.createNewFile());
            return null;
        }).when(packageCreateDAO).createErrorDefinitionsDumpForConnection(anyLong(), anyString());

        doAnswer(invocation -> {
            File file = new File(String.format(baseCsvPath, connectionId));

            File parentFile = file.getParentFile();
            if(!parentFile.exists()) assertTrue(parentFile.mkdir());

            Validate.isTrue(file.createNewFile());
            return null;
        }).when(packageCreateDAO).createLogDumpForConnection(anyLong(), anyString());

        when(packageCreateDAO.getPackageMetadata(any(Connection.class))).thenReturn(new PackageMetadata(1L, connectionId, scenarioName, phase, technology));

        createPackageService.createLogsPackageForConnection("WE1", "SE1", connectionId);

        File packageFile = new File("./target/test-data/packages/" + connectionId + "_package.zip");

        assertTrue(packageFile.exists());

        File directory = new File("./target/test-data/packages");

        unzip(packageFile, directory);

        assertNotNull(directory.listFiles());

        assertEquals(4, directory.listFiles().length);

        File logsFile = new File("./target/test-data/packages/" + connectionId + "_logs.csv");
        File errorsFile = new File("./target/test-data/packages/" + connectionId + "_errors.csv");
        File metadataFile = new File("./target/test-data/packages/" + connectionId + "_metadata.json");

        assertTrue(logsFile.exists());
        assertTrue(errorsFile.exists());
        assertTrue(metadataFile.exists());

        PackageMetadata packageMetadata = objectMapper.readValue(FileUtils.readFileToString(metadataFile), PackageMetadata.class);

        assertEquals(connectionId, packageMetadata.getConnectionId());
        assertEquals(scenarioName, packageMetadata.getScenarioName());
        assertEquals(phase, packageMetadata.getPhase());
        assertEquals(technology, packageMetadata.getTechnology());
    }

    @Test(expected = UnableToCreateDumpException.class)
    public void createPackageForConnection_passNonExistingConnection_shouldThrowException(){
        when(packageCreateDAO.getPackageMetadata(any(Connection.class))).thenReturn(null);
        createPackageService.createLogsPackageForConnection("WE1", "SE1", "CONNECTION");
    }

    private void unzip(File file, File targetDirectory) throws IOException {
        ZipArchiveInputStream zipArchiveInputStream = new ZipArchiveInputStream(new FileInputStream(file));

        ZipArchiveEntry entry = null;
        FileOutputStream fos = null;
        while((entry = zipArchiveInputStream.getNextZipEntry()) != null){
            File outputFile = new File(targetDirectory, entry.getName());
            fos = new FileOutputStream(outputFile);
            IOUtils.copy(zipArchiveInputStream, fos);
            IOUtils.closeQuietly(fos);
        }

        IOUtils.closeQuietly(zipArchiveInputStream);
    }
}
