package ServiceTests;

import java.util.Collections;
import java.util.Optional;

import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.MockitoJUnit;
import org.mockito.junit.MockitoRule;
import org.mockito.quality.Strictness;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Import;
import org.springframework.context.annotation.PropertySource;
import org.springframework.scheduling.concurrent.ThreadPoolTaskScheduler;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringRunner;

import TestConfig.ServiceTestsConfiguration;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.Connection;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.ErrorDefinition;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.ErrorMetadata;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.ScenarioExecution;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.WorkflowExecution;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.DebugLog;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.ErrorLog;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.ConnectionsDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.ErrorDefinitionDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.ErrorMetadataDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.LogDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.ScenarioExecutionDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.WorkflowExecutionDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.LogPersistenceService;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.PruneService;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.impl.LogPersistenceServiceImpl;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.impl.PruneServiceImpl;

import static org.junit.Assert.assertEquals;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.never;
import static org.mockito.Mockito.reset;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;
import static utils.Utils.getDebugLog;
import static utils.Utils.getErrorLog;

@RunWith(SpringRunner.class)
@ContextConfiguration(classes = {ServiceTestsConfiguration.class, LogPersistenceServiceTests.LogPersistenceServiceConfig.class})
@ActiveProfiles({"test"})
public class LogPersistenceServiceTests {

    @Configuration
    @PropertySource("classpath:testrepository.properties")
    @Import(ServiceTestsConfiguration.class)
    static class LogPersistenceServiceConfig {
        @Bean
        public LogPersistenceService logPersistenceService(ThreadPoolTaskScheduler threadPoolTaskScheduler){
            return new LogPersistenceServiceImpl(logDAO, workflowExecutionDAO, scenarioExecutionDAO, pruneService(threadPoolTaskScheduler), errorMetadataDAO,
                    errorDefinitionDAO, connectionsDAO);
        }

        @Value("${logviewer.pruneAfter}")
        private double pruneAfter;

        @Bean
        public PruneService pruneService(ThreadPoolTaskScheduler threadPoolTaskScheduler){
            return new PruneServiceImpl(workflowExecutionDAO, pruneAfter, threadPoolTaskScheduler, errorDefinitionDAO, errorMetadataDAO, logDAO);
        }
    }

    @Rule
    public MockitoRule rule = MockitoJUnit.rule().strictness(Strictness.STRICT_STUBS);

    @Mock
    private static LogDAO logDAO;

    private static WorkflowExecutionDAO workflowExecutionDAO = getWorkflowMock();

    @Mock
    private static ScenarioExecutionDAO scenarioExecutionDAO;

    @Mock
    private static ErrorMetadataDAO errorMetadataDAO;

    @Mock
    private static ErrorDefinitionDAO errorDefinitionDAO;

    @Mock
    private static ConnectionsDAO connectionsDAO;

    @Autowired
    @InjectMocks
    private PruneService pruneService;

    @Autowired
    @InjectMocks
    private LogPersistenceService logPersistenceService;

    @Before
    public void before(){
        reset(logDAO);
        reset(workflowExecutionDAO);
        reset(scenarioExecutionDAO);
        reset(errorMetadataDAO);
        reset(errorDefinitionDAO);
        reset(connectionsDAO);
    }

    /**
     * Due to post-construct method in PruneSchedulingService, which takes precedence before @Before annotation of tests, therefore the mock
     * behaviour cannot be defined before calling it and thus results in null ptr exception.
     */
    private static WorkflowExecutionDAO getWorkflowMock(){
        WorkflowExecutionDAO workflowExecutionDAO = mock(WorkflowExecutionDAO.class);
        when(workflowExecutionDAO.getAll()).thenReturn(Collections.emptyList());
        return workflowExecutionDAO;
    }

    @Test
    public void persistErrorLog_insertErrorLogBelongingToExistingConnection_logSuccessfullySaved(){
        ErrorLog errorLog = getErrorLog("WE1", "SE1", "OracleTestDB");

        when(connectionsDAO.getIdByData(any(Connection.class))).thenReturn(1L);
        when(errorMetadataDAO.getIdByData(any(ErrorMetadata.class))).thenReturn(1L);

        logPersistenceService.persistErrorLog(errorLog);

        verify(connectionsDAO, never()).save(any(Connection.class));
        verify(scenarioExecutionDAO, never()).find("SE1");
        verify(workflowExecutionDAO, never()).find("WE1");
        verify(errorMetadataDAO, never()).save(any(ErrorMetadata.class));
        verify(errorDefinitionDAO, never()).save(any(ErrorDefinition.class));
        verify(logDAO, times(1)).save(errorLog, 1L, 1L);
    }

    @Test
    public void persistErrorLog_insertErrorLogToNewConnectionWhileScenarioAlreadyExists_logInsertedAndConnectionInserted(){
        ErrorLog errorLog = getErrorLog("WE2", "SE2", "OracleTestDB1");

        when(connectionsDAO.getIdByData(any(Connection.class))).thenReturn(null);
        when(errorMetadataDAO.getIdByData(any(ErrorMetadata.class))).thenReturn(1L);
        when(scenarioExecutionDAO.find(anyString())).thenReturn(Optional.of(new ScenarioExecution()));
        when(connectionsDAO.save(any(Connection.class))).thenAnswer(invocation -> {
            Connection connection = invocation.getArgument(0);
            connection.setConnectionDbId(1L);
            return 1L;
        });

        logPersistenceService.persistErrorLog(errorLog);

        verify(connectionsDAO, times(1)).save(any(Connection.class));
        verify(scenarioExecutionDAO, times(1)).find("SE2");
        verify(workflowExecutionDAO, never()).find("WE2");
        verify(errorMetadataDAO, never()).save(any(ErrorMetadata.class));
        verify(errorDefinitionDAO, never()).save(any(ErrorDefinition.class));
        verify(logDAO, times(1)).save(errorLog, 1L, 1L);
    }

    @Test
    public void persistErrorLog_insertErrorLogToNewScenarioExecutionWhileWorkflowExecutionExists_logInsertedConnectioninsertedScenarioInserted(){
        ErrorLog errorLog = getErrorLog("WE3", "SE3", "OracleTestDB3");

        when(connectionsDAO.getIdByData(any(Connection.class))).thenReturn(null);
        when(errorMetadataDAO.getIdByData(any(ErrorMetadata.class))).thenReturn(1L);
        when(scenarioExecutionDAO.find(anyString())).thenReturn(Optional.empty());
        when(workflowExecutionDAO.find(anyString())).thenReturn(Optional.of(new WorkflowExecution("WE3")));
        when(connectionsDAO.save(any(Connection.class))).thenAnswer(invocation -> {
            Connection connection = invocation.getArgument(0);
            connection.setConnectionDbId(1L);
            return 1L;
        });

        logPersistenceService.persistErrorLog(errorLog);

        verify(connectionsDAO, times(1)).save(any(Connection.class));
        verify(scenarioExecutionDAO, times(1)).save(any(ScenarioExecution.class));
        verify(workflowExecutionDAO, never()).save(any(WorkflowExecution.class));
        verify(scenarioExecutionDAO, times(1)).find(anyString());
        verify(workflowExecutionDAO, times(1)).find(anyString());
        verify(errorMetadataDAO, never()).save(any(ErrorMetadata.class));
        verify(errorDefinitionDAO, never()).save(any(ErrorDefinition.class));
        verify(logDAO, times(1)).save(errorLog, 1L, 1L);
    }

    @Test
    public void persistErrorLog_insertErrorLogNewWorkflowExecution_logInsertedAndAllOtherAreInsertedAsWell(){
        ErrorLog errorLog = getErrorLog("WE4", "SE4", "OracleTestDB4");

        when(connectionsDAO.getIdByData(any(Connection.class))).thenReturn(null);
        when(errorMetadataDAO.getIdByData(any(ErrorMetadata.class))).thenReturn(1L);
        when(scenarioExecutionDAO.find(anyString())).thenReturn(Optional.empty());
        when(workflowExecutionDAO.find(anyString())).thenReturn(Optional.empty());
        when(connectionsDAO.save(any(Connection.class))).thenAnswer(invocation -> {
            Connection connection = invocation.getArgument(0);
            connection.setConnectionDbId(1L);
            return 1L;
        });

        assertEquals(0, pruneService.getScheduledTasks().size());

        logPersistenceService.persistErrorLog(errorLog);

        verify(connectionsDAO, times(1)).save(any(Connection.class));
        verify(scenarioExecutionDAO, times(1)).save(any(ScenarioExecution.class));
        verify(workflowExecutionDAO, times(1)).save(any(WorkflowExecution.class));
        verify(scenarioExecutionDAO, times(1)).find(anyString());
        verify(workflowExecutionDAO, times(1)).find(anyString());
        verify(errorMetadataDAO, never()).save(any(ErrorMetadata.class));
        verify(errorDefinitionDAO, never()).save(any(ErrorDefinition.class));
        verify(logDAO, times(1)).save(errorLog, 1L, 1L);

        assertEquals(1, pruneService.getScheduledTasks().size());
    }

    @Test
    public void persistErrorLog_insertErrorLogExistingConnectionNewErrorMetadata_logInsertedErrorMetadataInserted(){
        ErrorLog errorLog = getErrorLog("WE1", "SE1", "OracleTestDB");

        when(connectionsDAO.getIdByData(any(Connection.class))).thenReturn(1L);
        when(errorMetadataDAO.getIdByData(any(ErrorMetadata.class))).thenReturn(null);
        when(errorDefinitionDAO.getIdByData(any(ErrorDefinition.class))).thenReturn(1L);
        when(errorMetadataDAO.save(any(ErrorMetadata.class))).thenAnswer(invocation -> {
            ErrorMetadata errorMetadata = invocation.getArgument(0);
            errorMetadata.setErrorId(2L);
            return 1L;
        });

        logPersistenceService.persistErrorLog(errorLog);

        verify(connectionsDAO, never()).save(any(Connection.class));
        verify(errorMetadataDAO, times(1)).save(any(ErrorMetadata.class));
        verify(scenarioExecutionDAO, never()).find("SE1");
        verify(workflowExecutionDAO, never()).find("WE1");
        verify(errorDefinitionDAO, never()).save(any(ErrorDefinition.class));
        verify(logDAO, times(1)).save(errorLog, 2L, 1L);
    }

    @Test
    public void persistErrorLog_insertErrorLogExistingConnectionInsertMetadataAndDefinition_logInsertedErrorMetadataInsertedErrorDefinitionInserted(){
        ErrorLog errorLog = getErrorLog("WE1", "SE1", "OracleTestDB");

        when(connectionsDAO.getIdByData(any(Connection.class))).thenReturn(1L);
        when(errorMetadataDAO.getIdByData(any(ErrorMetadata.class))).thenReturn(null);
        when(errorDefinitionDAO.getIdByData(any(ErrorDefinition.class))).thenReturn(null);
        when(errorMetadataDAO.save(any(ErrorMetadata.class))).thenAnswer(invocation -> {
            ErrorMetadata errorMetadata = invocation.getArgument(0);
            errorMetadata.setErrorId(2L);
            return 1L;
        });
        when(errorDefinitionDAO.save(any(ErrorDefinition.class))).thenAnswer(invocation -> {
            ErrorDefinition errorDefinition = invocation.getArgument(0);
            errorDefinition.setErrorDefinitionId(2L);
            return 1L;
        });

        logPersistenceService.persistErrorLog(errorLog);

        verify(connectionsDAO, never()).save(any(Connection.class));
        verify(errorMetadataDAO, times(1)).save(any(ErrorMetadata.class));
        verify(scenarioExecutionDAO, never()).find("SE1");
        verify(workflowExecutionDAO, never()).find("WE1");
        verify(errorDefinitionDAO, times(1)).save(any(ErrorDefinition.class));
        verify(logDAO, times(1)).save(errorLog, 2L, 1L);
    }

    @Test
    public void persistDebugLog_insertDebugLogBelongingToExistingConnection_logSuccessfullySaved(){
        DebugLog debugLog = getDebugLog("WE1", "SE1", "OracleTestDB");

        when(connectionsDAO.getIdByData(any(Connection.class))).thenReturn(1L);

        logPersistenceService.persistDebugLog(debugLog);

        verify(connectionsDAO, never()).save(any(Connection.class));
        verify(scenarioExecutionDAO, never()).find("SE1");
        verify(workflowExecutionDAO, never()).find("WE1");
        verify(logDAO, times(1)).save(debugLog, 1L);
    }

    @Test
    public void persistDebugLog_insertDebugLogToNewConnectionWhileScenarioAlreadyExists_logInsertedAndConnectionInserted(){
        DebugLog debugLog = getDebugLog("WE1", "SE1", "OracleTestDB");

        when(connectionsDAO.getIdByData(any(Connection.class))).thenReturn(null);
        when(scenarioExecutionDAO.find(anyString())).thenReturn(Optional.of(new ScenarioExecution()));
        when(connectionsDAO.save(any(Connection.class))).thenAnswer(invocation -> {
            Connection connection = invocation.getArgument(0);
            connection.setConnectionDbId(1L);
            return 1L;
        });

        logPersistenceService.persistDebugLog(debugLog);

        verify(connectionsDAO, times(1)).save(any(Connection.class));
        verify(scenarioExecutionDAO, times(1)).find("SE1");
        verify(workflowExecutionDAO, never()).find("WE1");
        verify(logDAO, times(1)).save(debugLog,  1L);
    }

    @Test
    public void persistDebugLog_insertDebugLogToNewScenarioExecutionWhileWorkflowExecutionExists_logInsertedConnectioninsertedScenarioInserted(){
        DebugLog debugLog = getDebugLog("WE1", "SE1", "OracleTestDB");

        when(connectionsDAO.getIdByData(any(Connection.class))).thenReturn(null);
        when(scenarioExecutionDAO.find(anyString())).thenReturn(Optional.empty());
        when(workflowExecutionDAO.find(anyString())).thenReturn(Optional.of(new WorkflowExecution("WE3")));
        when(connectionsDAO.save(any(Connection.class))).thenAnswer(invocation -> {
            Connection connection = invocation.getArgument(0);
            connection.setConnectionDbId(1L);
            return 1L;
        });

        logPersistenceService.persistDebugLog(debugLog);

        verify(connectionsDAO, times(1)).save(any(Connection.class));
        verify(scenarioExecutionDAO, times(1)).save(any(ScenarioExecution.class));
        verify(workflowExecutionDAO, never()).save(any(WorkflowExecution.class));
        verify(scenarioExecutionDAO, times(1)).find(anyString());
        verify(workflowExecutionDAO, times(1)).find(anyString());
        verify(logDAO, times(1)).save(debugLog, 1L);
    }

    @Test
    public void persistDebugLog_insertDebugLogNewWorkflowExecution_logInsertedAndAllOtherAreInsertedAsWell(){
        DebugLog debugLog = getDebugLog("WE1", "SE1", "OracleTestDB");

        when(connectionsDAO.getIdByData(any(Connection.class))).thenReturn(null);
        when(scenarioExecutionDAO.find(anyString())).thenReturn(Optional.empty());
        when(workflowExecutionDAO.find(anyString())).thenReturn(Optional.empty());
        when(connectionsDAO.save(any(Connection.class))).thenAnswer(invocation -> {
            Connection connection = invocation.getArgument(0);
            connection.setConnectionDbId(1L);
            return 1L;
        });

        logPersistenceService.persistDebugLog(debugLog);

        verify(connectionsDAO, times(1)).save(any(Connection.class));
        verify(scenarioExecutionDAO, times(1)).save(any(ScenarioExecution.class));
        verify(workflowExecutionDAO, times(1)).save(any(WorkflowExecution.class));
        verify(scenarioExecutionDAO, times(1)).find(anyString());
        verify(workflowExecutionDAO, times(1)).find(anyString());
        verify(logDAO, times(1)).save(debugLog, 1L);
    }

}
