package ServiceTests;

import java.util.Arrays;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;

import org.junit.After;
import org.junit.FixMethodOrder;
import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.MethodSorters;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.internal.util.reflection.FieldSetter;
import org.mockito.junit.MockitoJUnit;
import org.mockito.junit.MockitoRule;
import org.mockito.quality.Strictness;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Import;
import org.springframework.context.annotation.PropertySource;
import org.springframework.scheduling.concurrent.ThreadPoolTaskScheduler;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringRunner;

import TestConfig.ServiceTestsConfiguration;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.WorkflowExecution;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.ErrorDefinitionDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.ErrorMetadataDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.LogDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.WorkflowExecutionDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.PruneService;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.exceptions.UnableToPruneWorkflowExecutionException;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.impl.PruneServiceImpl;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@RunWith(SpringRunner.class)
@ContextConfiguration(classes = {ServiceTestsConfiguration.class, PruneServiceTests.PruneSchedulingConfig.class})
@ActiveProfiles({"test"})
@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class PruneServiceTests {

    @Configuration
    @Import(ServiceTestsConfiguration.class)
    @PropertySource("classpath:testrepository.properties")
    static class PruneSchedulingConfig {

        @Value("${logviewer.pruneAfter}")
        private double pruneAfter;

        @Bean
        public PruneService pruneService(ThreadPoolTaskScheduler threadPoolTaskScheduler){
            return new PruneServiceImpl(workflowExecutionDAO, pruneAfter, threadPoolTaskScheduler, errorDefinitionDAO, errorMetadataDAO, logDAO);
        }

    }

    @Rule
    public MockitoRule rule = MockitoJUnit.rule().strictness(Strictness.STRICT_STUBS);

    private static WorkflowExecutionDAO workflowExecutionDAO = getWorkflowMock();

    @Mock
    private static ErrorDefinitionDAO errorDefinitionDAO;

    @Mock
    private static ErrorMetadataDAO errorMetadataDAO;

    @Mock
    private static LogDAO logDAO;

    @Autowired
    private ThreadPoolTaskScheduler threadPoolTaskScheduler;

    @Value("${logviewer.pruneAfter}")
    private double pruneAfter;

    @Autowired
    @InjectMocks
    private PruneService pruneService;

    /**
     * Due to post-construct method in PruneSchedulingService, which takes precedence before @Before annotation of tests, therefore the mock
     * behaviour cannot be defined before calling it and thus results in null ptr exception.
     */
    private static WorkflowExecutionDAO getWorkflowMock(){
        WorkflowExecutionDAO workflowExecutionDAO = mock(WorkflowExecutionDAO.class);
        when(workflowExecutionDAO.getAll()).thenReturn(Collections.emptyList());
        return workflowExecutionDAO;
    }

    @After
    public void after() throws NoSuchFieldException {
        FieldSetter.setField(pruneService, pruneService.getClass().getDeclaredField("pruneTasks"), new HashMap<>());
    }

    @Test
    public void dropPruningFor_dropScheduledPruning_pruningDropped() {
        WorkflowExecution workflowExecution = new WorkflowExecution("WE5");
        pruneService.schedulePruning(workflowExecution);
        assertEquals(1, pruneService.getScheduledTasks().size());
        pruneService.dropPruningFor(workflowExecution);
        assertTrue(pruneService.getScheduledTasks().isEmpty());
    }

    @Test(expected = UnableToPruneWorkflowExecutionException.class)
    public void dropPruningFor_dropScheduledPruning_exceptionThrownBecauseUnderWay() throws NoSuchFieldException {
        WorkflowExecution workflowExecution = new WorkflowExecution("WE4");
        Date pruneDate = new Date(0);

        PruneServiceImpl.PruneTask pruneTask = pruneService.newPruningTask(workflowExecution, pruneDate);
        ScheduledFuture<?> scheduledFuture = threadPoolTaskScheduler.schedule(pruneTask, pruneDate);

        Map<PruneServiceImpl.PruneTask, ScheduledFuture<?>> scheduledTaskMap = new HashMap<>();
        scheduledTaskMap.put(pruneTask, scheduledFuture);

        FieldSetter.setField(pruneService, pruneService.getClass().getDeclaredField("pruneTasks"), scheduledTaskMap);

        pruneService.dropPruningFor(workflowExecution);
    }

    @Test
    public void schedulePruning_demandNewPruningToBeScheduled_pruningScheduled() {
        WorkflowExecution workflowExecution = new WorkflowExecution("WE1");
        assertTrue(pruneService.getScheduledTasks().isEmpty());
        pruneService.schedulePruning(workflowExecution);
        assertEquals(1, pruneService.getScheduledTasks().size());
    }

    @Test
    public void schedulePruning_schedulePruningWaitUntilItIsPrunedCheckDeletionWasInvoked_deletionInvoked() throws NoSuchFieldException, InterruptedException {
        FieldSetter.setField(pruneService, pruneService.getClass().getDeclaredField("pruneAfter"), 0.000005);

        WorkflowExecution workflowExecution = new WorkflowExecution("WE3");

        assertTrue(pruneService.getScheduledTasks().isEmpty());

        pruneService.schedulePruning(workflowExecution);

        assertEquals(1, pruneService.getScheduledTasks().size());

        Thread.sleep(1000);

        verify(workflowExecutionDAO, times(1)).delete(workflowExecution);

        assertTrue(pruneService.getScheduledTasks().isEmpty());

        FieldSetter.setField(pruneService, pruneService.getClass().getDeclaredField("pruneAfter"), pruneAfter);
        FieldSetter.setField(pruneService, pruneService.getClass().getDeclaredField("pruneTasks"), new HashMap<>());
    }

    @Test
    public void updatePruneAfter_scheduleSomePruningsChangeValueCheckIfCorrectlyUpdated_scheduledFutureDelayUpdated() throws InterruptedException, NoSuchFieldException {
        WorkflowExecution workflowExecution1 = new WorkflowExecution("WE1");
        WorkflowExecution workflowExecution2 = new WorkflowExecution("WE2");

        when(workflowExecutionDAO.getAll()).thenReturn(Arrays.asList(workflowExecution1, workflowExecution2));

        assertTrue(pruneService.getScheduledTasks().isEmpty());

        pruneService.schedulePruning(workflowExecution1);
        pruneService.schedulePruning(workflowExecution2);

        assertEquals(2, pruneService.getScheduledTasks().size());

        long twoDaysDelayMax = (long) pruneAfter * 86400000; // two days
        long oneDayDelayMax = 86400000; // one day, the updated value

        Thread.sleep(1000);

        pruneService.getScheduledTasks().entrySet().forEach(entry -> {
            ScheduledFuture<?> scheduledFuture = entry.getValue();
            long delay = scheduledFuture.getDelay(TimeUnit.MILLISECONDS);
            assertTrue(delay < twoDaysDelayMax && delay > oneDayDelayMax);
        });

        pruneService.updateScheduledPruning(1);

        Thread.sleep(1000);

        pruneService.getScheduledTasks().entrySet().forEach(entry -> {
            ScheduledFuture<?> scheduledFuture = entry.getValue();
            long delay = scheduledFuture.getDelay(TimeUnit.MILLISECONDS);
            assertTrue(delay < oneDayDelayMax);
        });

        FieldSetter.setField(pruneService, pruneService.getClass().getDeclaredField("pruneTasks"), new HashMap<>());
    }

    @Test
    public void updatePruneAfter_scheduleSomePruningsSetNewPruneAfterSoThatTheyAreAutomaticallyPruned_twoPrunedNoneScheduledAfterUpdate() throws InterruptedException {
        WorkflowExecution workflowExecution1 = new WorkflowExecution("WE7");
        WorkflowExecution workflowExecution2 = new WorkflowExecution("WE8");

        when(workflowExecutionDAO.getAll()).thenReturn(Arrays.asList(workflowExecution1, workflowExecution2));

        assertTrue(pruneService.getScheduledTasks().isEmpty());

        pruneService.schedulePruning(workflowExecution1);
        pruneService.schedulePruning(workflowExecution2);

        assertEquals(2, pruneService.getScheduledTasks().size());

        long twoDaysDelayMax = (long) pruneAfter * 86400000; // two days

        Thread.sleep(1000);

        pruneService.getScheduledTasks().entrySet().forEach(entry -> {
            ScheduledFuture<?> scheduledFuture = entry.getValue();
            long delay = scheduledFuture.getDelay(TimeUnit.MILLISECONDS);
            assertTrue(delay < twoDaysDelayMax);
        });

        pruneService.updateScheduledPruning(0.000005);

        Thread.sleep(1000);

        verify(workflowExecutionDAO, times(1)).delete(workflowExecution1);
        verify(workflowExecutionDAO, times(1)).delete(workflowExecution2);

        assertTrue(pruneService.getScheduledTasks().isEmpty());
    }

}
