package ServiceTests;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Arrays;
import java.util.Collections;
import java.util.Properties;
import java.util.concurrent.TimeUnit;

import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.MockitoJUnit;
import org.mockito.junit.MockitoRule;
import org.mockito.quality.Strictness;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Import;
import org.springframework.context.annotation.PropertySource;
import org.springframework.scheduling.concurrent.ThreadPoolTaskScheduler;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringRunner;

import TestConfig.ServiceTestsConfiguration;
import eu.profinit.manta.bachelor.thesis.log.viewer.controllers.dto.SetExpiryCheckResponseDTO;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.WorkflowExecution;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.ConnectionsDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.ErrorDefinitionDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.ErrorMetadataDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.LogDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.PruneCheckingDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.ScenarioExecutionDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.WorkflowExecutionDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.PruneService;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.UpdateSettingsService;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.impl.PruneServiceImpl;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.impl.UpdateSettingsServiceImpl;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.mockito.ArgumentMatchers.anyDouble;
import static org.mockito.ArgumentMatchers.anyList;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.reset;
import static org.mockito.Mockito.when;

@RunWith(SpringRunner.class)
@ContextConfiguration(classes = {ServiceTestsConfiguration.class, UpdateSettingsServiceTests.UpdateSettingsConfig.class})
@ActiveProfiles({"test"})
public class UpdateSettingsServiceTests {

    @Configuration
    @PropertySource("classpath:testrepository.properties")
    @Import(ServiceTestsConfiguration.class)
    static class UpdateSettingsConfig {

        @Value("${logviewer.pruneAfter}")
        private double pruneAfter;

        @Value("${property.path}")
        private File propertyFile;

        @Bean
        public PruneService pruneService(ThreadPoolTaskScheduler threadPoolTaskScheduler){
            return new PruneServiceImpl(workflowExecutionDAO, pruneAfter, threadPoolTaskScheduler, errorDefinitionDAO, errorMetadataDAO, logDAO);
        }

        @Bean
        public UpdateSettingsService updateSettingsService(ThreadPoolTaskScheduler threadPoolTaskScheduler){
            return new UpdateSettingsServiceImpl(pruneService(threadPoolTaskScheduler), pruneCheckingDAO, propertyFile);
        }
    }

    @Rule
    public MockitoRule rule = MockitoJUnit.rule().strictness(Strictness.STRICT_STUBS);

    @Mock
    private static LogDAO logDAO;

    private static WorkflowExecutionDAO workflowExecutionDAO = getWorkflowMock();

    @Mock
    private static ScenarioExecutionDAO scenarioExecutionDAO;

    @Mock
    private static ErrorMetadataDAO errorMetadataDAO;

    @Mock
    private static ErrorDefinitionDAO errorDefinitionDAO;

    @Mock
    private static ConnectionsDAO connectionsDAO;

    @Mock
    private static PruneCheckingDAO pruneCheckingDAO;

    @Autowired
    @InjectMocks
    private PruneService pruneService;

    @Autowired
    @InjectMocks
    private UpdateSettingsService updateSettingsService;

    @Value("${property.path}")
    private String path;

    @Before
    public void before(){
        reset(logDAO);
        reset(workflowExecutionDAO);
        reset(scenarioExecutionDAO);
        reset(errorMetadataDAO);
        reset(errorDefinitionDAO);
        reset(connectionsDAO);
    }

    private static WorkflowExecutionDAO getWorkflowMock(){
        WorkflowExecutionDAO workflowExecutionDAO = mock(WorkflowExecutionDAO.class);
        when(workflowExecutionDAO.getAll()).thenReturn(Collections.emptyList());
        return workflowExecutionDAO;
    }

    @Test
    public void checkIfSomeWorkflowsWillBePruned_checkCorrectOutput_correct(){
        when(pruneCheckingDAO.getAmountOfPotentiallyDeletedLogs(anyDouble())).thenReturn(10);
        when(pruneCheckingDAO.getPotentiallyDeletedWorkflows(anyDouble())).thenReturn(Arrays.asList(new WorkflowExecution("WE1")));
        when(pruneCheckingDAO.getAmountOfPotentiallyDeletedWorkflowLogs(anyDouble(), anyList())).thenReturn(10);

        SetExpiryCheckResponseDTO setExpiryCheckResponseDTO = updateSettingsService.checkIfSomeLogsWillBePruned(0);

        assertEquals(10, setExpiryCheckResponseDTO.getNonWorkflowLogsPotentiallyPruned());
        assertEquals(1, setExpiryCheckResponseDTO.getWorkflowsPotentiallyPruned().size());
        assertEquals(10, setExpiryCheckResponseDTO.getWorkflowLogsPotentiallyPruned());
    }

    @Test
    public void updatePruningProperty_changePruningPropertyCheckCorrectBehaviour_propertyChangedPruningsRescheduled() throws IOException {
        File propertiesFile = new File(path);
        Properties props = new Properties();

        FileInputStream fileInputStream = new FileInputStream(propertiesFile);
        props.load(fileInputStream);
        fileInputStream.close();

        String oldPruningValue = (String) props.get("logviewer.pruneAfter");

        assertEquals(2, Double.parseDouble(oldPruningValue), 0.001);

        long twoDaysDelayMax = (long) 2 * 86400000; // two days
        long oneDayDelayMax = 86400000; // one day, the updated value

        pruneService.getScheduledTasks().forEach((key, value) -> {
            long delay = value.getDelay(TimeUnit.MILLISECONDS);
            assertTrue(delay < twoDaysDelayMax && delay > oneDayDelayMax);
        });

        updateSettingsService.updatePruningDelayProperty(1);

        Properties prop2 = new Properties();

        FileInputStream fileInputStream2 = new FileInputStream(propertiesFile);
        prop2.load(fileInputStream2);
        fileInputStream2.close();

        String newPruningValue = (String) prop2.get("logviewer.pruneAfter");

        assertEquals(1, Double.parseDouble(newPruningValue), 0.001);

        updateSettingsService.updatePruningDelayProperty(2);

        pruneService.getScheduledTasks().forEach((key, value) -> {
            long delay = value.getDelay(TimeUnit.MILLISECONDS);
            assertTrue(delay < oneDayDelayMax);
        });
    }

}
