package ServiceTests;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.time.Instant;
import java.util.Arrays;

import javax.sql.DataSource;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import org.apache.ibatis.session.SqlSessionFactory;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.Spy;
import org.mockito.junit.MockitoJUnit;
import org.mockito.junit.MockitoRule;
import org.mockito.quality.Strictness;
import org.mybatis.spring.SqlSessionFactoryBean;
import org.mybatis.spring.annotation.MapperScan;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Import;
import org.springframework.context.annotation.PropertySource;
import org.springframework.core.io.ClassPathResource;
import org.springframework.jdbc.datasource.DataSourceTransactionManager;
import org.springframework.jdbc.datasource.DriverManagerDataSource;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.transaction.annotation.EnableTransactionManagement;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import com.fasterxml.jackson.databind.ObjectMapper;

import TestConfig.DAOTestConfiguration;
import TestConfig.ServiceTestsConfiguration;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.ErrorDefinition;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.ErrorMetadata;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.UploadedPackage;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.UploadedLog;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.ErrorDefinitionDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.ErrorMetadataDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.PackageUploadDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.impl.ErrorDefinitionDAOImpl;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.impl.ErrorMetadataDAOImpl;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.impl.PackageUploadDAOImpl;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.mapper.ErrorDefinitionsMapper;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.mapper.ErrorMetadataMapper;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.mapper.PackageUploadMapper;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.UploadPackageService;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.exceptions.InvalidPackageFileException;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.impl.UploadPackageServiceImpl;

import static org.junit.Assert.assertNotNull;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.anyLong;
import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.ArgumentMatchers.eq;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;

@RunWith(SpringRunner.class)
@ContextConfiguration(classes = {ServiceTestsConfiguration.class, UploadPackageServiceTests.UploadPackageServiceConfig.class})
@ActiveProfiles({"test"})
@Transactional
public class UploadPackageServiceTests {

    @Configuration
    @PropertySource("classpath:testrepository.properties")
    @MapperScan("eu.profinit.manta.bachelor.thesis.log.viewer.repository.mapper")
    @Import({ServiceTestsConfiguration.class, DAOTestConfiguration.class})
    @EnableTransactionManagement
    static class UploadPackageServiceConfig {

        @Value("${spring.datasource.test.driver}")
        private String datasourceDriver;

        @Value("${spring.datasource.test.url}")
        private String datasourceUrl;

        @Value("${spring.datasource.test.username}")
        private String datasourceUsername;

        @Value("${spring.datasource.test.password}")
        private String datasourcePassword;

        @Bean
        public UploadPackageService uploadPackageService(PackageUploadMapper packageUploadMapper, ErrorDefinitionsMapper errorDefinitionsMapper,
                                                         ErrorMetadataMapper errorMetadataMapper, ObjectMapper objectMapper, @Value("${packaging.upload.directory}") String uploadDirectory){
            return new UploadPackageServiceImpl(packageUploadDAO(packageUploadMapper), objectMapper, uploadDirectory, errorDefinitionDAO(errorDefinitionsMapper), errorMetadataDAO(errorMetadataMapper));
        }

        @Bean
        public PackageUploadDAO packageUploadDAO(PackageUploadMapper packageUploadMapper){
            return new PackageUploadDAOImpl(packageUploadMapper);
        }

        @Bean
        public ErrorDefinitionDAO errorDefinitionDAO(ErrorDefinitionsMapper errorDefinitionsMapper){
            return new ErrorDefinitionDAOImpl(errorDefinitionsMapper);
        }

        @Bean
        public ErrorMetadataDAO errorMetadataDAO(ErrorMetadataMapper errorMetadataMapper){
            return new ErrorMetadataDAOImpl(errorMetadataMapper);
        }
    }

    @Rule
    public MockitoRule rule = MockitoJUnit.rule().strictness(Strictness.STRICT_STUBS);

    @Spy
    @Autowired
    private PackageUploadDAO packageUploadDAO;

    @Spy
    @Autowired
    private ErrorDefinitionDAO errorDefinitionDAO;

    @Spy
    @Autowired
    private ErrorMetadataDAO errorMetadataDAO;

    @Value("classpath:applicationPackage.zip")
    private File applicationPackage;

    @Value("classpath:connectionPackage.zip")
    private File connectionPackage;

    @Value("${packaging.upload.directory}")
    private String uploadDirectory;

    @Autowired
    @InjectMocks
    private UploadPackageService uploadPackageService;

    @Before
    public void before() throws IOException {
        File directory = new File("./target/test-data/temp");
        FileUtils.deleteDirectory(directory);
    }

    @Test(expected = InvalidPackageFileException.class)
    public void uploadPackage_passNullFile_shouldThrowException(){
        uploadPackageService.uploadPackage(null, "My Package");
    }

    @Test
    public void uploadApplicationPackage_passCorrectFile_everythingShouldWork() throws IOException {
        assertNotNull(applicationPackage);

        FileInputStream fileInputStream = new FileInputStream(applicationPackage);
        MultipartFile multiPartFile = new MockMultipartFile("file", applicationPackage.getName(), "application/zip", IOUtils.toByteArray(fileInputStream));

        String packageName = "My Package";

        uploadPackageService.uploadPackage(multiPartFile, packageName);

        verify(packageUploadDAO, times(1)).save(eq(new UploadedPackage(packageName, "Manta Server")));
        verify(packageUploadDAO, times(20)).upload(any(UploadedLog.class), anyLong(), eq(packageName));
        verify(packageUploadDAO, times(100)).upload(any(UploadedLog.class), eq(packageName));
        verify(errorDefinitionDAO, times(10)).save(any(ErrorDefinition.class));
        verify(errorMetadataDAO, times(10)).save(any(ErrorMetadata.class));
    }

    @Test
    public void uploadConnectionPackage_passCorrectFile_everythingShouldWork() throws IOException {
        assertNotNull(connectionPackage);

        FileInputStream fileInputStream = new FileInputStream(connectionPackage);
        MultipartFile multiPartFile = new MockMultipartFile("file", connectionPackage.getName(), "application/zip", IOUtils.toByteArray(fileInputStream));

        String packageName = "My Package";

        uploadPackageService.uploadPackage(multiPartFile, packageName);

        verify(packageUploadDAO, times(1)).save(eq(new UploadedPackage(packageName, "Oracle", "Analysis", "Oracle DDL Analysis0", "OracleTestDB0")));
        verify(packageUploadDAO, times(20)).upload(any(UploadedLog.class), anyLong(), eq(packageName));
        verify(packageUploadDAO, times(100)).upload(any(UploadedLog.class), eq(packageName));
        verify(errorDefinitionDAO, times(10)).save(any(ErrorDefinition.class));
        verify(errorMetadataDAO, times(10)).save(any(ErrorMetadata.class));
    }
}
