package TransportLayerTests;

import java.lang.reflect.InvocationTargetException;

import javax.jms.JMSException;
import javax.jms.TextMessage;

import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.ArgumentCaptor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.Spy;
import org.mockito.invocation.InvocationOnMock;
import org.mockito.junit.MockitoJUnit;
import org.mockito.junit.MockitoRule;
import org.mockito.quality.Strictness;
import org.mockito.stubbing.Answer;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringRunner;

import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;

import eu.profinit.manta.bachelor.thesis.log.viewer.broker.consumer.LogResolver;
import eu.profinit.manta.bachelor.thesis.log.viewer.broker.consumer.LogResolverImpl;
import eu.profinit.manta.bachelor.thesis.log.viewer.broker.consumer.UnableToResolveException;
import eu.profinit.manta.bachelor.thesis.log.viewer.broker.listener.LogListener;
import eu.profinit.manta.bachelor.thesis.log.viewer.broker.messagehandling.MessageHandler;
import eu.profinit.manta.bachelor.thesis.log.viewer.broker.messagehandling.MessageHandlerImpl;
import eu.profinit.manta.bachelor.thesis.log.viewer.broker.messagehandling.handlers.LogHandler;
import eu.profinit.manta.bachelor.thesis.log.viewer.broker.messagehandling.handlers.LogHandlerImpl;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.DebugLog;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.ErrorLog;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.Log;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.LogPersistenceService;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;


@RunWith(SpringRunner.class)
@ContextConfiguration(classes = {TransportLayerTests.TransportLayerTestsConfig.class})
@ActiveProfiles({"test"})
public class TransportLayerTests {

    @Configuration
    static class TransportLayerTestsConfig {
        @Bean
        LogListener logListener(){
            return new LogListener(logResolver(), messageHandler());
        }

        @Bean
        LogResolver logResolver(){
            return new LogResolverImpl(objectMapper());
        }

        @Bean
        public ObjectMapper objectMapper() {
            ObjectMapper objectMapper = new ObjectMapper();
            objectMapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
            return objectMapper;
        }

        @Bean
        public MessageHandler messageHandler(){
            return new MessageHandlerImpl(logHandler());
        }

        @Bean
        public LogHandler logHandler(){
            return new LogHandlerImpl(logPersistenceService);
        }
    }

    @Autowired
    @InjectMocks
    private LogListener logListener;

    @Rule
    public MockitoRule rule = MockitoJUnit.rule().strictness(Strictness.STRICT_STUBS);

    @Spy
    @Autowired
    LogResolver logResolver;

    @Autowired
    @InjectMocks
    LogHandler logHandler;

    @Mock
    static LogPersistenceService logPersistenceService;

    @Autowired
    MessageHandler messageHandler;

    @Test
    public void handleMessage_passMessageResolveItAndHandleIt_logPersistedCorrectly(){
        String jsonLog =  "{\n" +
                "  \"thread\" : \"main\",\n" +
                "  \"level\" : \"INFO\",\n" +
                "  \"loggerName\" : \"Producer\",\n" +
                "  \"marker\" : {\n" +
                "    \"name\" : \"DEBUG\"\n" +
                "  },\n" +
                "  \"message\" : \"Test message.\",\n" +
                "  \"endOfBatch\" : false,\n" +
                "  \"loggerFqcn\" : \"org.apache.logging.log4j.spi.AbstractLogger\",\n" +
                "  \"instant\" : {\n" +
                "    \"epochSecond\" : 1589103392,\n" +
                "    \"nanoOfSecond\" : 13000000\n" +
                "  },\n" +
                "  \"contextMap\" : {" +
                "\"CONNECTION_ID\" : \"OracleTestDB\",\n" +
                "    \"PHASE\" : \"Analysis\",\n" +
                "    \"TECHNOLOGY\" : \"Oracle\"\n" +
                "  },\n" +
                "  \"threadId\" : 1,\n" +
                "  \"threadPriority\" : 5,\n" +
                "  \"type\" : \"DEBUG\",\n" +
                "  \"APPLICATION\" : \"Manta CLI\"\n" +
                "}\n";

        ArgumentCaptor<DebugLog> passedLog = ArgumentCaptor.forClass(DebugLog.class);

        logListener.handleMessage(jsonLog);

        verify(logPersistenceService, times(1)).persistDebugLog(any(DebugLog.class));
        verify(logPersistenceService).persistDebugLog(passedLog.capture());

        DebugLog log = passedLog.getValue();

        assertEquals("main", log.getThread());
        assertEquals("INFO", log.getLevel());
        assertEquals("Producer", log.getLoggerName());
        assertEquals("Test message.", log.getMessage());
        assertEquals(1589103392, log.getEpochSecond());
        assertEquals(13000000, log.getNanoSecond());
        assertEquals("OracleTestDB", log.getConnectionId());
        assertEquals("Analysis", log.getPhase());
        assertEquals("Oracle", log.getTechnology());
        assertEquals(1, log.getThreadId());
        assertEquals("DEBUG", log.getType());
        assertEquals("Manta CLI", log.getApplication());
    }

    @Test
    public void logResolver_passDebugLogAsItWouldCome_correctlyResolved() throws UnableToResolveException {
        String jsonLog =  "{\n" +
                "  \"thread\" : \"main\",\n" +
                "  \"level\" : \"INFO\",\n" +
                "  \"loggerName\" : \"Producer\",\n" +
                "  \"marker\" : {\n" +
                "    \"name\" : \"DEBUG\"\n" +
                "  },\n" +
                "  \"message\" : \"Test message.\",\n" +
                "  \"endOfBatch\" : false,\n" +
                "  \"loggerFqcn\" : \"org.apache.logging.log4j.spi.AbstractLogger\",\n" +
                "  \"instant\" : {\n" +
                "    \"epochSecond\" : 1589103392,\n" +
                "    \"nanoOfSecond\" : 13000000\n" +
                "  },\n" +
                "  \"contextMap\" : {" +
                "\"CONNECTION_ID\" : \"OracleTestDB\",\n" +
                "    \"PHASE\" : \"Analysis\",\n" +
                "    \"TECHNOLOGY\" : \"Oracle\"\n" +
                "  },\n" +
                "  \"threadId\" : 1,\n" +
                "  \"threadPriority\" : 5,\n" +
                "  \"type\" : \"DEBUG\",\n" +
                "  \"APPLICATION\" : \"Manta CLI\"\n" +
                "}\n";

        Log log = logResolver.resolve(jsonLog);

        assertEquals("main", log.getThread());
        assertEquals("INFO", log.getLevel());
        assertEquals("Producer", log.getLoggerName());
        assertEquals("Test message.", log.getMessage());
        assertEquals(1589103392, log.getEpochSecond());
        assertEquals(13000000, log.getNanoSecond());
        assertEquals("OracleTestDB", log.getConnectionId());
        assertEquals("Analysis", log.getPhase());
        assertEquals("Oracle", log.getTechnology());
        assertEquals(1, log.getThreadId());
        assertEquals("DEBUG", log.getType());
        assertEquals("Manta CLI", log.getApplication());
    }

    @Test
    public void logResolver_passErrorLogAsItWouldNormallyCome_correctlyResolved() throws UnableToResolveException {
        String jsonLog =  "{\n" +
                "  \"thread\" : \"main\",\n" +
                "  \"level\" : \"ERROR\",\n" +
                "  \"loggerName\" : \"Producer\",\n" +
                "  \"marker\" : {\n" +
                "    \"name\" : \"ERROR\"\n" +
                "  },\n" +
                "  \"message\" : \"\",\n" +
                "  \"endOfBatch\" : false,\n" +
                "  \"loggerFqcn\" : \"org.apache.logging.log4j.spi.AbstractLogger\",\n" +
                "  \"instant\" : {\n" +
                "    \"epochSecond\" : 1589104642,\n" +
                "    \"nanoOfSecond\" : 901000000\n" +
                "  },\n" +
                "  \"contextMap\" : {\n" +
                "    \"CATEGORY\" : \"TEST_ERRORS\",\n" +
                "    \"CONNECTION_ID\" : \"OracleTestDB\",\n" +
                "    \"ERROR_TYPE\" : \"TEST_ERROR\"," +
                "\"IMPACT\" : \"UNDEFINED\",\n" +
                "    \"PHASE\" : \"Analysis\",\n" +
                "    \"SOLUTION\" : \"Test solution.\",\n" +
                "    \"TECHNICAL_MESSAGE\" : \"Test technical message.\",\n" +
                "    \"TECHNOLOGY\" : \"Oracle\",\n" +
                "    \"USER_MESSAGE\" : \"Test user message.\"\n" +
                "  },\n" +
                "  \"threadId\" : 1,\n" +
                "  \"threadPriority\" : 5,\n" +
                "  \"type\" : \"ERROR\",\n" +
                "  \"APPLICATION\" : \"Manta CLI\"\n" +
                "}";

        ErrorLog log = (ErrorLog) logResolver.resolve(jsonLog);

        assertEquals("main", log.getThread());
        assertEquals("ERROR", log.getLevel());
        assertEquals("Producer", log.getLoggerName());
        assertTrue(log.getMessage().isEmpty());
        assertEquals(1589104642, log.getEpochSecond());
        assertEquals(901000000, log.getNanoSecond());
        assertEquals("OracleTestDB", log.getConnectionId());
        assertEquals("Analysis", log.getPhase());
        assertEquals("Oracle", log.getTechnology());
        assertEquals(1, log.getThreadId());
        assertEquals("ERROR", log.getType());
        assertEquals("TEST_ERRORS", log.getCategory());
        assertEquals("TEST_ERROR", log.getErrorType());
        assertEquals("UNDEFINED", log.getLineageImpact());
        assertEquals("Test solution.", log.getSolution());
        assertEquals("Test technical message.", log.getTechnicalMessage());
        assertEquals("Test user message.", log.getUserMessage());
    }

    @Test
    public void resolveLog_passUnknownType_resolvedAsDebug() throws UnableToResolveException {
        String jsonLog =  "{\n" +
                "  \"thread\" : \"main\",\n" +
                "  \"level\" : \"ERROR\",\n" +
                "  \"loggerName\" : \"Producer\",\n" +
                "  \"marker\" : {\n" +
                "    \"name\" : \"ERROR\"\n" +
                "  },\n" +
                "  \"message\" : \"\",\n" +
                "  \"endOfBatch\" : false,\n" +
                "  \"loggerFqcn\" : \"org.apache.logging.log4j.spi.AbstractLogger\",\n" +
                "  \"instant\" : {\n" +
                "    \"epochSecond\" : 1589104642,\n" +
                "    \"nanoOfSecond\" : 901000000\n" +
                "  },\n" +
                "  \"contextMap\" : {\n" +
                "    \"CATEGORY\" : \"TEST_ERRORS\",\n" +
                "    \"CONNECTION_ID\" : \"OracleTestDB\",\n" +
                "    \"ERROR_TYPE\" : \"TEST_ERROR\"," +
                "\"IMPACT\" : \"UNDEFINED\",\n" +
                "    \"PHASE\" : \"Analysis\",\n" +
                "    \"SOLUTION\" : \"Test solution.\",\n" +
                "    \"TECHNICAL_MESSAGE\" : \"Test technical message.\",\n" +
                "    \"TECHNOLOGY\" : \"Oracle\",\n" +
                "    \"USER_MESSAGE\" : \"Test user message.\"\n" +
                "  },\n" +
                "  \"threadId\" : 1,\n" +
                "  \"threadPriority\" : 5,\n" +
                "  \"type\" : \"SOME TYPE THAT DOES NOT EXIST\",\n" +
                "  \"APPLICATION\" : \"Manta CLI\"\n" +
                "}";

        Log log = logResolver.resolve(jsonLog);

        assertEquals("DEBUG", log.getType());
    }

    @Test
    public void handleLog_checkIfPersistenceCalled() throws UnableToResolveException {
        String jsonLog =  "{\n" +
                "  \"thread\" : \"main\",\n" +
                "  \"level\" : \"ERROR\",\n" +
                "  \"loggerName\" : \"Producer\",\n" +
                "  \"marker\" : {\n" +
                "    \"name\" : \"ERROR\"\n" +
                "  },\n" +
                "  \"message\" : \"\",\n" +
                "  \"endOfBatch\" : false,\n" +
                "  \"loggerFqcn\" : \"org.apache.logging.log4j.spi.AbstractLogger\",\n" +
                "  \"instant\" : {\n" +
                "    \"epochSecond\" : 1589104642,\n" +
                "    \"nanoOfSecond\" : 901000000\n" +
                "  },\n" +
                "  \"contextMap\" : {\n" +
                "    \"CATEGORY\" : \"TEST_ERRORS\",\n" +
                "    \"CONNECTION_ID\" : \"OracleTestDB\",\n" +
                "    \"ERROR_TYPE\" : \"TEST_ERROR\"," +
                "\"IMPACT\" : \"UNDEFINED\",\n" +
                "    \"PHASE\" : \"Analysis\",\n" +
                "    \"SOLUTION\" : \"Test solution.\",\n" +
                "    \"TECHNICAL_MESSAGE\" : \"Test technical message.\",\n" +
                "    \"TECHNOLOGY\" : \"Oracle\",\n" +
                "    \"USER_MESSAGE\" : \"Test user message.\"\n" +
                "  },\n" +
                "  \"threadId\" : 1,\n" +
                "  \"threadPriority\" : 5,\n" +
                "  \"type\" : \"ERROR\",\n" +
                "  \"APPLICATION\" : \"Manta CLI\"\n" +
                "}";

        ErrorLog log = (ErrorLog) logResolver.resolve(jsonLog);

        log.handle(messageHandler);

        verify(logPersistenceService, times(1)).persistErrorLog(log);
    }

}
