CREATE TABLE IF NOT EXISTS workflow_executions
(
    workflow_execution_id VARCHAR(100) NOT NULL CONSTRAINT workflow_execution_id_pk PRIMARY KEY,
    execution_timestamp TIMESTAMP NOT NULL
);

CREATE TABLE IF NOT EXISTS scenario_executions
(
    scenario_execution_id VARCHAR(100) NOT NULL CONSTRAINT scenario_execution_id_pk PRIMARY KEY,
    workflow_execution_id VARCHAR(100) NOT NULL,
    technology VARCHAR(100) NOT NULL,
    phase VARCHAR(100) NOT NULL,
    scenario_name VARCHAR(100) NOT NULL,
    CONSTRAINT scenarios_workflow_execution_id_fk FOREIGN KEY (workflow_execution_id) REFERENCES workflow_executions(workflow_execution_id) ON DELETE CASCADE
);

CREATE TABLE IF NOT EXISTS uploaded_packages
(
    package_name VARCHAR(100) NOT NULL CONSTRAINT package_id_pk PRIMARY KEY,
    technology VARCHAR(100) NULL,
    phase VARCHAR(100) NULL,
    scenario_name VARCHAR(100) NULL,
    connection VARCHAR(100) NULL,
    application VARCHAR(100) NULL
);

CREATE TABLE IF NOT EXISTS connections
(
    conn_db_id IDENTITY NOT NULL CONSTRAINT conn_id_pk PRIMARY KEY,
    connection_id VARCHAR(200) NOT NULL,
    scenario_execution_id VARCHAR(100) NULL,
    package_name VARCHAR(100) NULL,
    CONSTRAINT conn_scenario_execution_id_fk FOREIGN KEY (scenario_execution_id) REFERENCES scenario_executions(scenario_execution_id) ON DELETE CASCADE,
    CONSTRAINT conn_package_fk FOREIGN KEY (package_name) REFERENCES uploaded_packages(package_name) ON DELETE CASCADE
);

CREATE TABLE IF NOT EXISTS error_definitions
(
    error_definition_id IDENTITY NOT NULL CONSTRAINT error_definition_id_pk PRIMARY KEY,
    error_category VARCHAR(100) NOT NULL,
    error_type VARCHAR(100) NOT NULL,
    user_message VARCHAR NOT NULL,
    technical_message VARCHAR NOT NULL,
    solution VARCHAR NOT NULL,
    impact VARCHAR(100) NOT NULL,
    log_level VARCHAR(25) NOT NULL
);

CREATE TABLE IF NOT EXISTS error_metadata
(
    error_id IDENTITY NOT NULL CONSTRAINT error_id_pk PRIMARY KEY,
    error_definition_id BIGINT NOT NULL,
    workflow_execution_id VARCHAR(100) NULL,
    scenario_execution_id VARCHAR(100) NULL,
    package_name VARCHAR(100) NULL,
    conn_db_id BIGINT NULL,
    application VARCHAR(50) NOT NULL,
    CONSTRAINT error_definition_id FOREIGN KEY (error_definition_id) REFERENCES error_definitions(error_definition_id) ON DELETE CASCADE,
    CONSTRAINT err_workflow_execution_id FOREIGN KEY (workflow_execution_id) REFERENCES workflow_executions(workflow_execution_id) ON DELETE CASCADE,
    CONSTRAINT err_scenario_execution_id_fk FOREIGN KEY (scenario_execution_id) REFERENCES scenario_executions(scenario_execution_id) ON DELETE CASCADE,
    CONSTRAINT err_conn_id_fk FOREIGN KEY (conn_db_id) REFERENCES connections(conn_db_id) ON DELETE CASCADE,
    CONSTRAINT err_package_name_fk FOREIGN KEY (package_name) REFERENCES uploaded_packages(package_name) ON DELETE CASCADE
);

CREATE TABLE IF NOT EXISTS logs
(
    log_id IDENTITY NOT NULL CONSTRAINT log_id_pk PRIMARY KEY,
    type VARCHAR(5) NOT NULL,
    workflow_execution_id VARCHAR(100) NULL,
    scenario_execution_id VARCHAR(100) NULL,
    package_name VARCHAR(100) NULL,
    conn_db_id BIGINT NULL,
    application VARCHAR(50) NOT NULL,
    context VARCHAR(200) NULL,
    thread VARCHAR(25) NOT NULL,
    thread_id INTEGER NOT NULL,
    logger_name VARCHAR NOT NULL,
    worker_number INTEGER NULL,
    log_level VARCHAR(25) NOT NULL,
    epoch_second BIGINT NOT NULL,
    nano_second BIGINT NOT NULL,
    message VARCHAR NULL,
    exception_name VARCHAR NULL,
    exception_stacktrace VARCHAR NULL,
    error_id BIGINT NULL,
    CONSTRAINT workflow_execution_id_fk FOREIGN KEY (workflow_execution_id) REFERENCES workflow_executions(workflow_execution_id) ON DELETE CASCADE,
    CONSTRAINT scenario_execution_id_fk FOREIGN KEY (scenario_execution_id) REFERENCES scenario_executions(scenario_execution_id) ON DELETE CASCADE,
    CONSTRAINT conn_id_fk FOREIGN KEY (conn_db_id) REFERENCES connections(conn_db_id) ON DELETE CASCADE,
    CONSTRAINT package_name_fk FOREIGN KEY (package_name) REFERENCES uploaded_packages(package_name) ON DELETE CASCADE,
    CONSTRAINT error_id_fk FOREIGN KEY (error_id) REFERENCES error_metadata(error_id) ON DELETE CASCADE
);

-- init full text indexing
CREATE ALIAS IF NOT EXISTS FTL_INIT FOR "org.h2.fulltext.FullTextLucene.init";
CALL FTL_INIT();

-- create index on log messages
CALL FTL_CREATE_INDEX('PUBLIC', 'ERROR_DEFINITIONS', 'USER_MESSAGE,TECHNICAL_MESSAGE');

CREATE INDEX IF NOT EXISTS logs_context_index ON logs(workflow_execution_id, scenario_execution_id, conn_db_id, thread_id, epoch_second DESC NULLS LAST, nano_second DESC NULLS LAST, log_id DESC NULLS LAST);
CREATE INDEX IF NOT EXISTS logs_context_reverse_index ON logs(workflow_execution_id, scenario_execution_id, conn_db_id, thread_id, epoch_second ASC NULLS LAST, nano_second ASC NULLS LAST, log_id ASC NULLS LAST);
CREATE INDEX IF NOT EXISTS applications_index ON logs(application);
CREATE INDEX IF NOT EXISTS error_metadata_scenario_executions_index ON error_metadata(scenario_execution_id, error_definition_id, error_id);
CREATE INDEX IF NOT EXISTS logs_error_id_index ON logs(error_id);

CREATE INDEX IF NOT EXISTS error_types_index ON error_definitions(error_type, error_definition_id);
CREATE INDEX IF NOT EXISTS error_types_index ON error_definitions(error_category, error_type, error_definition_id);

CREATE INDEX IF NOT EXISTS logs_scenario_executions_index ON logs(scenario_execution_id, error_id);
CREATE INDEX IF NOT EXISTS logs_workflow_executions_index ON logs(workflow_execution_id, error_id);
CREATE INDEX IF NOT EXISTS logs_package_index ON logs(package_name, error_id);
CREATE INDEX IF NOT EXISTS logs_connections_index ON logs(conn_db_id, error_id);

CREATE INDEX IF NOT EXISTS error_metadata_scenario_executions_index ON error_metadata(scenario_execution_id, error_id);
CREATE INDEX IF NOT EXISTS error_metadata_workflow_executions_index ON error_metadata(workflow_execution_id, error_id);
CREATE INDEX IF NOT EXISTS error_metadata_package_index ON error_metadata(package_name, error_id);
CREATE INDEX IF NOT EXISTS error_metadata_connections_index ON error_metadata(conn_db_id, error_id);