package eu.profinit.manta.bachelor.thesis.log.viewer.broker.config;

import java.util.HashMap;
import java.util.Map;

import javax.jms.ConnectionFactory;

import org.apache.activemq.artemis.api.core.TransportConfiguration;
import org.apache.activemq.artemis.api.jms.ActiveMQJMSClient;
import org.apache.activemq.artemis.api.jms.JMSFactoryType;
import org.apache.activemq.artemis.core.remoting.impl.netty.NettyConnectorFactory;
import org.apache.activemq.artemis.core.remoting.impl.netty.TransportConstants;
import org.apache.activemq.artemis.jms.client.ActiveMQConnectionFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.PropertySource;
import org.springframework.jms.annotation.EnableJms;
import org.springframework.jms.config.DefaultJmsListenerContainerFactory;
import org.springframework.util.ErrorHandler;

import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;

import eu.profinit.manta.bachelor.thesis.log.viewer.errors.LogConsumerErrorHandler;

@Configuration
@PropertySource("classpath:transport.properties")
@ComponentScan("eu.profinit.manta.bachelor.thesis.log.viewer.logic.broker")
@EnableJms
public class ConsumerConfig {

    /**
     * Port of the embedded ActiveMQ Artemis server to which consumer is connecting.
     */
    @Value("${artemis.consumer.port}")
    private int port;

    /**
     * Addres of the embedded ActiveMQ Artemis server to which consumer is connecting.
     */
    @Value("${artemis.host}")
    private String host;

    /**
     * Name of the designated logging queue.
     */
    @Value("${artemis.consumer.loggingQueue}")
    private String loggingQueue;

    /**
     * Bean representing ActiveMQ connection factory.
     * @return JMS connection factory.
     */
    @Bean
    ConnectionFactory activeMQJMSConnectionFactory(){
        Map<String, Object> connectionParams = new HashMap<>();
        connectionParams.put(TransportConstants.HOST_PROP_NAME, host);
        connectionParams.put(TransportConstants.PORT_PROP_NAME, port);

        TransportConfiguration transportConfiguration =
                new TransportConfiguration(NettyConnectorFactory.class.getName(), connectionParams);

        ActiveMQConnectionFactory connectionFactory =
                ActiveMQJMSClient.createConnectionFactoryWithoutHA(JMSFactoryType.CF, transportConfiguration);

        return connectionFactory;
    }

    /**
     * Bean representing error handler for the log consumer.
     * @return Error handler
     */
    @Bean
    public ErrorHandler logConsumerErrorHandler(){
        return new LogConsumerErrorHandler();
    }

    /**
     * Bean representing JMS listener container, wrapping connecting to JMS, provided by Spring.
     * @param activeMQJMSConnectionFactory Connection factory bean
     * @param logConsumerErrorHandler Error handler bean
     * @return JMS listener container
     */
    @Bean
    public DefaultJmsListenerContainerFactory jmsListenerContainerFactory(ConnectionFactory activeMQJMSConnectionFactory, ErrorHandler logConsumerErrorHandler){
        DefaultJmsListenerContainerFactory factory = new DefaultJmsListenerContainerFactory();
        factory.setConnectionFactory(activeMQJMSConnectionFactory);
        factory.setErrorHandler(logConsumerErrorHandler);
        return factory;
    }

    /**
     * Jackson object mapper bean
     * @return object mapper
     */
    @Bean
    public ObjectMapper objectMapper(){
        ObjectMapper objectMapper = new ObjectMapper();
        objectMapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
        return objectMapper;
    }

    /**
     * Returns name of the designated logging queue.
     * @return name of the logging queue
     */
    public String getLoggingQueue() {
        return loggingQueue;
    }
}
