package eu.profinit.manta.bachelor.thesis.log.viewer.broker.config;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.activemq.artemis.api.core.RoutingType;
import org.apache.activemq.artemis.api.core.TransportConfiguration;
import org.apache.activemq.artemis.core.config.CoreAddressConfiguration;
import org.apache.activemq.artemis.core.config.CoreQueueConfiguration;
import org.apache.activemq.artemis.core.remoting.impl.netty.NettyAcceptorFactory;
import org.apache.activemq.artemis.jms.server.config.JMSConfiguration;
import org.apache.activemq.artemis.jms.server.config.JMSQueueConfiguration;
import org.apache.activemq.artemis.jms.server.config.impl.JMSConfigurationImpl;
import org.apache.activemq.artemis.jms.server.config.impl.JMSQueueConfigurationImpl;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;
import org.apache.activemq.artemis.core.server.embedded.EmbeddedActiveMQ;
import org.apache.activemq.artemis.core.config.impl.ConfigurationImpl;
import org.springframework.context.annotation.PropertySource;
import org.apache.activemq.artemis.core.remoting.impl.netty.TransportConstants;

import eu.profinit.manta.bachelor.thesis.logging.api.logging.Logger;

/**
 * Configuration for an embedded ActiveMQ Artemis server.
 * @author jkovac
 */
@Configuration
@ComponentScan("eu.profinit.manta.bachelor.thesis.log.viewer.logic.broker")
@PropertySource("classpath:transport.properties")
public class EmbeddedServerConfig {

    /**
     * Logger
     */
    private final Logger logger = new Logger(EmbeddedServerConfig.class);

    /**
     * Port on which the embedded server will be listening.
     */
    @Value("${artemis.server.port}")
    private int serverPort;

    /**
     * Port for non-secured connection. Used by the local consumer.
     */
    @Value("${artemis.consumer.port}")
    private int consumerPort;

    /**
     * Address of the embedded server (should be localhost).
     */
    @Value("${artemis.host}")
    private String host;

    /**
     * Path to the server keystore.
     */
    @Value("WEB-INF/classes/logviewerkeystore")
    private File keyStoreFile;

    /**
     * Keystore password
     */
    @Value("${encryption.keystore.password}")
    private String keyStorePassword;

    /**
     * Further configured queues beyond the logging queue.
     */
    @Value("${artemis.consumer.queues}")
    private String[] queues;

    /**
     * Name of the designated logging queue.
     */
    @Value("${artemis.consumer.loggingQueue}")
    private String loggingQueue;

    /**
     * SSL communication enabled by default.
     */
    private boolean sslEnabled = true;

    /**
     * Bean representing configuration of the embedded server.
     * @return server configuration
     */
    @Bean
    org.apache.activemq.artemis.core.config.Configuration configuration(){
        String logDirectory = getLogsDirectory();
        String keyStorePath = keyStoreFile.getAbsolutePath();

        Map<String, Object> sslConnectionParams = new HashMap<>();
        sslConnectionParams.put(TransportConstants.HOST_PROP_NAME, host);
        sslConnectionParams.put(TransportConstants.PORT_PROP_NAME, serverPort);
        sslConnectionParams.put(TransportConstants.SSL_ENABLED_PROP_NAME, sslEnabled);
        sslConnectionParams.put(TransportConstants.KEYSTORE_PATH_PROP_NAME, keyStorePath);
        sslConnectionParams.put(TransportConstants.KEYSTORE_PASSWORD_PROP_NAME, keyStorePassword);

        TransportConfiguration sslTransportConfiguration =
                new TransportConfiguration(NettyAcceptorFactory.class.getName(), sslConnectionParams);

        /*
         * Non-SSL secured port for local consumer or non-secured connections.
         */
        Map<String, Object> normalConnectionParams = new HashMap<>();
        normalConnectionParams.put(TransportConstants.HOST_PROP_NAME, host);
        normalConnectionParams.put(TransportConstants.PORT_PROP_NAME, consumerPort);

        TransportConfiguration normalTransportConfiguration =
                new TransportConfiguration(NettyAcceptorFactory.class.getName(), normalConnectionParams);

        ConfigurationImpl configuration = new ConfigurationImpl()
                .setSecurityEnabled(false)
                .setPersistenceEnabled(false)
                .setJournalDirectory(logDirectory)
                .addAcceptorConfiguration(sslTransportConfiguration)
                .addAcceptorConfiguration(normalTransportConfiguration)
                ;

        return configuration;
    }

    /**
     * JMS configuration bean
     * @return JMS configuration
     */
    @Bean
    JMSConfiguration jmsConfiguration(){
        JMSConfiguration configuration = new JMSConfigurationImpl();

        List<JMSQueueConfiguration> queueConfigurations = new ArrayList<>();

        Arrays.asList(queues).forEach(queue -> {
                JMSQueueConfiguration jmsQueue = new JMSQueueConfigurationImpl()
                        .setName(queue);
                queueConfigurations.add(jmsQueue);
        });

        JMSQueueConfiguration jmsLoggingQueue =
                new JMSQueueConfigurationImpl().setName(loggingQueue);
        queueConfigurations.add(jmsLoggingQueue);

        configuration.setQueueConfigurations(queueConfigurations);

        return configuration;
    }

    /**
     * Bean representing the embedded ActiveMQ Artemis server.
     * @param configuration Server configration
     * @param jmsConfiguration JMS configuration
     * @return embedded server instance
     */
    @Bean(initMethod = "start", destroyMethod = "stop")
    EmbeddedActiveMQ embeddedActiveMQ(org.apache.activemq.artemis.core.config.Configuration configuration,
                                      JMSConfiguration jmsConfiguration){

        jmsConfiguration.getQueueConfigurations().forEach(queue -> {
            configuration.addAddressConfiguration(createCoreAddressConfig(queue));
            logger.info("Creating queue: {}", queue.getName());
        });

        EmbeddedActiveMQ activeMQ = new EmbeddedActiveMQ();
        activeMQ.setConfiguration(configuration);

        logger.info("Created ActiveMQ Artemis server, configured at tcp://{}:{}.", host, serverPort);

        return activeMQ;
    }

    /**
     * Creates core address configuration for the given queue.
     * @param configuration Configuration of the JMS queue.
     * @return configuration of the core address
     */
    private CoreAddressConfiguration createCoreAddressConfig(JMSQueueConfiguration configuration){
        CoreAddressConfiguration coreAddressConfiguration = new CoreAddressConfiguration()
                .setName(configuration.getName())
                .addRoutingType(RoutingType.ANYCAST)
                .addQueueConfiguration(createCoreQueueConfig(configuration));
        return coreAddressConfiguration;
    }

    /**
     * Creates core queue configuration for the given queue.
     * @param configuration Configuration of the JMS queue.
     * @return configuration of the core queue
     */
    private CoreQueueConfiguration createCoreQueueConfig(JMSQueueConfiguration configuration){
        CoreQueueConfiguration coreQueueConfiguration = new CoreQueueConfiguration()
                .setAddress(configuration.getName())
                .setName(configuration.getName())
                .setDurable(configuration.isDurable())
                .setRoutingType(RoutingType.ANYCAST);
        return coreQueueConfiguration;
    }

    /**
     * Returns logs directory
     * @return path to the logs directory
     */
    private String getLogsDirectory(){
        String catalinaHome = System.getProperty("catalina.home");
        return new File(catalinaHome, "/logs/artemis").getAbsolutePath();
    }
}
