package eu.profinit.manta.bachelor.thesis.log.viewer.controllers;


import javax.validation.constraints.DecimalMin;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;

import eu.profinit.manta.bachelor.thesis.log.viewer.controllers.dto.SetExpiryCheckResponseDTO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.UpdateSettingsService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;

/**
 * Controller handling requests to change any kind of settings in Log Viewer.
 * @author jkovac
 */
@Api(tags = "Settings")
@Controller
@Validated
public class ChangeSettingsController extends LogViewerBaseController {

    /**
     * Service that changes the settings and provides data about possible impact of those changes
     */
    private UpdateSettingsService updateSettingsService;

    /**
     * Ctr for Change settings controller
     * @param updateSettingsService Service that changes the settings and provides data about possible impact of those changes
     */
    @Autowired
    public ChangeSettingsController(UpdateSettingsService updateSettingsService) {
        this.updateSettingsService = updateSettingsService;
    }

    /**
     * Request to change the time period after which logs expire and are pruned.
     * @param daysUntilExpiry new amount of days until expiry
     */
    @ApiOperation(
            value = "Change expiration time of the logs and reschedule pending prunings.",
            notes = "Force deletes logs if they are expired under the new settings.",
            tags = "Settings"
    )
    @PostMapping(value = "/settings/set-log-expiry")
    @ResponseStatus(value = HttpStatus.OK)
    public void setLogExpiryTime(
            @ApiParam(value = "Amount of days after which logs will expire. Can be decimal and cannot be zero.", required = true)
            @RequestParam @DecimalMin(value = "0.00", inclusive = false, message = "The expiry time must be bigger than 0!") double daysUntilExpiry
    ){
        updateSettingsService.updatePruningDelayProperty(daysUntilExpiry);
    }

    /**
     * Request to check whether changing setting to a given value would automatically prune any logs.
     * @param daysUntilExpiry potential new amount of days until expiry
     * @return Information regarding amount of logs that may be deleted
     */
    @ApiOperation(value = "Check if changing expiration time of the logs to a given value would delete any logs.", tags = "Settings")
    @GetMapping(value = "/settings/set-log-expiry-check")
    @ResponseBody
    public ResponseEntity<SetExpiryCheckResponseDTO> checkIfLogsWillBeDeleted(
            @ApiParam(value = "Amount of days after which logs will expire. Can be decimal and cannot be zero.", required = true)
            @RequestParam @DecimalMin(value = "0.00", inclusive = false, message = "The expiry time must be bigger than 0!") double daysUntilExpiry){
        SetExpiryCheckResponseDTO setExpiryCheckResponse = updateSettingsService.checkIfSomeLogsWillBePruned(daysUntilExpiry);
        return new ResponseEntity<>(setExpiryCheckResponse, HttpStatus.OK);
    }

}
