package eu.profinit.manta.bachelor.thesis.log.viewer.controllers;

import java.util.List;

import javax.validation.constraints.Min;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.Log;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.LogContextService;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.LogQueryService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;

/**
 * Controller for requests regarding log context, i.e. debug logs surrounding it.
 * @author jkovac
 */
@Api(tags = "Log Context")
@Controller
@Validated
public class ContextController extends LogViewerBaseController {

    /**
     * Provider of the log context
     */
    private LogContextService logContextService;

    /**
     * Ctr for the log context controller
     * @param logContextService Provider of the log context
     */
    @Autowired
    public ContextController(LogContextService logContextService) {
        this.logContextService = logContextService;
    }

    /**
     * Request to build context view for a given error log. Context view is a list of any logs that happened before and
     * after the requested log in its thread.
     *
     * @param logId ID of the log
     * @return list of logs
     */
    @ApiOperation(value = "Build context view for a given error. " +
            "Context view is a String imitating a log file that includes logs with severity INFO and below.", tags = "Log Context")
    @GetMapping("/logs/{logId}")
    @ResponseBody
    public ResponseEntity<List<? extends Log>> getContext(
            @ApiParam(value = "ID of the requested log.", required = true)
            @PathVariable @NotNull(message = "Log id must be null.") Long logId
    ){
        List<? extends Log> logs = logContextService.getContextForLog(logId);
        return new ResponseEntity<>(logs, HttpStatus.OK);
    }

    /**
     * Request to load more logs into the log context that happened before the error log.
     * @param logId ID of the log
     * @param offset the amount of logs already loaded in this direction
     * @param amount amount of logs that should be loaded
     * @return list of logs
     */
    @ApiOperation(value = "Loads more logs that were logged before the error log.", tags = "Log Context")
    @GetMapping("/logs/{logId}/before")
    @ResponseBody
    public ResponseEntity<List<? extends Log>> loadEarlierContext(
            @ApiParam(value = "ID of the requested log.", required = true)
            @PathVariable @NotNull(message = "Log id must be null.") Long logId,

            @ApiParam(value = "Amount of logs already loaded in this direction.")
            @RequestParam(value = "offset") @Min(value = 0, message = "Offset cannot be negative!") long offset,

            @ApiParam(value = "Amount of logs that should be further loaded.", defaultValue = "5")
            @RequestParam(value = "amount", required = false, defaultValue = "5") @Min(value = 0, message = "Amount cannot be negative!") long amount
    ){
        List<? extends Log> logs = logContextService.loadEarlier(logId, offset, amount);
        return new ResponseEntity<>(logs, HttpStatus.OK);
    }

    /**
     * Request to load more logs into the log context that happened after the error log.
     * @param logId ID of the log
     * @param offset the amount of logs already loaded in this direction
     * @param amount amount of logs that should be loaded
     * @return list of logs
     */
    @ApiOperation(value = "Loads more logs that were logged after the error log.", tags = "Log Context")
    @GetMapping("/logs/{logId}/after")
    @ResponseBody
    public ResponseEntity<List<? extends Log>> loadLaterContext(
            @ApiParam(value = "ID of the requested log.", required = true)
            @PathVariable @NotNull(message = "Log id must be null.") Long logId,

            @ApiParam(value = "Amount of logs already loaded in this direction.")
            @RequestParam(value = "offset") @Min(value = 0, message = "Offset cannot be negative!") long offset,

            @ApiParam(value = "Amount of logs that should be further loaded.", defaultValue = "5")
            @RequestParam(value = "amount", required = false, defaultValue = "5") @Min(value = 0, message = "Amount cannot be negative!") long amount
    ){
        List<? extends Log> logs = logContextService.loadLater(logId, offset, amount);
        return new ResponseEntity<>(logs, HttpStatus.OK);
    }

}
