package eu.profinit.manta.bachelor.thesis.log.viewer.controllers;

import java.util.List;

import javax.validation.constraints.Min;
import javax.validation.constraints.NotBlank;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import eu.profinit.manta.bachelor.thesis.log.viewer.controllers.dto.ErrorDTO;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.filter.Filter;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.FullTextSearchService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;

/**
 * Controller for full text queries.
 * @author jkovac
 */
@Api(tags = "Full Text Search")
@Controller
@Validated
public class FullTextController extends LogViewerFilterBasedController {

    /**
     * Service providing full text search capability
     */
    private FullTextSearchService fullTextSearchService;

    /**
     * Ctr for the full text search controller
     * @param fullTextSearchService Service providing full text search capability
     */
    @Autowired
    public FullTextController(FullTextSearchService fullTextSearchService) {
        this.fullTextSearchService = fullTextSearchService;
    }

    /**
     * Request to full text search over errors.
     * @param param inputted string
     * @param categories which categories the user wants to see
     * @param errorTypes which error types the user wants to see
     * @param applications from which applications the user wants to see logs
     * @param offset how many logs were already loaded
     * @return error logs that fit
     */
    @ApiOperation(value = "Perform full text search over errors. Search is being done over user and technical message.", tags = "Full Text Search")
    @GetMapping("/search")
    @ResponseBody
    public ResponseEntity<List<ErrorDTO>> fulltext(
            @ApiParam(value = "User inputted query.", required = true)
            @RequestParam(value = "query") @NotBlank(message = "Search cannot be empty!") String param,

            @ApiParam(value = "List of categories that the user wants to see. If null, logs from all are returned.")
            @RequestParam(required = false, defaultValue = "") List<String> categories,

            @ApiParam(value = "List of error types that the user wants to see. If null, logs from all are returned.")
            @RequestParam(required = false, defaultValue = "") List<String> errorTypes,

            @ApiParam(value = "List of applications that the user wants to see. If null, logs from all are returned.")
            @RequestParam(required = false, defaultValue = "") List<String> applications,

            @ApiParam(value = "Offset, how many elements were returned since the beginning. Used due to possible large amount of results.", required = true)
            @RequestParam @Min(value = 0, message = "Offset cannot be negative!") Long offset
    ){
        Filter filter = createFilter(EMPTY_STRING, EMPTY_STRING, EMPTY_STRING, EMPTY_STRING,
                categories, errorTypes, applications, offset);
        List<ErrorDTO> errors = fullTextSearchService.fullText(param, filter);
        return new ResponseEntity<>(errors, HttpStatus.OK);
    }

    /**
     * Request to full text search over errors from workflow execution.
     * @param workflowExecutionId ID of the workflow execution
     * @param param inputted string
     * @param categories which categories the user wants to see
     * @param errorTypes which error types the user wants to see
     * @param applications from which applications the user wants to see logs
     * @param offset how many logs were already loaded
     * @return error logs that fit
     */
    @ApiOperation(value = "Perform full text search over errors from a workflow execution. Search is being done over user and technical message.", tags = "Full Text Search")
    @GetMapping("/workflow-executions/{workflowExecutionId}/search")
    @ResponseBody
    public ResponseEntity<List<ErrorDTO>> fulltext(
            @ApiParam(value = "Workflow execution ID, passed by the process manager.", required = true)
            @PathVariable String workflowExecutionId,

            @ApiParam(value = "User inputted query.", required = true)
            @RequestParam(value = "query") @NotBlank(message = "Search cannot be empty!") String param,

            @ApiParam(value = "List of categories that the user wants to see. If null, logs from all are returned.")
            @RequestParam(required = false, defaultValue = "") List<String> categories,

            @ApiParam(value = "List of error types that the user wants to see. If null, logs from all are returned.")
            @RequestParam(required = false, defaultValue = "") List<String> errorTypes,

            @ApiParam(value = "List of applications that the user wants to see. If null, logs from all are returned.")
            @RequestParam(required = false, defaultValue = "") List<String> applications,

            @ApiParam(value = "Offset, how many elements were returned since the beginning. Used due to possible large amount of results.", required = true)
            @RequestParam @Min(value = 0, message = "Offset cannot be negative!") Long offset
    ){
        Filter filter = createFilter(workflowExecutionId, EMPTY_STRING, EMPTY_STRING, EMPTY_STRING,
                categories, errorTypes, applications, offset);
        List<ErrorDTO> errors = fullTextSearchService.fullText(param, filter);
        return new ResponseEntity<>(errors, HttpStatus.OK);
    }

    /**
     * Request to full text search over errors from scenario execution.
     * @param workflowExecutionId ID of the workflow execution
     * @param scenarioExecutionId ID of the scenario execution
     * @param param inputted string
     * @param categories which categories the user wants to see
     * @param errorTypes which error types the user wants to see
     * @param applications from which applications the user wants to see logs
     * @param offset how many logs were already loaded
     * @return error logs that fit
     */
    @ApiOperation(value = "Perform full text search over errors from scenario execution. Search is being done over user and technical message.", tags = "Full Text Search")
    @GetMapping("/workflow-executions/{workflowExecutionId}/scenario-executions/{scenarioExecutionId}/search")
    @ResponseBody
    public ResponseEntity<List<ErrorDTO>> fulltext(
            @ApiParam(value = "Workflow execution ID, passed by the process manager.", required = true)
            @PathVariable String workflowExecutionId,

            @ApiParam(value = "Scenario execution ID, passed by the process manager.", required = true)
            @PathVariable String scenarioExecutionId,

            @ApiParam(value = "User inputted query.", required = true)
            @RequestParam(value = "query") @NotBlank(message = "Search cannot be empty!") String param,

            @ApiParam(value = "List of categories that the user wants to see. If null, logs from all are returned.")
            @RequestParam(required = false, defaultValue = "") List<String> categories,

            @ApiParam(value = "List of error types that the user wants to see. If null, logs from all are returned.")
            @RequestParam(required = false, defaultValue = "") List<String> errorTypes,

            @ApiParam(value = "List of applications that the user wants to see. If null, logs from all are returned.")
            @RequestParam(required = false, defaultValue = "") List<String> applications,

            @ApiParam(value = "Offset, how many elements were returned since the beginning. Used due to possible large amount of results.", required = true)
            @RequestParam @Min(value = 0, message = "Offset cannot be negative!") Long offset
    ){
        Filter filter = createFilter(workflowExecutionId, scenarioExecutionId, EMPTY_STRING, EMPTY_STRING,
                categories, errorTypes, applications, offset);
        List<ErrorDTO> errors = fullTextSearchService.fullText(param, filter);
        return new ResponseEntity<>(errors, HttpStatus.OK);
    }

    /**
     * Request to full text search over errors from connection.
     * @param workflowExecutionId ID of the workflow execution
     * @param scenarioExecutionId ID of the scenario execution
     * @param connectionId name of the connection
     * @param param inputted string
     * @param categories which categories the user wants to see
     * @param errorTypes which error types the user wants to see
     * @param applications from which applications the user wants to see logs
     * @param offset how many logs were already loaded
     * @return error logs that fit
     */
    @ApiOperation(value = "Perform full text search over errors from connection. Search is being done over user and technical message.", tags = "Full Text Search")
    @GetMapping("/workflow-executions/{workflowExecutionId}/scenario-executions/{scenarioExecutionId}/connections/{connectionId}/search")
    @ResponseBody
    public ResponseEntity<List<ErrorDTO>> fulltext(
            @ApiParam(value = "Workflow execution ID, passed by the process manager.", required = true)
            @PathVariable String workflowExecutionId,

            @ApiParam(value = "Scenario execution ID, passed by the process manager.", required = true)
            @PathVariable String scenarioExecutionId,

            @ApiParam(value = "Name of the connection.", required = true)
            @PathVariable String connectionId,

            @ApiParam(value = "User inputted query.", required = true)
            @RequestParam(value = "query") @NotBlank(message = "Search cannot be empty!") String param,

            @ApiParam(value = "List of categories that the user wants to see. If null, logs from all are returned.")
            @RequestParam(required = false, defaultValue = "") List<String> categories,

            @ApiParam(value = "List of error types that the user wants to see. If null, logs from all are returned.")
            @RequestParam(required = false, defaultValue = "") List<String> errorTypes,

            @ApiParam(value = "List of applications that the user wants to see. If null, logs from all are returned.")
            @RequestParam(required = false, defaultValue = "") List<String> applications,

            @ApiParam(value = "Offset, how many elements were returned since the beginning. Used due to possible large amount of results.", required = true)
            @RequestParam @Min(value = 0, message = "Offset cannot be negative!") Long offset
    ){
        Filter filter = createFilter(workflowExecutionId, scenarioExecutionId, connectionId, EMPTY_STRING,
                categories, errorTypes, applications, offset);
        List<ErrorDTO> errors = fullTextSearchService.fullText(param, filter);
        return new ResponseEntity<>(errors, HttpStatus.OK);
    }

    /**
     * Request to full text search over errors from package.
     * @param packageName name of the uploaded package
     * @param param inputted string
     * @param categories which categories the user wants to see
     * @param errorTypes which error types the user wants to see
     * @param applications from which applications the user wants to see logs
     * @param offset how many logs were already loaded
     * @return error logs that fit
     */
    @ApiOperation(value = "Perform full text search over errors from a package. Search is being done over user and technical message.", tags = "Full Text Search")
    @GetMapping("/packages/{packageName}/search")
    @ResponseBody
    public ResponseEntity<List<ErrorDTO>> fulltextPackage(
            @ApiParam(value = "Name of the uploaded package.", required = true)
            @PathVariable String packageName,

            @ApiParam(value = "User inputted query.", required = true)
            @RequestParam(value = "query") @NotBlank(message = "Search cannot be empty!") String param,

            @ApiParam(value = "List of categories that the user wants to see. If null, logs from all are returned.")
            @RequestParam(required = false, defaultValue = "") List<String> categories,

            @ApiParam(value = "List of error types that the user wants to see. If null, logs from all are returned.")
            @RequestParam(required = false, defaultValue = "") List<String> errorTypes,

            @ApiParam(value = "List of applications that the user wants to see. If null, logs from all are returned.")
            @RequestParam(required = false, defaultValue = "") List<String> applications,

            @ApiParam(value = "Offset, how many elements were returned since the beginning. Used due to possible large amount of results.", required = true)
            @RequestParam @Min(value = 0, message = "Offset cannot be negative!") Long offset
    ){
        Filter filter = createFilter(EMPTY_STRING, EMPTY_STRING, EMPTY_STRING, packageName,
                categories, errorTypes, applications, offset);
        List<ErrorDTO> errors = fullTextSearchService.fullText(param, filter);
        return new ResponseEntity<>(errors, HttpStatus.OK);
    }


}
