package eu.profinit.manta.bachelor.thesis.log.viewer.controllers;

import java.util.Collections;
import java.util.List;

import javax.validation.constraints.Min;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import eu.profinit.manta.bachelor.thesis.log.viewer.model.filter.Filter;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.ErrorLog;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.ExpandLogsService;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.LogQueryService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;

/**
 * Controller handling requests for concrete error logs
 * @author jkovac
 */
@Controller
@Api(tags = "Error expansion")
@Validated
public class LogExpandController extends LogViewerFilterBasedController {

    /**
     * Service providing concrete logs
     */
    private ExpandLogsService expandLogsService;

    /**
     * Ctr for error expansion controller
     * @param expandLogsService Service providing concrete logs
     */
    @Autowired
    public LogExpandController(ExpandLogsService expandLogsService) {
        this.expandLogsService = expandLogsService;
    }

    /**
     * Get concrete logs from an error type.
     * @param errorCategory name of the category to whith the log belongs
     * @param application name of the application that emitted the log
     * @param collapsedElement error type for which should concrete logs be provided
     * @param offset how many logs were already loaded
     * @return List of error logs stored for this scenario execution
     */
    @GetMapping("/expand")
    @ApiOperation(value = "Get concrete logs from an error type.", tags = "Error expansion")
    @ResponseBody
    public ResponseEntity<List<ErrorLog>> expandLogs(
            @ApiParam(value = "Name of the error category of the selected error.", required = true)
            @RequestParam String errorCategory,

            @ApiParam(value = "Name of the selected element to expand.", required = true)
            @RequestParam String collapsedElement,

            @ApiParam(value = "Name of the application from which should logs be returned.", required = true)
            @RequestParam String application,

            @ApiParam(value = "Offset, how many elements were returned since the beginning. Used due to possible large amount of results.", required = true)
            @RequestParam @Min(value = 0, message = "Offset cannot be negative!") Long offset
    ){
        Filter filter = createFilter(EMPTY_STRING, EMPTY_STRING, EMPTY_STRING, EMPTY_STRING, Collections.emptyList(), Collections.emptyList(), Collections.emptyList(), offset);
        List<ErrorLog> errors = expandLogsService.expandLogs(errorCategory, collapsedElement, application, filter);
        return new ResponseEntity<>(errors, HttpStatus.OK);
    }

    /**
     * Get concrete logs from an error type occurring in a given workflow execution.
     * @param workflowExecutionId ID of the workflow execution
     * @param errorCategory name of the category to whith the log belongs
     * @param application name of the application that emitted the log
     * @param collapsedElement error type for which should concrete logs be provided
     * @param offset how many logs were already loaded
     * @return List of error logs stored for this scenario execution
     */
    @GetMapping("/workflow-executions/{workflowExecutionId}/expand")
    @ApiOperation(value = "Get concrete logs from an error type occurring in a given workflow execution.", tags = "Error expansion")
    @ResponseBody
    public ResponseEntity<List<ErrorLog>> expandLogs(
            @ApiParam(value = "Workflow execution ID, passed by the process manager.", required = true)
            @PathVariable String workflowExecutionId,

            @ApiParam(value = "Name of the error category of the selected error.", required = true)
            @RequestParam String errorCategory,

            @ApiParam(value = "Name of the selected element to expand.", required = true)
            @RequestParam String collapsedElement,

            @ApiParam(value = "Name of the application from which should logs be returned.", required = true)
            @RequestParam String application,

            @ApiParam(value = "Offset, how many elements were returned since the beginning. Used due to possible large amount of results.", required = true)
            @RequestParam @Min(value = 0, message = "Offset cannot be negative!") Long offset
    ){
        Filter filter = createFilter(workflowExecutionId, EMPTY_STRING, EMPTY_STRING, EMPTY_STRING, Collections.emptyList(), Collections.emptyList(), Collections.emptyList(), offset);
        List<ErrorLog> errors = expandLogsService.expandLogs(errorCategory, collapsedElement, application, filter);
        return new ResponseEntity<>(errors, HttpStatus.OK);
    }

    /**
     * Get concrete logs from an error type occurring in a given scenario execution.
     * @param workflowExecutionId ID of the workflow execution
     * @param scenarioExecutionId ID of the scenario execution
     * @param errorCategory name of the category to whith the log belongs
     * @param application name of the application that emitted the log
     * @param collapsedElement error type for which should concrete logs be provided
     * @param offset how many logs were already loaded
     * @return List of error logs stored for this scenario execution
     */
    @GetMapping("/workflow-executions/{workflowExecutionId}/scenario-executions/{scenarioExecutionId}/expand")
    @ApiOperation(value = "Get concrete logs from an error type occurring in a given scenario execution.", tags = "Error expansion")
    @ResponseBody
    public ResponseEntity<List<ErrorLog>> expandLogs(
            @ApiParam(value = "Workflow execution ID, passed by the process manager.", required = true)
            @PathVariable String workflowExecutionId,

            @ApiParam(value = "Scenario execution ID, passed by the process manager.", required = true)
            @PathVariable String scenarioExecutionId,

            @ApiParam(value = "Name of the error category of the selected error.", required = true)
            @RequestParam String errorCategory,

            @ApiParam(value = "Name of the selected element to expand.", required = true)
            @RequestParam String collapsedElement,

            @ApiParam(value = "Name of the application from which should logs be returned.", required = true)
            @RequestParam String application,

            @ApiParam(value = "Offset, how many elements were returned since the beginning. Used due to possible large amount of results.", required = true)
            @RequestParam @Min(value = 0, message = "Offset cannot be negative!") Long offset
    ){
        Filter filter = createFilter(workflowExecutionId, scenarioExecutionId, EMPTY_STRING, EMPTY_STRING, Collections.emptyList(), Collections.emptyList(), Collections.emptyList(), offset);
        List<ErrorLog> errors = expandLogsService.expandLogs(errorCategory, collapsedElement, application, filter);
        return new ResponseEntity<>(errors, HttpStatus.OK);
    }

    /**
     * Get concrete logs from an error type occurring in a given connection.
     * @param workflowExecutionId ID of the workflow execution
     * @param scenarioExecutionId ID of the scenario execution
     * @param connectionId name of the connection
     * @param errorCategory name of the category to whith the log belongs
     * @param application name of the application that emitted the log
     * @param collapsedElement error type for which should concrete logs be provided
     * @param offset how many logs were already loaded
     * @return List of error logs stored for this scenario execution
     */
    @GetMapping("/workflow-executions/{workflowExecutionId}/scenario-executions/{scenarioExecutionId}/connections/{connectionId}/expand")
    @ApiOperation(value = "Get concrete logs from an error type occurring in a given connection.", tags = "Error expansion")
    @ResponseBody
    public ResponseEntity<List<ErrorLog>> expandLogs(
            @ApiParam(value = "Workflow execution ID, passed by the process manager.", required = true)
            @PathVariable String workflowExecutionId,

            @ApiParam(value = "Scenario execution ID, passed by the process manager.", required = true)
            @PathVariable String scenarioExecutionId,

            @ApiParam(value = "Name of the connection.", required = true)
            @PathVariable String connectionId,

            @ApiParam(value = "Name of the error category of the selected error.", required = true)
            @RequestParam String errorCategory,

            @ApiParam(value = "Name of the selected element to expand.", required = true)
            @RequestParam String collapsedElement,

            @ApiParam(value = "Name of the application from which should logs be returned.", required = true)
            @RequestParam String application,

            @ApiParam(value = "Offset, how many elements were returned since the beginning. Used due to possible large amount of results.", required = true)
            @RequestParam @Min(value = 0, message = "Offset cannot be negative!") Long offset
    ){
        Filter filter = createFilter(workflowExecutionId, scenarioExecutionId, connectionId, EMPTY_STRING, Collections.emptyList(), Collections.emptyList(), Collections.emptyList(), offset);
        List<ErrorLog> errors = expandLogsService.expandLogs(errorCategory, collapsedElement, application, filter);
        return new ResponseEntity<>(errors, HttpStatus.OK);
    }

    /**
     * Get concrete logs from an error type occurring in a given package.
     * @param packageName name of the uploaded package
     * @param errorCategory name of the category to whith the log belongs
     * @param application name of the application that emitted the log
     * @param collapsedElement error type for which should concrete logs be provide
     * @param offset how many logs were already loaded
     * @return List of error logs stored for this scenario execution
     */
    @GetMapping("/packages/{packageName}/expand")
    @ApiOperation(value = "Get concrete logs from an error type occurring in a given package.", tags = "Error expansion")
    @ResponseBody
    public ResponseEntity<List<ErrorLog>> expandPackageLogs(
            @ApiParam(value = "Workflow execution ID, passed by the process manager.", required = true)
            @PathVariable String packageName,

            @ApiParam(value = "Name of the error category of the selected error.", required = true)
            @RequestParam String errorCategory,

            @ApiParam(value = "Name of the selected element to expand.", required = true)
            @RequestParam String collapsedElement,

            @ApiParam(value = "Name of the application from which should logs be returned.", required = true)
            @RequestParam String application,

            @ApiParam(value = "Offset, how many elements were returned since the beginning. Used due to possible large amount of results.", required = true)
            @RequestParam @Min(value = 0, message = "Offset cannot be negative!") Long offset
    ){
        Filter filter = createFilter(EMPTY_STRING, EMPTY_STRING, EMPTY_STRING, packageName, Collections.emptyList(), Collections.emptyList(), Collections.emptyList(), offset);
        List<ErrorLog> errors = expandLogsService.expandLogs(errorCategory, collapsedElement, application, filter);
        return new ResponseEntity<>(errors, HttpStatus.OK);
    }

}
