package eu.profinit.manta.bachelor.thesis.log.viewer.controllers;

import java.util.List;
import java.util.Optional;

import javax.validation.constraints.Min;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import eu.profinit.manta.bachelor.thesis.log.viewer.controllers.dto.ErrorDTO;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.DistinctErrorType;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.ErrorMetadata;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.ScenarioExecution;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.UploadedPackage;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.WorkflowExecution;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.filter.Filter;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.ErrorLog;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.LogQueryService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;

/**
 * Controller for all query operations for logs.
 * @author jkovac
 */
@Controller
@Api(tags = "Log Querying")
@Validated
public class LogQueryController extends LogViewerFilterBasedController {

    /**
     * Service providing error metadata based on given parameters
     */
    private LogQueryService logQueryService;

    /**
     * Ctr for the log query controller
     * @param logQueryService Service providing error metadata based on given parameters
     */
    @Autowired
    public LogQueryController(LogQueryService logQueryService) {
        this.logQueryService = logQueryService;
    }


    /**
     * Query error logs based on given parameters.
     * @param categories which categories the user wants to see
     * @param errorTypes which error types the user wants to see
     * @param applications from which application the user wants to see logs
     * @param offset how many logs were already loaded
     * @return List of error logs stored for this scenario execution
     */
    @GetMapping("/")
    @ApiOperation(value = "Query error logs based on given parameters.", tags = "Log Querying")
    @ResponseBody
    public ResponseEntity<List<ErrorDTO>> queryLogs(
            @ApiParam(value = "List of categories that the user wants to see. If null, logs from all are returned.")
            @RequestParam(required = false, defaultValue = "") List<String> categories,

            @ApiParam(value = "List of error types that the user wants to see. If null, logs from all are returned.")
            @RequestParam(required = false, defaultValue = "") List<String> errorTypes,

            @ApiParam(value = "List of applications that the user wants to see. If null, logs from all are returned.")
            @RequestParam(required = false, defaultValue = "") List<String> applications,

            @ApiParam(value = "Offset, how many elements were returned since the beginning. Used due to possible large amount of results.", required = true)
            @RequestParam @Min(value = 0, message = "Offset cannot be negative!") Long offset
    ){
        Filter filter = createFilter(EMPTY_STRING, EMPTY_STRING, EMPTY_STRING, EMPTY_STRING, categories, errorTypes, applications, offset);
        List<ErrorDTO> errors = logQueryService.queryLogs(filter);
        return new ResponseEntity<>(errors, HttpStatus.OK);
    }

    /**
     * Query error logs from workflow execution based on given parameters.
     * @param workflowExecutionId ID of the workflow execution
     * @param categories which categories the user wants to see
     * @param errorTypes which error types the user wants to see
     * @param applications from which application the user wants to see logs
     * @param offset how many logs were already loaded
     * @return List of error logs stored for this scenario execution
     */
    @GetMapping("/workflow-executions/{workflowExecutionId}")
    @ApiOperation(value = "Query error logs from workflow execution based on given parameters.", tags = "Log Querying")
    @ResponseBody
    public ResponseEntity<List<ErrorDTO>> queryLogs(
            @ApiParam(value = "Workflow execution ID, passed by the process manager.", required = true)
            @PathVariable String workflowExecutionId,

            @ApiParam(value = "List of categories that the user wants to see. If null, logs from all are returned.")
            @RequestParam(required = false, defaultValue = "") List<String> categories,

            @ApiParam(value = "List of error types that the user wants to see. If null, logs from all are returned.")
            @RequestParam(required = false, defaultValue = "") List<String> errorTypes,

            @ApiParam(value = "List of applications that the user wants to see. If null, logs from all are returned.")
            @RequestParam(required = false, defaultValue = "") List<String> applications,

            @ApiParam(value = "Offset, how many elements were returned since the beginning. Used due to possible large amount of results.", required = true)
            @RequestParam @Min(value = 0, message = "Offset cannot be negative!") Long offset
    ){
        Filter filter = createFilter(workflowExecutionId, EMPTY_STRING, EMPTY_STRING, EMPTY_STRING, categories, errorTypes, applications, offset);
        List<ErrorDTO> errors = logQueryService.queryLogs(filter);
        return new ResponseEntity<>(errors, HttpStatus.OK);
    }

    /**
     * Query error logs from scenario execution based on given parameters.
     * @param workflowExecutionId ID of the workflow execution
     * @param scenarioExecutionId ID of the scenario execution
     * @param categories which categories the user wants to see
     * @param errorTypes which error types the user wants to see
     * @param applications from which application the user wants to see logs
     * @param offset how many logs were already loaded
     * @return List of error logs stored for this scenario execution
     */
    @GetMapping("/workflow-executions/{workflowExecutionId}/scenario-executions/{scenarioExecutionId}")
    @ApiOperation(value = "Query error logs from scenario execution based on given parameters.", tags = "Log Querying")
    @ResponseBody
    public ResponseEntity<List<ErrorDTO>> queryLogs(
            @ApiParam(value = "Workflow execution ID, passed by the process manager.", required = true)
            @PathVariable String workflowExecutionId,

            @ApiParam(value = "Scenario execution ID, passed by the process manager.", required = true)
            @PathVariable String scenarioExecutionId,

            @ApiParam(value = "List of categories that the user wants to see. If null, logs from all are returned.")
            @RequestParam(required = false, defaultValue = "") List<String> categories,

            @ApiParam(value = "List of error types that the user wants to see. If null, logs from all are returned.")
            @RequestParam(required = false, defaultValue = "") List<String> errorTypes,

            @ApiParam(value = "List of applications that the user wants to see. If null, logs from all are returned.")
            @RequestParam(required = false, defaultValue = "") List<String> applications,

            @ApiParam(value = "Offset, how many elements were returned since the beginning. Used due to possible large amount of results.", required = true)
            @RequestParam @Min(value = 0, message = "Offset cannot be negative!") Long offset
    ){
        Filter filter = createFilter(workflowExecutionId, scenarioExecutionId, EMPTY_STRING, EMPTY_STRING, categories, errorTypes, applications, offset);
        List<ErrorDTO> errors = logQueryService.queryLogs(filter);
        return new ResponseEntity<>(errors, HttpStatus.OK);
    }

    /**
     * Query error logs from connection based on given parameters.
     * @param workflowExecutionId ID of the workflow execution
     * @param scenarioExecutionId ID of the scenario execution
     * @param connectionId name of the connection
     * @param categories which categories the user wants to see
     * @param errorTypes which error types the user wants to see
     * @param applications from which application the user wants to see logs
     * @param offset how many logs were already loaded
     * @return List of error logs stored for this scenario execution
     */
    @GetMapping("/workflow-executions/{workflowExecutionId}/scenario-executions/{scenarioExecutionId}/connections/{connectionId}")
    @ApiOperation(value = "Query error logs from connection based on given parameters.", tags = "Log Querying")
    @ResponseBody
    public ResponseEntity<List<ErrorDTO>> queryLogs(
            @ApiParam(value = "Workflow execution ID, passed by the process manager.", required = true)
            @PathVariable String workflowExecutionId,

            @ApiParam(value = "Scenario execution ID, passed by the process manager.", required = true)
            @PathVariable String scenarioExecutionId,

            @ApiParam(value = "Name of the connection.", required = true)
            @PathVariable String connectionId,

            @ApiParam(value = "List of categories that the user wants to see. If null, logs from all are returned.")
            @RequestParam(required = false, defaultValue = "") List<String> categories,

            @ApiParam(value = "List of error types that the user wants to see. If null, logs from all are returned.")
            @RequestParam(required = false, defaultValue = "") List<String> errorTypes,

            @ApiParam(value = "List of applications that the user wants to see. If null, logs from all are returned.")
            @RequestParam(required = false, defaultValue = "") List<String> applications,

            @ApiParam(value = "Offset, how many elements were returned since the beginning. Used due to possible large amount of results.", required = true)
            @RequestParam @Min(value = 0, message = "Offset cannot be negative!") Long offset
    ){
        Filter filter = createFilter(workflowExecutionId, scenarioExecutionId, connectionId, EMPTY_STRING, categories, errorTypes, applications, offset);
        List<ErrorDTO> errors = logQueryService.queryLogs(filter);
        return new ResponseEntity<>(errors, HttpStatus.OK);
    }

    /**
     * Query error logs from a package based on given parameters.
     * @param packageName name of the uploaded package
     * @param categories which categories the user wants to see
     * @param errorTypes which error types the user wants to see
     * @param applications from which application the user wants to see logs
     * @param offset how many logs were already loaded
     * @return List of error logs stored for this scenario execution
     */
    @GetMapping("/packages/{packageName}")
    @ApiOperation(value = "Query error logs from a package based on given parameters.", tags = "Log Querying")
    @ResponseBody
    public ResponseEntity<List<ErrorDTO>> queryPackageLogs(
            @ApiParam(value = "Workflow execution ID, passed by the process manager.", required = true)
            @PathVariable String packageName,

            @ApiParam(value = "List of categories that the user wants to see. If null, logs from all are returned.")
            @RequestParam(required = false, defaultValue = "") List<String> categories,

            @ApiParam(value = "List of error types that the user wants to see. If null, logs from all are returned.")
            @RequestParam(required = false, defaultValue = "") List<String> errorTypes,

            @ApiParam(value = "List of applications that the user wants to see. If null, logs from all are returned.")
            @RequestParam(required = false, defaultValue = "") List<String> applications,

            @ApiParam(value = "Offset, how many elements were returned since the beginning. Used due to possible large amount of results.", required = true)
            @RequestParam @Min(value = 0, message = "Offset cannot be negative!") Long offset
    ){
        Filter filter = createFilter(EMPTY_STRING, EMPTY_STRING, EMPTY_STRING, packageName, categories, errorTypes, applications, offset);
        List<ErrorDTO> errors = logQueryService.queryLogs(filter);
        return new ResponseEntity<>(errors, HttpStatus.OK);
    }

}
