package eu.profinit.manta.bachelor.thesis.log.viewer.controllers;

import javax.validation.ConstraintViolationException;

import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import eu.profinit.manta.bachelor.thesis.log.viewer.controllers.dto.ErrorResponseDTO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.exceptions.InvalidContextException;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.exceptions.InvalidPackageFileException;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.exceptions.PackageAlreadyExistsException;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.exceptions.UnableToCreateDumpException;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.exceptions.UnableToDeleteTemporaryFiles;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.exceptions.UnableToPruneWorkflowExecutionException;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.exceptions.WorkflowDoesNotExistException;
import eu.profinit.manta.bachelor.thesis.logging.api.logging.Logger;

/**
 * Common ancestor for Log Viewer controllers. Exception handler.
 * @author jkovac
 */
@RequestMapping(value = "/private/log-viewer/v1")
public abstract class LogViewerBaseController {

    /**
     * Logger
     */
    private Logger logger = new Logger(getClass());

    /**
     * Build error response for client.
     * @param status HTTP error status
     * @param message Error message
     * @return response to client
     */
    protected ResponseEntity<ErrorResponseDTO> createErrorResponse(HttpStatus status, String message){
        return new ResponseEntity<>(new ErrorResponseDTO(status, message), status);
    }

    /**
     * Handler for a corrupt uploaded package file.
     * @param exception InvalidPackageFileException
     * @return error response
     */
    @ExceptionHandler(InvalidPackageFileException.class)
    @ResponseBody
    public ResponseEntity<ErrorResponseDTO> invalidPackageFile(InvalidPackageFileException exception){
        logger.debug("The uploaded package has an incorrect structure or is otherwise corrupted. Exception: " +
                exception.getMessage(), exception);
        return createErrorResponse(HttpStatus.BAD_REQUEST, exception.getMessage());
    }

    /**
     * Handler for invalid package name.
     * @param exception PackageAlreadyExistsException
     * @return error response
     */
    @ExceptionHandler(PackageAlreadyExistsException.class)
    @ResponseBody
    public ResponseEntity<ErrorResponseDTO> packageAlreadyExists(PackageAlreadyExistsException exception){
        logger.debug(exception.getMessage(), exception);
        return createErrorResponse(HttpStatus.BAD_REQUEST, exception.getMessage());
    }

    /**
     * Handler for constraint violation exceptions when validating input from the user.
     * @param exception ConstraintViolationException
     * @return error response
     */
    @ExceptionHandler(ConstraintViolationException.class)
    @ResponseBody
    public ResponseEntity<ErrorResponseDTO> constraintViolation(ConstraintViolationException exception){
        logger.debug(exception.getMessage(), exception);
        return createErrorResponse(HttpStatus.BAD_REQUEST, exception.getMessage());
    }

    /**
     * Handler for invalid context requests
     * @param exception InvalidContextException
     * @return bad request
     */
    @ExceptionHandler(InvalidContextException.class)
    @ResponseBody
    public ResponseEntity<ErrorResponseDTO> invalidContextException(InvalidContextException exception){
        logger.debug(exception.getMessage(), exception);
        return createErrorResponse(HttpStatus.BAD_REQUEST, exception.getMessage());
    }

    /**
     * Handler for exceptions thrown due to inability to provide log dump
     * @param exception UnableToCreateDumpException
     * @return bad request
     */
    @ExceptionHandler(UnableToCreateDumpException.class)
    @ResponseBody
    public ResponseEntity<ErrorResponseDTO> unableToCreateDumpException(UnableToCreateDumpException exception){
        logger.debug(exception.getMessage(), exception);
        return createErrorResponse(HttpStatus.BAD_REQUEST, exception.getMessage());
    }

    /**
     * Handler for exception thrown due to inability to delete temporary files after the dump upload
     * @param exception UnableToDeleteTemporaryFiles
     * @return bad request
     */
    @ExceptionHandler(UnableToDeleteTemporaryFiles.class)
    @ResponseBody
    public ResponseEntity<ErrorResponseDTO> unableToDeleteTemporaryFiles(UnableToDeleteTemporaryFiles exception){
        logger.debug(exception.getMessage(), exception);
        return createErrorResponse(HttpStatus.BAD_REQUEST, exception.getMessage());
    }

    /**
     * Handler for errors stemming from the inability to prune requested workflow execution
     * @param exception UnableToPruneWorkflowExecutionException
     * @return bad request
     */
    @ExceptionHandler(UnableToPruneWorkflowExecutionException.class)
    @ResponseBody
    public ResponseEntity<ErrorResponseDTO> unableToPruneWorkflowExecutionException(UnableToPruneWorkflowExecutionException exception){
        logger.debug(exception.getMessage(), exception);
        return createErrorResponse(HttpStatus.BAD_REQUEST, exception.getMessage());
    }

    /**
     * Handler for bad request for workflow execution
     * @param exception WorkflowDoesNotExistException
     * @return bad request
     */
    @ExceptionHandler(WorkflowDoesNotExistException.class)
    @ResponseBody
    public ResponseEntity<ErrorResponseDTO> workflowDoesNotExistException(WorkflowDoesNotExistException exception){
        logger.debug(exception.getMessage(), exception);
        return createErrorResponse(HttpStatus.BAD_REQUEST, exception.getMessage());
    }

    /**
     * Handler for all the other exceptions that may be thrown.
     * @param exception Exception
     * @return error response
     */
    @ExceptionHandler(Exception.class)
    @ResponseBody
    public ResponseEntity<ErrorResponseDTO> otherErrors(Exception exception){
        logger.debug("Internal error.", exception);
        return createErrorResponse(HttpStatus.INTERNAL_SERVER_ERROR,
                "Internal server error: " + exception.getMessage() + ". See logs for futher details.");
    }

}
