package eu.profinit.manta.bachelor.thesis.log.viewer.controllers;

import java.util.Collections;
import java.util.List;
import java.util.Optional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import eu.profinit.manta.bachelor.thesis.log.viewer.controllers.dto.executions.WorkflowExecutionDTO;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.DistinctErrorType;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.ScenarioExecution;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.UploadedPackage;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.WorkflowExecution;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.filter.Filter;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.LogQueryService;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.MetadataProviderService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;

/**
 * Controller for end-points that provide metadata of the repository, such as stored executions, distinct errors and others.
 * @author jkovac
 */
@Controller
@Api(tags = "Metadata")
public final class MetadataController extends LogViewerFilterBasedController {

    /**
     * Service providing metadata regarding the repository
     */
    private MetadataProviderService metadataProviderService;

    /**
     * Ctr for Metadata controller
     * @param metadataProviderService Service providing metadata regarding the repository
     */
    @Autowired
    public MetadataController(MetadataProviderService metadataProviderService) {
        this.metadataProviderService = metadataProviderService;
    }

    /**
     * Request a list of all executions stored in the repository.
     * @return Nested list of all workflow executions, scenario executions and connections used.
     */
    @ApiOperation(value = "Get all workflow and scenario executions and connections that were used.", tags = "Metadata")
    @GetMapping("/executions")
    @ResponseBody
    public ResponseEntity<List<WorkflowExecutionDTO>> listExecutions(){
        List<WorkflowExecutionDTO> executions = metadataProviderService.listExecutions();
        return new ResponseEntity<>(executions, HttpStatus.OK);
    }

    /**
     * Request list of all applications that have logs stored in the repository.
     * @return List of all applications
     */
    @ApiOperation(value = "List all applications stored in the log repository.", tags = "Metadata")
    @GetMapping("/applications")
    @ResponseBody
    public ResponseEntity<List<String>> getAllApplications(){
        List<String> applications = metadataProviderService.getDistinctApplications();
        return new ResponseEntity<>(applications, HttpStatus.OK);
    }

    /**
     * Request all packages that were uploaded to this repository.
     * @return All packages stored in this repository
     */
    @ApiOperation(value = "List all uploaded packages stored in the log repository.", tags = "Metadata")
    @GetMapping("/packages")
    @ResponseBody
    public ResponseEntity<List<UploadedPackage>> listAllUploadedPackages(){
        List<UploadedPackage> uploadedPackages = metadataProviderService.getAllUploadedPackages();
        return new ResponseEntity<>(uploadedPackages, HttpStatus.OK);
    }

    /**
     * ------------------------------------------ DISTINCT CATEGORIES ------------------------------------------
     */


    /**
     * List all categories stored in the repository, possible filtered by applications
     * @param applications requested applications, if empty all are returned
     * @return distinct error categories
     */
    @ApiOperation(value = "Get distinct error categories to enable front end category-based filtering.", tags = "Metadata")
    @GetMapping("/categories")
    public ResponseEntity<List<String>> getDistinctErrorCategories(
            @ApiParam(value = "List of applications that the user wants to see. If null, logs from all are returned.")
            @RequestParam(required = false, defaultValue = "") List<String> applications
    ){
        Filter filter = createFilter(EMPTY_STRING, EMPTY_STRING, EMPTY_STRING, EMPTY_STRING,
                Collections.emptyList(), Collections.emptyList(), applications, 0L);
        List<String> distinctCategories = metadataProviderService.getDistinctCategories(filter);
        return new ResponseEntity<>(distinctCategories, HttpStatus.OK);
    }

    /**
     * List all categories stored in the repository filtered by a particular workflow execution
     * @param workflowExecutionId worklfow execution id
     * @param applications requested applications
     * @return distinct categories
     */
    @ApiOperation(value = "Get distinct error categories from a workflow execution to enable front end category-based filtering.", tags = "Metadata")
    @GetMapping("/workflow-executions/{workflowExecutionId}/categories")
    public ResponseEntity<List<String>> getDistinctErrorCategories(
            @ApiParam(value = "Workflow execution ID, passed by the process manager.", required = true)
            @PathVariable String workflowExecutionId,

            @ApiParam(value = "List of applications that the user wants to see. If null, logs from all are returned.")
            @RequestParam(required = false, defaultValue = "") List<String> applications

    ){
        Filter filter = createFilter(workflowExecutionId, EMPTY_STRING, EMPTY_STRING, EMPTY_STRING,
                Collections.emptyList(), Collections.emptyList(), applications, 0L);
        List<String> distinctCategories = metadataProviderService.getDistinctCategories(filter);
        return new ResponseEntity<>(distinctCategories, HttpStatus.OK);
    }

    /**
     * List all distinct categories for a particular scenario execution
     * @param workflowExecutionId workflow execution id
     * @param scenarioExecutionId scenario execution id
     * @param applications requested applications
     * @return distinct categories
     */
    @ApiOperation(value = "Get distinct error categories from a scenario execution to enable front end category-based filtering.", tags = "Metadata")
    @GetMapping("/workflow-executions/{workflowExecutionId}/scenario-executions/{scenarioExecutionId}/categories")
    public ResponseEntity<List<String>> getDistinctErrorCategories(
            @ApiParam(value = "Workflow execution ID, passed by the process manager.", required = true)
            @PathVariable String workflowExecutionId,

            @ApiParam(value = "Scenario execution ID, passed by the process manager.", required = true)
            @PathVariable String scenarioExecutionId,

            @ApiParam(value = "List of applications that the user wants to see. If null, logs from all are returned.")
            @RequestParam(required = false, defaultValue = "") List<String> applications
    ){
        Filter filter = createFilter(workflowExecutionId, scenarioExecutionId, EMPTY_STRING, EMPTY_STRING,
                Collections.emptyList(), Collections.emptyList(), applications, 0L);
        List<String> distinctCategories = metadataProviderService.getDistinctCategories(filter);
        return new ResponseEntity<>(distinctCategories, HttpStatus.OK);
    }

    /**
     * Distinct categories from a connection
     * @param workflowExecutionId workflow execution id
     * @param scenarioExecutionId scenario execution id
     * @param connectionId connection id
     * @param applications requested applications
     * @return distinct categories
     */
    @ApiOperation(value = "Get distinct error categories from a connection to enable front end category-based filtering.", tags = "Metadata")
    @GetMapping("/workflow-executions/{workflowExecutionId}/scenario-executions/{scenarioExecutionId}/connections/{connectionId}/categories")
    public ResponseEntity<List<String>> getDistinctErrorCategories(
            @ApiParam(value = "Workflow execution ID, passed by the process manager.", required = true)
            @PathVariable String workflowExecutionId,

            @ApiParam(value = "Scenario execution ID, passed by the process manager.", required = true)
            @PathVariable String scenarioExecutionId,

            @ApiParam(value = "Name of the connection.", required = true)
            @PathVariable String connectionId,

            @ApiParam(value = "List of applications that the user wants to see. If null, logs from all are returned.")
            @RequestParam(required = false, defaultValue = "") List<String> applications
    ){
        Filter filter = createFilter(workflowExecutionId, scenarioExecutionId, connectionId, EMPTY_STRING,
                Collections.emptyList(), Collections.emptyList(), applications, 0L);
        List<String> distinctCategories = metadataProviderService.getDistinctCategories(filter);
        return new ResponseEntity<>(distinctCategories, HttpStatus.OK);
    }

    /**
     * Distinct categories from an uploaded package
     * @param packageName name of the uploaded package
     * @param applications requested applications
     * @return distinct categories
     */
    @ApiOperation(value = "Get distinct error categories from a package to enable front end category-based filtering.", tags = "Metadata")
    @GetMapping("packages/{packageName}/categories")
    public ResponseEntity<List<String>> getDistinctErrorCategoriesPackage(
            @ApiParam(value = "Name of the uploaded package.", required = true)
            @PathVariable String packageName,

            @ApiParam(value = "List of applications that the user wants to see. If null, logs from all are returned.")
            @RequestParam(required = false, defaultValue = "") List<String> applications
    ){
        Filter filter = createFilter(EMPTY_STRING, EMPTY_STRING, EMPTY_STRING, packageName,
                Collections.emptyList(), Collections.emptyList(), applications, 0L);
        List<String> distinctCategories = metadataProviderService.getDistinctCategories(filter);
        return new ResponseEntity<>(distinctCategories, HttpStatus.OK);
    }

    /**
     * ------------------------------------------ DISTINCT ERROR TYPES ------------------------------------------
     */

    /**
     * Distinct error types from the repository
     * @param categories categories to filter by
     * @param applications applications to filter by
     * @return distinct error types
     */
    @ApiOperation(value = "Get distinct error types to enable front end error-type-based filtering.", tags = "Metadata")
    @GetMapping("/errortypes")
    public ResponseEntity<List<DistinctErrorType>> getDistinctErrorTypes(
            @ApiParam(value = "List of categories that the user wants to see. If null, logs from all are returned.")
            @RequestParam(required = false, defaultValue = "") List<String> categories,

            @ApiParam(value = "List of applications that the user wants to see. If null, logs from all are returned.")
            @RequestParam(required = false, defaultValue = "") List<String> applications
    ){
        Filter filter = createFilter(EMPTY_STRING, EMPTY_STRING, EMPTY_STRING, EMPTY_STRING,
                categories, Collections.emptyList(), applications, 0L);
        List<DistinctErrorType> distinctErrorTypes = metadataProviderService.getDistinctErrorTypes(filter);
        return new ResponseEntity<>(distinctErrorTypes, HttpStatus.OK);
    }

    /**
     * Distinct error types from the repository
     * @param workflowExecutionId workflow execution id
     * @param categories categories to filter by
     * @param applications applications to filter by
     * @return distinct error types
     */
    @ApiOperation(value = "Get distinct error types from a workflow execution to enable front end error-type-based filtering.", tags = "Metadata")
    @GetMapping("/workflow-executions/{workflowExecutionId}/errortypes")
    public ResponseEntity<List<DistinctErrorType>> getDistinctErrorTypes(
            @ApiParam(value = "Workflow execution ID, passed by the process manager.", required = true)
            @PathVariable String workflowExecutionId,

            @ApiParam(value = "List of categories that the user wants to see. If null, logs from all are returned.")
            @RequestParam(required = false, defaultValue = "") List<String> categories,

            @ApiParam(value = "List of applications that the user wants to see. If null, logs from all are returned.")
            @RequestParam(required = false, defaultValue = "") List<String> applications
    ){
        Filter filter = createFilter(workflowExecutionId, EMPTY_STRING, EMPTY_STRING, EMPTY_STRING,
                categories, Collections.emptyList(), applications, 0L);
        List<DistinctErrorType> distinctErrorTypes = metadataProviderService.getDistinctErrorTypes(filter);
        return new ResponseEntity<>(distinctErrorTypes, HttpStatus.OK);
    }

    /**
     * Distinct error types from the repository
     * @param workflowExecutionId workflow execution id
     * @param scenarioExecutionId scenario execution id
     * @param categories categories to filter by
     * @param applications applications to filter by
     * @return distinct error types
     */
    @ApiOperation(value = "Get distinct error types from a scenario execution to enable front end error-type-based filtering.", tags = "Metadata")
    @GetMapping("/workflow-executions/{workflowExecutionId}/scenario-executions/{scenarioExecutionId}/errortypes")
    public ResponseEntity<List<DistinctErrorType>> getDistinctErrorTypes(
            @ApiParam(value = "Workflow execution ID, passed by the process manager.", required = true)
            @PathVariable String workflowExecutionId,

            @ApiParam(value = "Scenario execution ID, passed by the process manager.", required = true)
            @PathVariable String scenarioExecutionId,

            @ApiParam(value = "List of categories that the user wants to see. If null, logs from all are returned.")
            @RequestParam(required = false, defaultValue = "") List<String> categories,

            @ApiParam(value = "List of applications that the user wants to see. If null, logs from all are returned.")
            @RequestParam(required = false, defaultValue = "") List<String> applications
    ){
        Filter filter = createFilter(workflowExecutionId, scenarioExecutionId, EMPTY_STRING, EMPTY_STRING,
                categories, Collections.emptyList(), applications, 0L);
        List<DistinctErrorType> distinctErrorTypes = metadataProviderService.getDistinctErrorTypes(filter);
        return new ResponseEntity<>(distinctErrorTypes, HttpStatus.OK);
    }


    /**
     * Distinct error types from the repository
     * @param workflowExecutionId workflow execution id
     * @param scenarioExecutionId scenario execution id
     * @param connectionId connection id
     * @param categories categories to filter by
     * @param applications applications to filter by
     * @return distinct error types
     */
    @ApiOperation(value = "Get distinct error categories from a connection to enable front end category-based filtering.", tags = "Metadata")
    @GetMapping("/workflow-executions/{workflowExecutionId}/scenario-executions/{scenarioExecutionId}/connections/{connectionId}/errortypes")
    public ResponseEntity<List<DistinctErrorType>> getDistinctErrorTypes(
            @ApiParam(value = "Workflow execution ID, passed by the process manager.", required = true)
            @PathVariable String workflowExecutionId,

            @ApiParam(value = "Scenario execution ID, passed by the process manager.", required = true)
            @PathVariable String scenarioExecutionId,

            @ApiParam(value = "Name of the connection.", required = true)
            @PathVariable String connectionId,

            @ApiParam(value = "List of categories that the user wants to see. If null, logs from all are returned.")
            @RequestParam(required = false, defaultValue = "") List<String> categories,

            @ApiParam(value = "List of applications that the user wants to see. If null, logs from all are returned.")
            @RequestParam(required = false, defaultValue = "") List<String> applications
    ){
        Filter filter = createFilter(workflowExecutionId, scenarioExecutionId, connectionId, EMPTY_STRING,
                categories, Collections.emptyList(), applications, 0L);
        List<DistinctErrorType> distinctErrorTypes = metadataProviderService.getDistinctErrorTypes(filter);
        return new ResponseEntity<>(distinctErrorTypes, HttpStatus.OK);
    }

    /**
     * Distinct error types from an uploaded package
     * @param packageName name of the package
     * @param categories categories to filter by
     * @param applications applications to filter by
     * @return distinct error types
     */
    @ApiOperation(value = "Get distinct error types from a package to enable front end error-type-based filtering.", tags = "Metadata")
    @GetMapping("packages/{packageName}/errortypes")
    public ResponseEntity<List<DistinctErrorType>> getDistinctErrorTypesPackage(
            @ApiParam(value = "Name of the uploaded package.", required = true)
            @PathVariable String packageName,

            @ApiParam(value = "List of categories that the user wants to see. If null, logs from all are returned.")
            @RequestParam(required = false, defaultValue = "") List<String> categories,

            @ApiParam(value = "List of applications that the user wants to see. If null, logs from all are returned.")
            @RequestParam(required = false, defaultValue = "") List<String> applications

    ){
        Filter filter = createFilter(EMPTY_STRING, EMPTY_STRING, EMPTY_STRING, packageName,
                categories, Collections.emptyList(), applications, 0L);
        List<DistinctErrorType> distinctErrorTypes = metadataProviderService.getDistinctErrorTypes(filter);
        return new ResponseEntity<>(distinctErrorTypes, HttpStatus.OK);
    }
}
