package eu.profinit.manta.bachelor.thesis.log.viewer.controllers;

import java.io.File;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.FileSystemResource;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.multipart.MultipartFile;

import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.CreatePackageService;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.UploadPackageService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;

/**
 * Controller for requests regarding log package upload/download.
 * @author jkovac
 */
@Api(tags = "Packaging")
@Controller
public final class PackageController extends LogViewerBaseController {

    /**
     * Service providing package upload capability
     */
    private UploadPackageService uploadPackageService;

    /**
     * Service providing package creation capability
     */
    private CreatePackageService createPackageService;

    /**
     * Ctr for package controller
     * @param uploadPackageService Service providing package upload capability
     * @param createPackageService Service providing package creation capability
     */
    @Autowired
    public PackageController(UploadPackageService uploadPackageService, CreatePackageService createPackageService) {
        this.uploadPackageService = uploadPackageService;
        this.createPackageService = createPackageService;
    }

    /**
     * Request to create package for a given connection
     * @param workflowExecutionId ID of the workflow execution
     * @param scenarioExecutionId ID of the scenario execution
     * @param connectionId ID of the connection
     * @return ZIP-package containing scenario execution metadata and the database dump with logs
     */
    @ApiOperation(value = "Creates a zip file containing dump of logs for a given scenario execution.", tags = "Packaging")
    @GetMapping(value = "/workflow-executions/{workflowExecutionId}/scenario-executions/{scenarioExecutionId}/connection/{connectionId}/package",
            produces = "application/zip")
    @ResponseBody
    public FileSystemResource createPackageForConnection(
            @ApiParam(value = "Workflow execution ID of the requested scenario.", required = true)
            @PathVariable String workflowExecutionId,

            @ApiParam(value = "Execution ID of the requested scenario of which the connection is part of.", required = true)
            @PathVariable String scenarioExecutionId,

            @ApiParam(value = "Connection ID of the requested connection.", required = true)
            @PathVariable String connectionId
    ){
        File file = createPackageService.createLogsPackageForConnection(workflowExecutionId, scenarioExecutionId, connectionId);
        return new FileSystemResource(file);
    }

    /**
     * Request to create package for a given application.
     * @param application Name of the application.
     * @return error response
     */
    @ApiOperation(value = "Creates a zip file containing dump of logs for a given application.", tags = "Packaging")
    @GetMapping(value = "/applications/{application}/package", produces = "application/zip")
    @ResponseBody
    public FileSystemResource createPackageForApplication(
            @ApiParam(value = "Name of the application for which the package should be created.", required = true)
            @PathVariable String application
    ){
        File file = createPackageService.createLogsPackageForApplication(application);
        return new FileSystemResource(file);
    }

    /**
     * Request to uploade a log package to the repository.
     * @param multipartFile Uploaded file
     * @param packageName Name of the package under which logs will be stored.
     */
    @ApiOperation(value = "Upload a given log package.", tags = "Packaging")
    @PostMapping(value = "/upload")
    @ResponseStatus(value = HttpStatus.OK)
    public void uploadPackage(
            @ApiParam(value = "Uploaded zip file containing log dump.", required = true)
            @RequestParam(value = "uploadedPackage") MultipartFile multipartFile,

            @ApiParam(value = "Name under which the package will be shown.", required = true)
            @RequestParam(value = "packageName") String packageName
    ){
        uploadPackageService.uploadPackage(multipartFile, packageName);
    }
}
