package eu.profinit.manta.bachelor.thesis.log.viewer.controllers;

import javax.validation.constraints.NotEmpty;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.ResponseStatus;

import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.ForcePruneService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;

/**
 * Controller for requests regarding log pruning.
 * @author jkovac
 */
@Api(tags = "Pruning")
@Controller
public final class PruningController extends LogViewerBaseController {

    /**
     * Service providing log deletion by request
     */
    private ForcePruneService forcePruneService;

    /**
     * Ctr for pruning controller
     * @param forcePruneService Service providing log deletion by request
     */
    @Autowired
    public PruningController(ForcePruneService forcePruneService) {
        this.forcePruneService = forcePruneService;
    }

    /**
     * Request to delete logs from a given workflow execution
     * @param workflowExecutionId ID of the workflow execution
     */
    @ApiOperation(value = "Force delete logs from a given workflow.", tags = "Pruning")
    @DeleteMapping(value = "/workflow-executions/{workflowExecutionId}")
    @ResponseStatus(value = HttpStatus.OK)
    public void deleteWorkflowLogs(
            @ApiParam(value = "ID of the workflow execution for which logs should be deleted.", required = true)
            @PathVariable @NotEmpty String workflowExecutionId
    ){
        forcePruneService.forcePruneWorkflow(workflowExecutionId);
    }

    /**
     * Request to delete logs from a given application.
     * @param application Name of the application
     */
    @ApiOperation(value = "Force delete logs from a given application.", tags = "Pruning")
    @DeleteMapping(value = "/applications/{application}")
    @ResponseStatus(value = HttpStatus.OK)
    public void deleteApplicationLogs(
            @ApiParam(value = "Name of the application for which logs should be deleted.", required = true)
            @PathVariable @NotEmpty String application
    ){
        forcePruneService.forcePruneApplication(application);
    }

    /**
     * Request to delete logs from a given package.
     * @param packageName Name of the uploaded package.
     */
    @ApiOperation(value = "Delete uploaded logs stored under a given name.", tags = "Pruning")
    @DeleteMapping(value = "/packages/{packageName}")
    @ResponseStatus(value = HttpStatus.OK)
    public void deletePackageLogs(
            @ApiParam(value = "Name of the uploaded package that was provided by a user before its upload.", required = true)
            @PathVariable @NotEmpty String packageName
    ){
        forcePruneService.pruneUploadedPackage(packageName);
    }

    /**
     * Request to delete logs from a given workflow execution
     * @param workflowExecutionId ID of the workflow execution
     * @param scenarioExecutionId scenario execution ID
     */
    @ApiOperation(value = "Delete logs from a given scenario execution.", tags = "Pruning")
    @DeleteMapping(value = "/workflow-executions/{workflowExecutionId}/scenario-executions/{scenarioExecutionId}")
    @ResponseStatus(value = HttpStatus.OK)
    public void deleteScenarioExecution(
            @ApiParam(value = "ID of the workflow execution of which the scenario execution is part of.", required = true)
            @PathVariable @NotEmpty String workflowExecutionId,

            @ApiParam(value = "ID of the scenario execution for which logs should be deleted.", required = true)
            @PathVariable @NotEmpty String scenarioExecutionId
    ){
        forcePruneService.forcePruneScenarioExecution(scenarioExecutionId);
    }

    /**
     * Request to delete logs from a given workflow execution
     * @param workflowExecutionId ID of the workflow execution
     * @param scenarioExecutionId scenario execution ID
     * @param connectionId connection ID
     */
    @ApiOperation(value = "Delete logs from a given connection for a given scenario execution.", tags = "Pruning")
    @DeleteMapping(value = "/workflow-executions/{workflowExecutionId}/scenario-executions/{scenarioExecutionId}/connections/{connectionId}")
    @ResponseStatus(value = HttpStatus.OK)
    public void deleteConnectionFromScenarioExecution(
            @ApiParam(value = "ID of the workflow execution of which the scenario execution is part of.", required = true)
            @PathVariable @NotEmpty String workflowExecutionId,

            @ApiParam(value = "ID of the scenario execution of which the connection is part of.", required = true)
            @PathVariable @NotEmpty String scenarioExecutionId,

            @ApiParam(value = "ID of the connection for which logs should be deleted.", required = true)
            @PathVariable @NotEmpty String connectionId
    ){
        forcePruneService.forcePruneConnection(workflowExecutionId, scenarioExecutionId, connectionId);
    }
}
