package eu.profinit.manta.bachelor.thesis.log.viewer.model;

import org.apache.ibatis.annotations.Param;

/**
 * POJO representing error metadata, i.e. error occurring in a particular setting, used for fast filtering.
 * @author jkovac
 */
public class ErrorMetadata {

    /**
     * DB id of the error metadata
     */
    private Long errorId;

    /**
     * ID of the corresponding error definition
     */
    private Long errorDefinitionId;

    /**
     * Workflow (run) ID of which the scenario is a part of
     */
    private String workflowExecutionId;

    /**
     * Execution ID of a scenario to which the scenario belongs
     */
    private String scenarioExecutionId;

    /**
     * ConnectionId provided from the LoggingContext
     */
    private Long connDbId;

    /**
     * Name of the package
     */
    private String packageName;

    /**
     * Application that emitted the log
     */
    private String application;

    /**
     * Error definition
     */
    private ErrorDefinition errorDefinition;

    /**
     * Ctr for the error metadata
     * @param application Application that emitted the log
     * @param errorCategory error category
     * @param errorType error type
     * @param userMessage user error message
     * @param technicalMessage technical error message
     * @param solution solution
     * @param impact impact on the lineage
     * @param logLevel severity
     */
    public ErrorMetadata(@Param("application") String application,
                         @Param("errorCategory") String errorCategory,
                         @Param("errorType") String errorType,
                         @Param("userMessage") String userMessage,
                         @Param("technicalMessage") String technicalMessage,
                         @Param("solution") String solution,
                         @Param("impact") String impact,
                         @Param("logLevel") String logLevel) {
        this.application = application;
        this.errorDefinition = new ErrorDefinition(
                errorCategory,
                errorType,
                userMessage,
                technicalMessage,
                solution,
                impact,
                logLevel
        );
    }


    /**
     * Ctr for the error metadata
     * @param errorDefinitionId db id of the error definition
     * @param workflowExecutionId workflow execution id
     * @param scenarioExecutionId scenario execution id
     * @param connDbId db id of the connection
     * @param packageName package name
     * @param application application
     */
    public ErrorMetadata(@Param("errorDefinitionId") Long errorDefinitionId,
                         @Param("workflowExecutionId") String workflowExecutionId,
                         @Param("scenarioExecutionId") String scenarioExecutionId,
                         @Param("connDbId") Long connDbId,
                         @Param("packageName") String packageName,
                         @Param("application") String application) {
        this.errorDefinitionId = errorDefinitionId;
        this.workflowExecutionId = workflowExecutionId;
        this.scenarioExecutionId = scenarioExecutionId;
        this.connDbId = connDbId;
        this.packageName = packageName;
        this.application = application;
    }

    /**
     * Ctr for the error metadata
     * @param errorDefinitionId db id of the error definition
     * @param packageName package name
     * @param application application
     */
    public ErrorMetadata(Long errorDefinitionId, String packageName, String application) {
        this.errorDefinitionId = errorDefinitionId;
        this.packageName = packageName;
        this.application = application;
    }

    /**
     * Gets error definition id.
     *
     * @return the error definition id
     */
    public Long getErrorDefinitionId() {
        return errorDefinitionId;
    }

    /**
     * Gets error id.
     *
     * @return the error id
     */
    public Long getErrorId() {
        return errorId;
    }

    /**
     * Sets error id, used by MyBatis
     *
     * @param errorId the error id
     */
    public void setErrorId(Long errorId) {
        this.errorId = errorId;
    }

    /**
     * Gets workflow execution id.
     *
     * @return the workflow execution id
     */
    public String getWorkflowExecutionId() {
        return workflowExecutionId;
    }

    /**
     * Gets scenario execution id.
     *
     * @return the scenario execution id
     */
    public String getScenarioExecutionId() {
        return scenarioExecutionId;
    }

    /**
     * Gets conn db id.
     *
     * @return the conn db id
     */
    public Long getConnDbId() {
        return connDbId;
    }

    /**
     * Gets package name.
     *
     * @return the package name
     */
    public String getPackageName() {
        return packageName;
    }

    /**
     * Gets application.
     *
     * @return the application
     */
    public String getApplication() {
        return application;
    }

    /**
     * Gets error definition.
     *
     * @return the error definition
     */
    public ErrorDefinition getErrorDefinition() {
        return errorDefinition;
    }
}
