package eu.profinit.manta.bachelor.thesis.log.viewer.model.filter;

import java.util.ArrayList;
import java.util.List;

/**
 * POJO representing filter settings passed with a request
 */
public class Filter {

    /**
     * List of categories to filter by
     */
    private List<String> categories;

    /**
     * List of error types to filter by
     */
    private List<String> errorTypes;

    /**
     * List of applications to get applications from, if empty all are returned
     */
    private List<String> applications;

    /**
     * ID of the requested workflow execution
     */
    private String workflowExecutionId;

    /**
     * ID of the requested scenario execution
     */
    private String scenarioExecutionId;

    /**
     * ID of the requested connection
     */
    private String connectionId;

    /**
     * Name of the uploaded package
     */
    private String packageName;

    /**
     * Amount of already loaded elements
     */
    private Long offset;

    /**
     * Fitler ctr
     * @param categories List of categories to filter by
     * @param errorTypes List of error types to filter by
     * @param applications List of applications to get applications from, if empty all are returned
     * @param workflowExecutionId ID of the requested workflow execution
     * @param scenarioExecutionId ID of the requested scenario execution
     * @param connectionId ID of the requested connection
     * @param packageName Name of the uploaded package
     * @param offset Amount of already loaded elements
     */
    private Filter(List<String> categories,
                   List<String> errorTypes,
                   List<String> applications,
                   String workflowExecutionId,
                   String scenarioExecutionId,
                   String connectionId,
                   String packageName,
                   Long offset) {
        this.categories = categories;
        this.errorTypes = errorTypes;
        this.applications = applications;
        this.workflowExecutionId = workflowExecutionId;
        this.scenarioExecutionId = scenarioExecutionId;
        this.connectionId = connectionId;
        this.packageName = packageName;
        this.offset = offset;
    }


    /**
     * Gets workflow execution id.
     *
     * @return the workflow execution id
     */
    public String getWorkflowExecutionId() {
        return workflowExecutionId;
    }

    /**
     * Gets package name.
     *
     * @return the package name
     */
    public String getPackageName() {
        return packageName;
    }

    /**
     * Gets categories.
     *
     * @return the categories
     */
    public List<String> getCategories() {
        return categories;
    }

    /**
     * Gets error types.
     *
     * @return the error types
     */
    public List<String> getErrorTypes() {
        return errorTypes;
    }

    /**
     * Gets applications.
     *
     * @return the applications
     */
    public List<String> getApplications() {
        return applications;
    }

    /**
     * Gets scenario execution id.
     *
     * @return the scenario execution id
     */
    public String getScenarioExecutionId() {
        return scenarioExecutionId;
    }

    /**
     * Gets connection id.
     *
     * @return the connection id
     */
    public String getConnectionId() {
        return connectionId;
    }

    /**
     * Gets offset.
     *
     * @return the offset
     */
    public Long getOffset() {
        return offset;
    }

    /**
     * Is empty boolean.
     *
     * @return the boolean
     */
    public boolean isEmpty(){
        return  categories.isEmpty() &
                errorTypes.isEmpty() &
                applications.isEmpty() &
                workflowExecutionId.isEmpty() &
                scenarioExecutionId.isEmpty() &
                connectionId.isEmpty() &
                packageName.isEmpty();
    }

    /**
     * Builder for filter.
     * @author jkovac
     */
    public static class FilterBuilder {

        /**
         * List of categories to filter by
         */
        private List<String> categories;

        /**
         * List of error types to filter by
         */
        private List<String> errorTypes;

        /**
         * List of applications to get applications from, if empty all are returned
         */
        private List<String> applications;

        /**
         * ID of the requested workflow execution
         */
        private String workflowExecutionId;

        /**
         * ID of the requested scenario execution
         */
        private String scenarioExecutionId;

        /**
         * ID of the requested connection
         */
        private String connectionId;

        /**
         * Name of the uploaded package
         */
        private String packageName;

        /**
         * Amount of already loaded elements
         */
        private Long offset;

        /**
         * Instantiates a new Filter builder.
         */
        public FilterBuilder() {
            this.categories = new ArrayList<>();
            this.errorTypes = new ArrayList<>();
            this.applications = new ArrayList<>();
            this.workflowExecutionId = "";
            this.scenarioExecutionId = "";
            this.connectionId = "";
            this.packageName = "";
            this.offset = 0L;
        }

        /**
         * Build filter.
         *
         * @return the filter
         */
        public Filter buildFilter(){
            Filter filter = new Filter(
                    categories,
                    errorTypes,
                    applications,
                    workflowExecutionId,
                    scenarioExecutionId,
                    connectionId,
                    packageName,
                    offset
            );
            return filter;
        }

        /**
         * Sets categories.
         *
         * @param categories the categories
         */
        public void setCategories(List<String> categories) {
            this.categories = categories;
        }

        /**
         * Sets error types.
         *
         * @param errorTypes the error types
         */
        public void setErrorTypes(List<String> errorTypes) {
            this.errorTypes = errorTypes;
        }

        /**
         * Sets applications.
         *
         * @param applications the applications
         */
        public void setApplications(List<String> applications) {
            this.applications = applications;
        }

        /**
         * Sets workflow execution id.
         *
         * @param workflowExecutionId the workflow execution id
         */
        public void setWorkflowExecutionId(String workflowExecutionId) {
            this.workflowExecutionId = workflowExecutionId;
        }

        /**
         * Sets scenario execution id.
         *
         * @param scenarioExecutionId the scenario execution id
         */
        public void setScenarioExecutionId(String scenarioExecutionId) {
            this.scenarioExecutionId = scenarioExecutionId;
        }

        /**
         * Sets connection id.
         *
         * @param connectionId the connection id
         */
        public void setConnectionId(String connectionId) {
            this.connectionId = connectionId;
        }

        /**
         * Sets package name.
         *
         * @param packageName the package name
         */
        public void setPackageName(String packageName) {
            this.packageName = packageName;
        }

        /**
         * Sets offset.
         *
         * @param offset the offset
         */
        public void setOffset(Long offset) {
            this.offset = offset;
        }
    }
}
