package eu.profinit.manta.bachelor.thesis.log.viewer.model.logs;

import java.util.Optional;

import org.apache.ibatis.annotations.Param;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonProperty;

import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.additionalcontext.BaseContextMap;
import eu.profinit.manta.bachelor.thesis.log.viewer.broker.messagehandling.MessageHandler;

/**
 * Class representing logs with severity TRACE, DEBUG and INFO.
 * @author jkovac
 */
public class DebugLog extends Log {

    /**
     * Logged exception
     */
    private LoggedExceptionData exception;

    /**
     * Default constructor
     */
    public DebugLog(){
        exception = new LoggedExceptionData();
    }

    /**
     * Used due to possibility of unknown log types, which are by default stored as DEBUG logs.
     */
    private static String DEBUG_TYPE = "DEBUG";

    /**
     * Log with level TRACE, DEBUG or INFO.
     *
     * @param type           DEBUG
     * @param thread         Name of the thread which created the log
     * @param threadId       Id of the thread which created the log
     * @param level          Log level
     * @param loggerName     Name of the logger from which the log was emitted
     * @param instant        Log time
     * @param message        Message of the log
     * @param exception      Logged exception
     * @param debugData      Holder with additional thread context data.
     * @param application    Application which emitted the log
     */
    @JsonCreator
    public DebugLog(@JsonProperty("type") String type,
                    @JsonProperty("thread") String thread,
                    @JsonProperty("threadId") int threadId,
                    @JsonProperty("level") String level,
                    @JsonProperty("loggerName") String loggerName,
                    @JsonProperty("instant") Instant instant,
                    @JsonProperty("message") String message,
                    @JsonProperty("thrown") LoggedExceptionData exception,
                    @JsonProperty("APPLICATION") String application,
                    @JsonProperty("contextMap") BaseContextMap debugData) {
        super(DEBUG_TYPE, thread, threadId, level, loggerName, instant, message, application, debugData);

        this.exception = Optional.ofNullable(exception).orElse(new LoggedExceptionData());
    }

    /**
     * Instantiates a new Debug log.
     *
     * @param logId               the log id
     * @param type                the type
     * @param thread              the thread
     * @param threadId            the thread id
     * @param loggerName          the logger name
     * @param level               the level
     * @param epochSecond         the epoch second
     * @param nanoOfSecond        the nano of second
     * @param message             the message
     * @param application         the application
     * @param technology          the technology
     * @param phase               the phase
     * @param scenario            the scenario
     * @param workflowExecutionId the workflow execution id
     * @param scenarioExecutionId the scenario execution id
     * @param connectionId        the connection id
     * @param context             the context
     * @param workerNumber        the worker number
     * @param exceptionName       the exception name
     * @param extendedStacktrace  the extended stacktrace
     * @param packageName         the package name
     */
    public DebugLog(@Param("logId") Long logId,
                    @Param("type") String type,
                    @Param("thread") String thread,
                    @Param("threadId") int threadId,
                    @Param("level") String level,
                    @Param("loggerName") String loggerName,
                    @Param("epochSecond") long epochSecond,
                    @Param("nanoOfSecond") long nanoOfSecond,
                    @Param("message") String message,
                    @Param("application") String application,
                    @Param("technology") String technology,
                    @Param("phase") String phase,
                    @Param("scenario") String scenario,
                    @Param("workflowExecutionId") String workflowExecutionId,
                    @Param("scenarioExecutionId") String scenarioExecutionId,
                    @Param("connectionId") String connectionId,
                    @Param("context") String context,
                    @Param("workerNumber") int workerNumber,
                    @Param("exceptionName") String exceptionName,
                    @Param("extendedStacktrace") String extendedStacktrace,
                    @Param("packageName") String packageName){

        super(logId, type, thread, threadId, level, loggerName, new Instant(epochSecond, nanoOfSecond), message, application, new BaseContextMap(
                workflowExecutionId, scenarioExecutionId, technology, phase, scenario, connectionId, context, workerNumber, packageName
        ));

        this.exception = new LoggedExceptionData(exceptionName, extendedStacktrace);
    }

    @Override
    public void handle(MessageHandler handler) {
        handler.handle(this);
    }

    /**
     * @return name of the logged exception
     */
    public String getExceptionName(){
        return exception.getExceptionName();
    }

    /**
     * @return extended stacktrace of the logged exception
     */
    public String getExtendedStacktrace(){
        return exception.getExtendedStackTrace();
    }
}
