package eu.profinit.manta.bachelor.thesis.log.viewer.model.logs;

import java.util.Optional;

import org.apache.ibatis.annotations.Param;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonProperty;

import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.additionalcontext.ErrorContextMap;
import eu.profinit.manta.bachelor.thesis.log.viewer.broker.messagehandling.MessageHandler;

/**
 * Log with level WARN, ERROR and FATAL.
 * @author jkovac
 */
public class ErrorLog extends Log {

    /**
     * Logged exception
     */
    private LoggedExceptionData exception;

    /**
     * Holder of error data from ThreadContext
     */
    private ErrorContextMap errorData;

    /**
     * Default constructor
     */
    public ErrorLog(){
        this.errorData = new ErrorContextMap();
        this.exception = new LoggedExceptionData();
    }

    /**
     * Log with level WARN, ERROR and FATAL
     * @param thread Name of the thread which created the log
     * @param threadId Id of the thread which created the log
     * @param level Log level
     * @param loggerName Name of the logger from which the log was emitted
     * @param instant Log time
     * @param message Message of the log
     * @param exception Logged exception
     * @param errorData data from ThreadContext
     * @param application application that emitted the log
     * @param type log type
     */
    @JsonCreator
    public ErrorLog(@JsonProperty("type") String type,
                    @JsonProperty("thread") String thread,
                    @JsonProperty("threadId") int threadId,
                    @JsonProperty("level") String level,
                    @JsonProperty("loggerName") String loggerName,
                    @JsonProperty("instant") Instant instant,
                    @JsonProperty("message") String message,
                    @JsonProperty("thrown") LoggedExceptionData exception,
                    @JsonProperty("APPLICATION") String application,
                    @JsonProperty("contextMap") ErrorContextMap errorData) {
        super(type, thread, threadId, level, loggerName, instant, message, application, errorData);

        this.exception = Optional.ofNullable(exception).orElse(new LoggedExceptionData());
        this.errorData = errorData;
    }

    /**
     * Log with level WARN, ERROR and FATAL
     * @param thread Name of the thread which created the log
     * @param threadId Id of the thread which created the log
     * @param level Log level
     * @param loggerName Name of the logger from which the log was emitted
     * @param message Message of the log
     * @param errorData data from ThreadContext
     * @param application application that emitted the log
     * @param epochSecond timestamp in epoch seconds
     * @param logId primary key of the log
     * @param nanoOfSecond nano seconds of the timestamp
     * @param type ERROR
     */
    public ErrorLog(Long logId, String type, String thread, int threadId, String level, String loggerName,
                    long epochSecond, long nanoOfSecond, String message, String application, ErrorContextMap errorData){

        super(logId, type, thread, threadId, level, loggerName, new Instant(epochSecond, nanoOfSecond),
                message, application, errorData);

        this.errorData = errorData;
    }

    /**
     * Log with level WARN, ERROR and FATAL
     * @param thread Name of the thread which created the log
     * @param threadId Id of the thread which created the log
     * @param level Log level
     * @param loggerName Name of the logger from which the log was emitted
     * @param message Message of the log
     * @param application application that emitted the log
     * @param epochSecond timestamp in epoch seconds
     * @param logId primary key of the log
     * @param nanoOfSecond nano seconds of the timestamp
     * @param type ERROR
     * @param packageName name of the package to which the log belogns
     * @param phase name of the phase of the scenario
     * @param technology technology of the scenario
     * @param connectionId connection ID
     * @param scenarioExecutionId scenario execution iD
     * @param workerNumber worker number
     * @param category error category
     * @param context context provided with the log
     * @param errorType error type
     * @param exceptionName name of the exception
     * @param extendedStacktrace stacktrace
     * @param lineageImpact impact on the data lineage
     * @param scenario name of the scenario
     * @param solution solution to the error
     * @param technicalMessage technical error message
     * @param userMessage user error message
     * @param workflowExecutionId workflow execution ID
     */
    public ErrorLog(@Param("logId") Long logId,
                    @Param("type") String type,
                    @Param("thread") String thread,
                    @Param("threadId") int threadId,
                    @Param("level") String level,
                    @Param("loggerName") String loggerName,
                    @Param("epochSecond") long epochSecond,
                    @Param("nanoOfSecond") long nanoOfSecond,
                    @Param("message") String message,
                    @Param("application") String application,
                    @Param("technology") String technology,
                    @Param("phase") String phase,
                    @Param("scenario") String scenario,
                    @Param("workflowExecutionId") String workflowExecutionId,
                    @Param("scenarioExecutionId") String scenarioExecutionId,
                    @Param("connectionId") String connectionId,
                    @Param("context") String context,
                    @Param("workerNumber") int workerNumber,
                    @Param("exceptionName") String exceptionName,
                    @Param("extendedStacktrace") String extendedStacktrace,
                    @Param("userMessage") String userMessage,
                    @Param("technicalMessage") String technicalMessage,
                    @Param("solution") String solution,
                    @Param("lineageImpact") String lineageImpact,
                    @Param("category") String category,
                    @Param("errorType") String errorType,
                    @Param("packageName") String packageName){
        this(logId, type, thread, threadId, level, loggerName, epochSecond, nanoOfSecond, message, application,
                new ErrorContextMap(workflowExecutionId, scenarioExecutionId, technology, connectionId, phase, scenario, context,
                        workerNumber, userMessage, technicalMessage, solution, lineageImpact, category, errorType, packageName));

        this.exception = new LoggedExceptionData(exceptionName, extendedStacktrace);
    }

    /**
     * @return User-friendly error message
     */
    public String getUserMessage() {
        return errorData.getUserMessage();
    }

    /**
     * @return Message with technical details
     */
    public String getTechnicalMessage() {
        return errorData.getTechnicalMessage();
    }

    /**
     * @return Solution for the error
     */
    public String getSolution() {
        return errorData.getSolution();
    }

    /**
     * @return Impact of the error on the data lineage
     */
    public String getLineageImpact() {
        return errorData.getLineageImpact();
    }

    /**
     * @return Category of the error
     */
    public String getCategory() {
        return errorData.getCategory();
    }

    /**
     * @return Name of the error
     */
    public String getErrorType() {
        return errorData.getErrorType();
    }

    /**
     * @return name of the logged exception
     */
    public String getExceptionName(){
        return exception.getExceptionName();
    }

    /**
     * @return extended stacktrace of the logged exception
     */
    public String getExtendedStacktrace(){
        return exception.getExtendedStackTrace();
    }

    @Override
    public void handle(MessageHandler handler) {
        handler.handle(this);
    }
}
