package eu.profinit.manta.bachelor.thesis.log.viewer.model.logs;

import com.fasterxml.jackson.annotation.JsonSubTypes;
import com.fasterxml.jackson.annotation.JsonSubTypes.Type;
import com.fasterxml.jackson.annotation.JsonTypeInfo;

import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.additionalcontext.BaseContextMap;
import eu.profinit.manta.bachelor.thesis.log.viewer.broker.messagehandling.Handleable;

/**
 * Parent class for all incoming logs.
 * @author jkovac
 */
@JsonTypeInfo(
        use = JsonTypeInfo.Id.NAME,
        property = "type",
        defaultImpl = DebugLog.class,
        visible = true) // defaultImpl for backwards compatibility with old logging
@JsonSubTypes({
        @Type(value = DebugLog.class, name = "DEBUG"),
        @Type(value = ErrorLog.class, name = "ERROR")
})
public abstract class Log implements Handleable {

    /**
     * db id of the log
     */
    private Long logId;

    /**
     * Type of the log, can be DEBUG, ERROR or PROGRESS
     */
    private String type;

    /**
     * Name of the thread which created the log
     */
    private String thread;

    /**
     * Id of the thread which created the log
     */
    private int threadId;

    /**
     * Log level
     */
    private String level;

    /**
     * Name of the logger from which the log was emitted
     */
    private String loggerName;

    /**
     * Log time
     */
    private Instant instant;

    /**
     * Message of the log
     */
    private String message;

    /**
     * Name of the application which emitted the log
     */
    private String application;

    /**
     * Holder of data from ThreadContext
     */
    private BaseContextMap contextData;

    /**
     * Default constructor
     */
    public Log(){
        contextData = new BaseContextMap();
        instant = new Instant();
    }

    /**
     * Parent class for all incoming logs.
     * @param thread Name of the thread which created the log
     * @param threadId Id of the thread which created the log
     * @param level Log level
     * @param loggerName Name of the logger from which the log was emitted
     * @param instant Log time
     * @param message Message of the log
     * @param application Name of the application which emitted the log
     * @param contextData values from ThreadContext
     * @param type log type
     */
    public Log(String type, String thread, int threadId, String level, String loggerName,
               Instant instant, String message, String application, BaseContextMap contextData) {
        this.type = type;
        this.thread = thread;
        this.threadId = threadId;
        this.level = level;
        this.loggerName = loggerName;
        this.instant = instant;
        this.message = message;
        this.contextData = contextData;
        this.application = application;
    }


    /**
     * Instantiates a new Log.
     *
     * @param logId       the log id
     * @param type        the type
     * @param thread      the thread
     * @param threadId    the thread id
     * @param level       the level
     * @param loggerName  the logger name
     * @param instant     the instant
     * @param message     the message
     * @param application the application
     * @param contextData the context data
     */
    public Log(Long logId, String type, String thread, int threadId, String level, String loggerName,
               Instant instant, String message, String application, BaseContextMap contextData) {
        this(type, thread, threadId, level, loggerName, instant, message, application, contextData);
        this.logId = logId;
    }

    /**
     * Parent class for all incoming logs.
     * @param thread Name of the thread which created the log
     * @param threadId Id of the thread which created the log
     * @param level Log level
     * @param loggerName Name of the logger from which the log was emitted
     * @param message Message of the log
     * @param application Name of the application which emitted the log
     * @param contextData values from ThreadContext
     * @param nanoOfSecond nano seconds of the timestamps
     * @param type type of the incoming log
     * @param epochSecond timestamp in epoch seconds
     * @param logId db id of the log
     */
    public Log(Long logId, String type, String thread, int threadId, String level, String loggerName,
               long epochSecond, long nanoOfSecond, String message, String application, BaseContextMap contextData){
        this.logId = logId;
        this.type = type;
        this.thread = thread;
        this.threadId = threadId;
        this.loggerName = loggerName;
        this.level = level;
        this.instant = new Instant(epochSecond, nanoOfSecond);
        this.message = message;
        this.application = application;
        this.contextData = contextData;
    }

    /**
     * @return DB id of the log
     */
    public Long getLogId() {
        return logId;
    }

    /**
     * Setter for DB id, used by the MyBatis after insert
     * @param logId db log id
     */
    public void setLogId(Long logId) {
        this.logId = logId;
    }

    /**
     * @return Name of the thread which created the log
     */
    public String getThread() {
        return thread;
    }

    /**
     * @return Id of the thread which created the log
     */
    public int getThreadId() {
        return threadId;
    }

    /**
     * @return Log level
     */
    public String getLevel() {
        return level;
    }

    /**
     * @return Name of the logger from which the log was emitted
     */
    public String getLoggerName() {
        return loggerName;
    }

    /**
     * @return Message of the log
     */
    public String getMessage() {
        return message;
    }

    /**
     * @return Workflow (run) ID of which the scenario is a part of
     */
    public String getWorkflowExecutionId(){
        return contextData.getWorkflowExecutionId();
    }

    /**
     * @return execution id of a scenario to which the log belongs
     */
    public String getScenarioExecutionId(){
        return contextData.getScenarioExecutionId();
    }

    /**
     * @return identificator of a connection
     */
    public String getConnectionId(){
        return contextData.getConnectionId();
    }

    /**
     * @return application which emitted the log
     */
    public String getApplication() {
        return application;
    }

    /**
     * @return context of the log
     */
    public String getContext() {
        return contextData.getContext();
    }

    /**
     * @return Log time in epoch seconds
     */
    public long getEpochSecond(){
        return instant.getEpochSecond();
    }

    /**
     * @return nano seconds
     */
    public long getNanoSecond(){
        return instant.getNanoOfSecond();
    }

    /**
     * @return Worker number
     */
    public int getWorkerNumber(){ return contextData.getWorkerNumber(); }

    /**
     * @return Technology provided from the LoggingContext
     */
    public String getTechnology(){
        return contextData.getTechnology();
    }

    /**
     * @return Phase of the scenario provided from the LoggingContext
     */
    public String getPhase(){
        return contextData.getPhase();
    }

    /**
     * @return Name of the scenario
     */
    public String getScenario(){
        return contextData.getScenario();
    }

    /**
     * @return type of the log (DEBUG, ERROR or PROGRESS)
     */
    public String getType(){ return type; }

    /**
     * @return Name of the package
     */
    public String getPackageName(){
        return contextData.getPackageName();
    }
}
