package eu.profinit.manta.bachelor.thesis.log.viewer.repository.config;

import javax.sql.DataSource;

import org.apache.commons.dbcp2.BasicDataSource;
import org.apache.ibatis.session.SqlSessionFactory;
import org.mybatis.spring.SqlSessionFactoryBean;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.PropertySource;
import org.springframework.core.io.ClassPathResource;
import org.springframework.jdbc.datasource.DataSourceTransactionManager;
import org.springframework.transaction.annotation.EnableTransactionManagement;

import liquibase.integration.spring.SpringLiquibase;

/**
 * Configuration of the data source
 * @author jkovac
 */
@Configuration
@PropertySource("classpath:repository.properties")
@EnableTransactionManagement
public class DataSourceConfig {

    /**
     * JDBC driver
     */
    @Value("${spring.datasource.driver}")
    private String dataSourceDriver;

    /**
     * Path to the H2 database file
     */
    @Value("${spring.datasource.url}")
    private String dataSourceUrl;

    /**
     * Username for connecting to the db
     */
    @Value("${spring.datasource.username}")
    private String dbUsername;

    /**
     * Password for connecting to the db
     */
    @Value("${spring.datasource.password}")
    private String dbPassword;

    /**
     * DB connection
     * @return basic datasource
     */
    @Bean
    public DataSource dataSource(){
        BasicDataSource dataSource = new BasicDataSource();
        dataSource.setDriverClassName(dataSourceDriver);
        dataSource.setUrl(dataSourceUrl);
        dataSource.setUsername(dbUsername);
        dataSource.setPassword(dbPassword);
        return dataSource;
    }

    /**
     * Spring liquibase bean
     * @param dataSource DB connection
     * @return spring liquibase
     */
    @Bean("liquibase")
    public SpringLiquibase liquibase(DataSource dataSource) {
        SpringLiquibase liquibase = new SpringLiquibase();
        liquibase.setChangeLog("classpath:/repository/liquibase-changelog.xml");
        liquibase.setDataSource(dataSource);
        return liquibase;
    }

    /**
     * MyBatis SqlSessionFactory bean
     * @param dataSource db connection
     * @return MyBatis session factory
     * @throws Exception thrown when failed to create the session factory
     */
    @Bean
    public SqlSessionFactory sqlSessionFactory(DataSource dataSource) throws Exception {
        SqlSessionFactoryBean factoryBean = new SqlSessionFactoryBean();
        factoryBean.setDataSource(dataSource);
        factoryBean.setConfigLocation(new ClassPathResource("mybatis-config.xml"));
        return factoryBean.getObject();
    }

    /**
     * @param dataSource datasource
     * @return Spring transaction manager bean
     */
    @Bean
    public DataSourceTransactionManager transactionManager(DataSource dataSource) {
        return new DataSourceTransactionManager(dataSource);
    }

}
