package eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao;

import java.util.List;

import eu.profinit.manta.bachelor.thesis.log.viewer.model.ErrorMetadata;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.filter.Filter;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.DebugLog;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.ErrorLog;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.Log;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.LogMetadata;

/**
 * DAO for log manipulation ('logs' table)
 * @author jkovac
 */
public interface LogDAO {

    /**
     * Persist error log.
     * @param errorLog error log that should be persisted
     * @param errorId error metadata id
     * @param connDbId connection db id
     * @return amount of inserted rows
     */
    Long save(ErrorLog errorLog, Long errorId, Long connDbId);

    /**
     * Persist debug log
     * @param debugLog debug log that should be persisted
     * @param connDbId connection db id
     * @return amount of inserted rows
     */
    Long save(DebugLog debugLog, Long connDbId);

    /**
     * Calls H2 full text search function and returns appropriate results.
     * @param fullTextSearchString String inputted by a user which should be used for full text searching.
     * @param filter request filter
     * @return Found error logs.
     */
    List<ErrorMetadata> fullText(String fullTextSearchString, Filter filter);

    /**
     * Get log with a given ID
     * @param logId id of the wanted log
     * @param logMetadata metadata of the log used to dynamically choose the correct table to join
     * @return log
     */
    Log find(Long logId, LogMetadata logMetadata);

    /**
     * Load logs that occurred BEFORE the wanted log that have the same metaata and occurred in the same thread
     * @param log root log
     * @param logId id of the root log
     * @param offset amount of logs already loaded
     * @param amount amount to load
     * @param logMetadata root log metadata
     * @return list of logs that fit the criteria
     */
    List<Log> loadEarlier(Log log, Long logId, long offset, long amount, LogMetadata logMetadata);

    /**
     * Load logs that occurred AFTER the wanted log
     * @param log root log
     * @param logId id of the root log
     * @param offset amount of logs already loaded
     * @param amount amount to load
     * @param logMetadata root log metadata
     * @return list of logs that fit the criteria
     */
    List<Log> loadLater(Log log, Long logId, long offset, long amount, LogMetadata logMetadata);

    /**
     * Query logs based on the passed filter
     * @param filter filter settings passed with the request
     * @return erro metadata that fit the criteria
     */
    List<ErrorMetadata> queryLogs(Filter filter);

    /**
     * Get concrete error logs fitting the passed filter settings
     * @param errorCategory category to which the error type belongs
     * @param collapsedElement error type of the logs
     * @param application application which emitted the error
     * @param filter filter settings
     * @return logs fitting the criteria
     */
    List<ErrorLog> expandLogs(String errorCategory, String collapsedElement, String application, Filter filter);

    /**
     * Retrieve log metadata for the root log
     * @param logId id of the root log
     * @return root log metadata
     */
    LogMetadata getLogMetadata(Long logId);

    /**
     * Get all logs, used by tests
     * @return all logs from the repository
     */
    List<? extends Log> getAll();

    /**
     * Prune logs that were emitted from a given application
     * @param application application from which should the logs be deleted
     * @return amount of logs pruned
     */
    Integer pruneLogsFromApplication(String application);

    /**
     * Deletes expired logs that do not belong to any execution.
     * @param pruneAfter time period, after which logs expire
     * @return number of logs deleted
     */
    Integer pruneExpiredNonExecutionLogs(double pruneAfter);

}
