package eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import eu.profinit.manta.bachelor.thesis.log.viewer.model.ErrorMetadata;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.filter.Filter;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.DebugLog;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.ErrorLog;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.Log;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.LogMetadata;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.LogDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.mapper.ErrorExpansionMapper;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.mapper.FullTextMapper;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.mapper.LogMapper;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.mapper.LogContextMapper;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.mapper.LogQueryMapper;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.mapper.PruneMapper;

/**
 * Log DAO implementation
 * @author jkovac
 */
@Repository
public class LogDAOImpl implements LogDAO {

    /**
     * MyBatis mapper
     */
    private LogQueryMapper logQueryMapper;

    /**
     * MyBatis mapper for context queries
     */
    private LogContextMapper logContextMapper;

    /**
     * MyBatis mapper for full text queries
     */
    private FullTextMapper fullTextMapper;

    /**
     * MyBatis mapper for error expansion queries
     */
    private ErrorExpansionMapper errorExpansionMapper;

    /**
     * MyBatis mapper for logs table basic queries
     */
    private LogMapper logMapper;

    /**
     * Pattern to parse full text ids
     */
    private static final Pattern pattern = Pattern.compile("\\[(.*?)\\]");

    /**
     * Instantiates a new Log dao.
     *
     * @param logQueryMapper       the log query mapper
     * @param logContextMapper     the log context mapper
     * @param fullTextMapper       the full text mapper
     * @param errorExpansionMapper the error expansion mapper
     * @param logMapper            the log mapper
     */
    @Autowired
    public LogDAOImpl(LogQueryMapper logQueryMapper,
                      LogContextMapper logContextMapper,
                      FullTextMapper fullTextMapper,
                      ErrorExpansionMapper errorExpansionMapper,
                      LogMapper logMapper) {
        this.logQueryMapper = logQueryMapper;
        this.logContextMapper = logContextMapper;
        this.fullTextMapper = fullTextMapper;
        this.errorExpansionMapper = errorExpansionMapper;
        this.logMapper = logMapper;
    }

    @Override
    public Long save(ErrorLog errorLog, Long errorId, Long connDbId) {
        return logMapper.insertError(errorLog, errorId, connDbId);
    }

    @Override
    public Long save(DebugLog debugLog, Long connDbId) {
        return logMapper.insertDebug(debugLog, connDbId);
    }

    @Override
    public List<ErrorMetadata> fullText(String query, Filter filter) {
        List<String> idx = fullTextMapper.getIndicesFittingTheQuery(query);

        if (idx.isEmpty()){
            return Collections.emptyList();
        }

        List<Long> longIdx = new ArrayList<>();

        idx.forEach(luceneId -> {
            Matcher matcher = pattern.matcher(luceneId);
            while(matcher.find()){
                String id = matcher.group(1);
                longIdx.add(Long.parseLong(id));
            }
        });

        return fullTextMapper.performFullTextOverErrorLogs(query, longIdx, filter);
    }

    @Override
    public Log find(Long logId, LogMetadata logMetadata) {
        return logContextMapper.getLog(logId, logMetadata);
    }

    @Override
    public List<Log> loadEarlier(Log log, Long logId, long offset, long amount, LogMetadata logMetadata) {
        return logContextMapper.loadEarlier(log, logId, offset, amount, logMetadata);
    }

    @Override
    public List<Log> loadLater(Log log, Long logId, long offset, long amount, LogMetadata logMetadata) {
        return logContextMapper.loadLater(log, logId, offset, amount, logMetadata);
    }

    @Override
    public List<ErrorMetadata> queryLogs(Filter filter) {
        return logQueryMapper.queryLogs(filter);
    }

    @Override
    public List<ErrorLog> expandLogs(String errorCategory, String collapsedElement, String application, Filter filter) {
        return errorExpansionMapper.expandLogs(errorCategory, collapsedElement, application, filter);
    }

    @Override
    public LogMetadata getLogMetadata(Long logId) {
        return logContextMapper.getLogMetadata(logId);
    }

    @Override
    public List<? extends Log> getAll() {
        return logMapper.getAll();
    }

    @Override
    public Integer pruneLogsFromApplication(String application) {
        return logMapper.pruneLogsFromApplication(application);
    }

    @Override
    public Integer pruneExpiredNonExecutionLogs(double pruneAfter) {
        return logMapper.pruneExpiredNonExecutionLogs(pruneAfter);
    }

}
