package eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.impl;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;
import java.util.HashMap;
import java.util.Map;
import java.util.Optional;

import org.apache.commons.compress.archivers.ArchiveEntry;
import org.apache.commons.compress.archivers.ArchiveOutputStream;
import org.apache.commons.compress.archivers.zip.ZipArchiveOutputStream;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.Validate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import com.fasterxml.jackson.databind.ObjectMapper;

import eu.profinit.manta.bachelor.thesis.log.viewer.errors.Categories;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.Connection;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.PackageMetadata;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.PackageCreateDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.CreatePackageService;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.exceptions.UnableToCreateDumpException;
import eu.profinit.manta.bachelor.thesis.logging.api.logging.Logger;

/**
 * Package creation service implementation
 * @author jkovac
 */
@Service
public class CreatePackageServiceImpl implements CreatePackageService {

    /**
     * JSON key
     */
    private static final String APPLICATION_KEY = "APPLICATION";

    /**
     * Logger
     */
    private Logger logger = new Logger(getClass());

    /**
     * Package create DAO
     */
    private PackageCreateDAO packageCreateDAO;

    /**
     * Jackson object mapper
     */
    private ObjectMapper objectMapper;

    /**
     * Package path pointing to 'logs' folder of Tomcat
     */
    private String basePackagePath;

    /**
     * Path to the CSV dump pointing to 'logs' folder of Tomcat
     */
    private String baseCsvPath;

    /**
     * Path to the JSON metadata pointing to 'logs' folder of Tomcat
     */
    private String baseJsonPath;

    /**
     * Path to the error csv dump pointing to 'logs' folder of Tomcat
     */
    private String errorsCsvPath;

    /**
     * Path to the application package metadata pointing to 'logs' folder of Tomcat
     */
    private String applicationJsonPath;

    /**
     * Service creating log packages
     * @param objectMapper jackson mapper
     * @param basePackagePath base path to the zip package
     * @param baseCsvPath base path to the temp csv file
     * @param baseScenarioJsonPath base path to the temp json file
     * @param applicationMetadataJsonPath Path to the application package metadata pointing to 'logs' folder of Tomcat
     * @param errorsCsvPath Path to the error csv dump pointing to 'logs' folder of Tomcat
     * @param packageCreateDAO Package create DAO
     */
    @Autowired
    public CreatePackageServiceImpl(PackageCreateDAO packageCreateDAO,
                                    ObjectMapper objectMapper,
                                    @Value("${packaging.base.zip.path}") String basePackagePath,
                                    @Value("${packaging.base.csv.path}") String baseCsvPath,
                                    @Value("${packaging.base.json.scenario.path}") String baseScenarioJsonPath,
                                    @Value("${packaging.errors.csv.path}") String errorsCsvPath,
                                    @Value("${packaging.base.json.application.path}") String applicationMetadataJsonPath) {
        this.packageCreateDAO = packageCreateDAO;
        this.objectMapper = objectMapper;
        this.basePackagePath = basePackagePath;
        this.baseCsvPath = baseCsvPath;
        this.baseJsonPath = baseScenarioJsonPath;
        this.errorsCsvPath = errorsCsvPath;
        this.applicationJsonPath = applicationMetadataJsonPath;
    }

    @Override
    public File createLogsPackageForApplication(String application){
        String zipPath = String.format(basePackagePath, application);
        String csvPath = String.format(baseCsvPath, application);
        String errorsPath = String.format(errorsCsvPath, application);

        packageCreateDAO.createErrorDefinitionsDumpForApplication(application, errorsPath);

        // create a temporary log dump file, handled by H2
        packageCreateDAO.createLogDumpForApplication(application, csvPath);

        File logDump = new File(csvPath);
        File zipArchive = new File(zipPath);
        File errorMetadataFile = new File(errorsPath);
        File applicationMetadataFile = new File(applicationJsonPath);

        Map<String, String> applicationName = new HashMap<>();
        applicationName.put(APPLICATION_KEY, application);

        try (FileWriter fileWriter = new FileWriter(applicationMetadataFile)) {
            String jsonMetadata = objectMapper.writeValueAsString(applicationName);
            fileWriter.write(jsonMetadata);
        } catch (Exception e) {
            throw new UnableToCreateDumpException(e);
        }

        // put files into the archive
        try(ArchiveOutputStream aos = new ZipArchiveOutputStream(zipArchive)){
            archive(aos, logDump);
            archive(aos, errorMetadataFile);
            archive(aos, applicationMetadataFile);
            aos.finish();
        } catch (IOException e) {
            logger.log(Categories.repositoryErrors().unableToCreateDump().entity(application).catching(e));
            throw new UnableToCreateDumpException(e);
        }

        // delete temp file
        Validate.isTrue(logDump.delete());
        Validate.isTrue(errorMetadataFile.delete());
        Validate.isTrue(applicationMetadataFile.delete());

        return zipArchive;
    }

    @Override
    public File createLogsPackageForConnection(String workflowExecutionId, String scenarioExecutionId, String connectionId) {
        Connection connection = new Connection(
                null,
                connectionId,
                scenarioExecutionId,
                null
        );

        Optional<PackageMetadata> packageMetadataOpt = Optional.ofNullable(packageCreateDAO.getPackageMetadata(connection));
        PackageMetadata packageMetadata = packageMetadataOpt.<UnableToCreateDumpException>orElseThrow(() -> {
            throw new UnableToCreateDumpException("Connection with ID " + connectionId + " that is part of " +
                    scenarioExecutionId + " does not exist.");
        });

        String zipPath = String.format(basePackagePath, connectionId);
        String csvPath = String.format(baseCsvPath, connectionId);
        String jsonPath = String.format(baseJsonPath, connectionId);
        String errorsPath = String.format(errorsCsvPath, connectionId);

        packageCreateDAO.createErrorDefinitionsDumpForConnection(packageMetadata.getConnDbId(), errorsPath);

        // create a temporary log dump file, handled by H2
        packageCreateDAO.createLogDumpForConnection(packageMetadata.getConnDbId(), csvPath);

        File packageMetadataFile = new File(jsonPath);

        // map scenario execution metadata to json
        try {
            objectMapper.writeValue(packageMetadataFile, packageMetadata);
        } catch (IOException e) {
            logger.log(Categories.repositoryErrors().unableToCreateDump().entity(connectionId).catching(e));
            throw new UnableToCreateDumpException(e);
        }

        File logDump = new File(csvPath);
        File zipArchive = new File(zipPath);
        File errorMetadataFile = new File(errorsPath);

        // put files into the archive
        try(ArchiveOutputStream aos = new ZipArchiveOutputStream(zipArchive)){
            archive(aos, packageMetadataFile);
            archive(aos, logDump);
            archive(aos, errorMetadataFile);
            aos.finish();
        } catch (IOException e) {
            logger.log(Categories.repositoryErrors().unableToCreateDump().entity(scenarioExecutionId).catching(e));
            throw new UnableToCreateDumpException(e);
        }

        // delete temp files
        Validate.isTrue(packageMetadataFile.delete());
        Validate.isTrue(logDump.delete());
        Validate.isTrue(errorMetadataFile.delete());

        return zipArchive;
    }

    /**
     * Archives a single file.
     * @param aos stream to the zip file
     * @param file file that should be archived
     * @throws IOException thrown in case of missing files
     */
    private void archive(ArchiveOutputStream aos, File file) throws IOException {
        ArchiveEntry archiveEntry = aos.createArchiveEntry(file, file.getName());
        aos.putArchiveEntry(archiveEntry);
        if (file.isFile()) {
            try (InputStream i = Files.newInputStream(file.toPath())) {
                IOUtils.copy(i, aos);
            }
        }
        aos.closeArchiveEntry();
    }

}
