package eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.impl;

import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.apache.commons.lang3.Validate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.ErrorLog;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.Log;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.LogMetadata;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.LogDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.LogContextService;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.exceptions.InvalidContextException;

/**
 * Service providing context to logs
 * @author jkovac
 */
@Service
public class LogContextServiceImpl implements LogContextService {

    /**
     * Log DAO
     */
    private LogDAO logDAO;

    /**
     * Initial amount of loaded logs
     */
    private static final int INITIAL_OFFSET = 0;

    /**
     * Initial amount of logs to load
     */
    private static final int INITIAL_AMOUNT = 5;

    /**
     * Instantiates a new Log context service.
     *
     * @param logDAO the log dao
     */
    @Autowired
    public LogContextServiceImpl(LogDAO logDAO) {
        this.logDAO = logDAO;
    }

    @Override
    public List<? extends Log> loadLater(Long logId, long offset, long amount) {
        Optional<LogMetadata> rootLogMetadataOpt = Optional.ofNullable(logDAO.getLogMetadata(logId));
        LogMetadata rootLogMetadata = rootLogMetadataOpt.orElseThrow(() ->
                new InvalidContextException("Metadata context is invalid. The log with ID=" + logId + " does not exist.")
        );

        Log log = logDAO.find(logId, rootLogMetadata);
        return logDAO.loadLater(log, logId, offset, amount, rootLogMetadata);
    }

    @Override
    public List<? extends Log> loadEarlier(Long logId, long offset, long amount) {
        Optional<LogMetadata> rootLogMetadataOpt = Optional.ofNullable(logDAO.getLogMetadata(logId));
        LogMetadata rootLogMetadata = rootLogMetadataOpt.orElseThrow(() ->
                new InvalidContextException("Metadata context is invalid. The log with ID=" + logId + " does not exist.")
        );

        Log log = logDAO.find(logId, rootLogMetadata);
        List<? extends Log> logs = logDAO.loadEarlier(log, logId, offset, amount, rootLogMetadata);

        Collections.reverse(logs);
        return logs;
    }

    @Override
    public List<? extends Log> getContextForLog(Long logId) {
        LogMetadata rootLogMetadata = logDAO.getLogMetadata(logId);

        if (!rootLogMetadata.getType().equals("ERROR")){
            throw new InvalidContextException("Context can only be given to error logs.");
        }

        ErrorLog rootLog = (ErrorLog) logDAO.find(logId, rootLogMetadata);

        List<? extends Log> before = logDAO.loadEarlier(rootLog, logId, INITIAL_OFFSET, INITIAL_AMOUNT, rootLogMetadata);
        List<? extends Log> after = logDAO.loadLater(rootLog, logId, INITIAL_OFFSET, INITIAL_AMOUNT, rootLogMetadata);

        Collections.reverse(before);

        return Stream
                .of(before, Collections.singletonList(rootLog), after)
                .flatMap(Collection::stream)
                .collect(Collectors.toList())
                ;
    }

}
