package eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.impl;

import java.util.Optional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import eu.profinit.manta.bachelor.thesis.log.viewer.model.Connection;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.ErrorDefinition;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.ErrorMetadata;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.DebugLog;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.ErrorLog;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.Log;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.ConnectionsDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.ErrorDefinitionDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.ErrorMetadataDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.LogDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.ScenarioExecutionDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.WorkflowExecutionDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.ScenarioExecution;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.WorkflowExecution;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.LogPersistenceService;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.PruneService;

/**
 * Log service implementation
 * @author jkovac
 */
@Service
public class LogPersistenceServiceImpl implements LogPersistenceService {

    /**
     * Log table DAO
     */
    private LogDAO logDAO;

    /**
     * Workflow executions DAO
     */
    private WorkflowExecutionDAO workflowExecutionDAO;

    /**
     * Scenario executions DAO
     */
    private ScenarioExecutionDAO scenarioExecutionDAO;

    /**
     * Error metadata DAO
     */
    private ErrorMetadataDAO errorMetadataDAO;

    /**
     * Error Definition DAO
     */
    private ErrorDefinitionDAO errorDefinitionDAO;

    /**
     * Connections DAO
     */
    private ConnectionsDAO connectionsDAO;

    /**
     * Prune scheduling service
     */
    private PruneService pruneService;

    /**
     * Service handling business logic for logs
     * @param logDAO log DAO
     * @param workflowExecutionDAO workflow DAO
     * @param scenarioExecutionDAO scenario executions DAO
     * @param pruneService service for scheduling log pruning
     * @param connectionsDAO connections DAO
     * @param errorDefinitionDAO error definitions DAO
     * @param errorMetadataDAO error metadata DAO
     */
    @Autowired
    public LogPersistenceServiceImpl(LogDAO logDAO,
                                     WorkflowExecutionDAO workflowExecutionDAO,
                                     ScenarioExecutionDAO scenarioExecutionDAO,
                                     PruneService pruneService,
                                     ErrorMetadataDAO errorMetadataDAO,
                                     ErrorDefinitionDAO errorDefinitionDAO,
                                     ConnectionsDAO connectionsDAO) {
        this.logDAO = logDAO;
        this.workflowExecutionDAO = workflowExecutionDAO;
        this.scenarioExecutionDAO = scenarioExecutionDAO;
        this.pruneService = pruneService;
        this.errorMetadataDAO = errorMetadataDAO;
        this.errorDefinitionDAO = errorDefinitionDAO;
        this.connectionsDAO = connectionsDAO;
    }

    @Transactional
    public void persistErrorLog(ErrorLog errorLog){
        Long connectionDbId = checkExecutionMetadata(errorLog);
        Long errorId = checkErrorMetadata(errorLog, connectionDbId);
        logDAO.save(errorLog, errorId, connectionDbId);
    }

    @Transactional
    public void persistDebugLog(DebugLog debugLog){
        Long connectionDbId = checkExecutionMetadata(debugLog);
        logDAO.save(debugLog, connectionDbId);
    }

    /**
     * Check if metadata for such error is already stored (i.e. this same error already occurred in this scenario execution for this connection)
     * @param errorLog error log to be checked
     * @param connDbId db id of the connection
     * @return error metadata ID
     */
    private Long checkErrorMetadata(ErrorLog errorLog, Long connDbId){
        Long errorDefinitionId = checkErrorDefinition(errorLog);

        ErrorMetadata errorMetadata = new ErrorMetadata(
                errorDefinitionId,
                errorLog.getWorkflowExecutionId(),
                errorLog.getScenarioExecutionId(),
                connDbId,
                errorLog.getPackageName(),
                errorLog.getApplication()
        );

        Optional<Long> errorId = Optional.ofNullable(errorMetadataDAO.getIdByData(errorMetadata));

        // MyBatis stores ID of the inserted object automatically
        return errorId.orElseGet(() -> {
            errorMetadataDAO.save(errorMetadata);
            return errorMetadata.getErrorId();
        });
    }

    /**
     * Check if definition for such error is already stored (i.e. this error has already occurred elsewhere)
     * @param errorLog error log to be checked
     * @return error definition ID
     */
    private Long checkErrorDefinition(ErrorLog errorLog){
        ErrorDefinition errorDefinition = new ErrorDefinition(
                errorLog.getCategory(),
                errorLog.getErrorType(),
                errorLog.getUserMessage(),
                errorLog.getTechnicalMessage(),
                errorLog.getSolution(),
                errorLog.getLineageImpact(),
                errorLog.getLevel()
        );

        Optional<Long> errorDefinitionId = Optional.ofNullable(errorDefinitionDAO.getIdByData(errorDefinition));

        return errorDefinitionId.orElseGet(() -> {
            errorDefinitionDAO.save(errorDefinition);
            return errorDefinition.getErrorDefinitionId();
        });
    }

    /**
     * Check if scenario and workflow executions, to which the log belongs, exist. If not, insert them.
     * @param log Log that should be persisted
     */
    private synchronized Long checkExecutionMetadata(Log log){
        Optional<String> connectionIdOptional = Optional.ofNullable(log.getConnectionId());
        if (connectionIdOptional.isPresent()){
            return checkConnection(log);
        }

        // has scenario execution but is not part of connection
        Optional<String> scenarioExecutionIdOptional = Optional.ofNullable(log.getScenarioExecutionId());
        if (scenarioExecutionIdOptional.isPresent()){
            checkScenarioExecution(log);
            return null;
        }

        // has workflow execution but is not scenario execution
        Optional<String> workflowExecutionIdOptional = Optional.ofNullable(log.getWorkflowExecutionId());
        if (workflowExecutionIdOptional.isPresent()){
            checkWorkflowExecution(log);
        }
        return null;
    }

    /**
     * Check if given connection is already stored
     * @param log log to be checked
     * @return connection db id
     */
    private Long checkConnection(Log log){
        Connection connection = new Connection(
                null,
                log.getConnectionId(),
                log.getScenarioExecutionId(),
                null
        );

        Optional<Long> connectionIdOpt = Optional.ofNullable(connectionsDAO.getIdByData(connection));

        return connectionIdOpt.orElseGet(() -> {
            checkScenarioExecution(log);
            connectionsDAO.save(connection);
            return connection.getConnectionDbId();
        });
    }

    /**
     * Check if given scenario execution is already stored
     * @param log log to be checked
     */
    private void checkScenarioExecution(Log log){
        Optional<ScenarioExecution> scenarioExecutionOpt = scenarioExecutionDAO.find(log.getScenarioExecutionId());

        if (!scenarioExecutionOpt.isPresent()){
            checkWorkflowExecution(log);

            ScenarioExecution scenarioExecution = new ScenarioExecution(
                    log.getScenarioExecutionId(),
                    log.getWorkflowExecutionId(),
                    log.getTechnology(),
                    log.getPhase(),
                    log.getScenario()
            );
            scenarioExecutionDAO.save(scenarioExecution);
        }
    }

    /**
     * Check if given workflow execution is already stored
     * @param log log to be checked
     */
    private void checkWorkflowExecution(Log log){
        Optional<WorkflowExecution> workflowExecutionOpt = workflowExecutionDAO.find(log.getWorkflowExecutionId());

        if (!workflowExecutionOpt.isPresent()){
            handleWorkflowExecutionDoesNotExist(log);
        }
    }

    /**
     * Inserts a new workflow execution to database.
     * @param log log that is being persisted.
     */
    private void handleWorkflowExecutionDoesNotExist(Log log){
        WorkflowExecution workflowExecution = new WorkflowExecution(log.getWorkflowExecutionId());
        workflowExecutionDAO.save(workflowExecution);

        // schedule pruning for a newly created workflow execution
        pruneService.schedulePruning(workflowExecution);
    }

}
