package eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.impl;

import java.time.Instant;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;

import javax.annotation.PostConstruct;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.DependsOn;
import org.springframework.scheduling.Trigger;
import org.springframework.scheduling.TriggerContext;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.scheduling.concurrent.ThreadPoolTaskScheduler;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import eu.profinit.manta.bachelor.thesis.log.viewer.model.WorkflowExecution;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.ErrorDefinitionDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.ErrorMetadataDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.LogDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.WorkflowExecutionDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.PruneService;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.PruningCommon;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.exceptions.UnableToPruneWorkflowExecutionException;
import eu.profinit.manta.bachelor.thesis.logging.api.logging.Logger;

/**
 * Prune scheduling service implementation
 * @author jkovac
 */
@Service
@DependsOn("liquibase")
public class PruneServiceImpl extends PruningCommon implements PruneService {

    /**
     * Logger
     */
    private final Logger logger = new Logger(getClass());

    /**
     * One day in milliseconds.
     * 24 * 60 * 60 * 1000
     */
    private static final long oneDayInMillis = 86400000L;

    /**
     * DAO for workflow manipulation
     */
    private WorkflowExecutionDAO workflowExecutionDAO;

    /**
     * How long until workflow expires
     */
    private double pruneAfter;

    /**
     * Log DAO
     */
    private LogDAO logDAO;

    /**
     * Spring thread pool task scheudler
     */
    private ThreadPoolTaskScheduler threadPoolTaskScheduler;

    /**
     * Scheduled prune tasks
     */
    private Map<PruneTask, ScheduledFuture<?>> pruneTasks = new HashMap<>();


    /**
     * Instantiates a new Prune service.
     *
     * @param workflowExecutionDAO    the workflow execution dao
     * @param pruneAfter              the prune after
     * @param threadPoolTaskScheduler the thread pool task scheduler
     * @param errorDefinitionDAO      the error definition dao
     * @param errorMetadataDAO        the error metadata dao
     * @param logDAO                  the log dao
     */
    @Autowired
    public PruneServiceImpl(WorkflowExecutionDAO workflowExecutionDAO,
                            @Value("${logviewer.pruneAfter}") double pruneAfter,
                            ThreadPoolTaskScheduler threadPoolTaskScheduler,
                            ErrorDefinitionDAO errorDefinitionDAO,
                            ErrorMetadataDAO errorMetadataDAO,
                            LogDAO logDAO) {
        super(errorMetadataDAO, errorDefinitionDAO);
        this.workflowExecutionDAO = workflowExecutionDAO;
        this.pruneAfter = pruneAfter;
        this.threadPoolTaskScheduler = threadPoolTaskScheduler;
        this.logDAO = logDAO;
    }

    @Override
    public void dropPruning(){
        pruneTasks.forEach((pruneTask, scheduledFuture) -> {
            scheduledFuture.cancel(false); // do NOT interrupt the thread if it is currently pruning

            logger.info("Cancelling pruning of logs from workflow execution {}.", pruneTask.getWorkflowExecution().getWorkflowExecutionId());
        });

        pruneTasks.clear();
    }

    @Override
    public void dropPruningFor(WorkflowExecution workflowExecution){
        pruneTasks.entrySet().removeIf(entry -> {
            PruneTask pruneTask = entry.getKey();
            ScheduledFuture<?> scheduledFuture = entry.getValue();

            WorkflowExecution w = pruneTask.getWorkflowExecution();
            if (scheduledFuture.getDelay(TimeUnit.MILLISECONDS) <= 0){
                throw new UnableToPruneWorkflowExecutionException(
                        "Unable to prune workflow execution that is currently being pruned or has been pruned.");
            }

            scheduledFuture.cancel(false);
            return w.getWorkflowExecutionId().equals(workflowExecution.getWorkflowExecutionId());
        });
    }

    @Override
    public void schedulePruning(WorkflowExecution workflowExecution) {
        doSchedulePrune(workflowExecution);
    }

    @Transactional
    @PostConstruct
    @Override
    public void schedulePruning(){
        List<WorkflowExecution> workflowExecutions = workflowExecutionDAO.getAll();
        workflowExecutions.forEach(this::doSchedulePrune);
    }

    /**
     * Schedules pruning for single workflow execution
     * @param workflowExecution workflow execution that will be pruned
     */
    private void doSchedulePrune(WorkflowExecution workflowExecution){
        Date executionDate = workflowExecution.getTimestamp();
        long executionDateEpochMillis = executionDate.getTime();
        long pruneTimeMillis = executionDateEpochMillis + daysToMillis(pruneAfter);
        long delay = pruneTimeMillis - Instant.now().toEpochMilli();

        logger.info("Scheduling pruning for workflow execution {} at {}. Pruning after {}.",
                workflowExecution.getWorkflowExecutionId(),
                new Date(pruneTimeMillis).toString(), delay);

        Date date = new Date(pruneTimeMillis);

        PruneTask pruneTask = new PruneTask(workflowExecution, date);
        ScheduledFuture<?> scheduledFuture = threadPoolTaskScheduler.schedule(pruneTask, date);
        pruneTasks.put(pruneTask, scheduledFuture);
    }

    /**
     * Converts days to milliseconds. Days can be a decimal value.
     * @param days days that will be converted
     * @return given amount of days in millis
     */
    private long daysToMillis(double days){
        return (long) (days * oneDayInMillis);
    }

    @Override
    public void updateScheduledPruning(double updatedValue){
        this.pruneAfter = updatedValue;
        dropPruning();
        schedulePruning();
    }

    @Override
    public Map<PruneTask, ScheduledFuture<?>> getScheduledTasks(){
        return pruneTasks;
    }

    @Transactional
    protected void prune(WorkflowExecution workflowExecution) {
        workflowExecutionDAO.delete(workflowExecution);
        updateErrors();
    }

    @Scheduled(cron = "0 0 * ? * *")
    @Transactional
    public void pruneNonExecutionLogs(){
        int deletedLogs = logDAO.pruneExpiredNonExecutionLogs(pruneAfter);
        logger.info("Pruned {} logs. Next pruning is scheduled in one hour.", deletedLogs);
        updateErrors();
    }

    @Override
    public PruneTask newPruningTask(WorkflowExecution workflowExecution, Date date) {
        return new PruneTask(workflowExecution, date);
    }

    /**
     * Runnable scheduled task that runs in future and executes pruning of logs.
     * @author jkovac
     */
    public class PruneTask implements Runnable {
        /**
         * Workflow execution that should be pruned
         */
        private WorkflowExecution workflowExecution;

        /**
         * Logger
         */
        private Logger logger = new Logger(getClass());

        /**
         * Time when prune task is triggered
         */
        private Date triggerDate;

        /**
         * Pruning task
         * @param workflowExecution workflow execution that will be pruned
         * @param triggerDate trigger date
         */
        public PruneTask(WorkflowExecution workflowExecution, Date triggerDate) {
            this.workflowExecution = workflowExecution;
            this.triggerDate = triggerDate;
        }

        @Override
        public void run() {
            prune(workflowExecution);
            pruneTasks.remove(this);
            logger.info("Pruned logs from workflow {}.", workflowExecution.getWorkflowExecutionId());
        }

        /**
         * @return workflow execution that will be pruned
         */
        public WorkflowExecution getWorkflowExecution() {
            return workflowExecution;
        }

        /**
         * @return trigger date
         */
        public Date getTriggerDate() {
            return new Date(triggerDate.getTime());
        }
    }
}

