package eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.impl;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.List;
import java.util.Properties;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import eu.profinit.manta.bachelor.thesis.log.viewer.controllers.dto.SetExpiryCheckResponseDTO;
import eu.profinit.manta.bachelor.thesis.log.viewer.errors.Categories;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.WorkflowExecution;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.PruneCheckingDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.PruneService;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.UpdateSettingsService;
import eu.profinit.manta.bachelor.thesis.logging.api.logging.Logger;

/**
 * Update settings service implementation
 * @author jkovac
 */
@Service
public class UpdateSettingsServiceImpl implements UpdateSettingsService {

    /**
     * Logger
     */
    private Logger logger = new Logger(getClass());

    /**
     * Property holding after how long will logs expire
     */
    private static final String PRUNE_PROPERTY = "logviewer.pruneAfter";

    /**
     * Prune scheduling service
     */
    private PruneService pruneService;

    /**
     * One day in seconds
     */
    private static final double ONE_DAY_SECONDS = 86400;

    /**
     * Prune checking DAO
     */
    private PruneCheckingDAO pruneCheckingDAO;

    /**
     * Property file holding the current setting
     */
    private File propertiesFile;

    /**
     * Instantiates a new Update settings service.
     *
     * @param pruneService     the prune service
     * @param pruneCheckingDAO the prune checking dao
     * @param propertiesFile the property file path
     */
    @Autowired
    public UpdateSettingsServiceImpl(PruneService pruneService,
                                     PruneCheckingDAO pruneCheckingDAO,
                                     @Value("${property.path}") File propertiesFile) {
        this.pruneService = pruneService;
        this.pruneCheckingDAO = pruneCheckingDAO;
        this.propertiesFile = propertiesFile;
    }

    @Override
    public SetExpiryCheckResponseDTO checkIfSomeLogsWillBePruned(double updatedPruningValue){
        double updatedPruningValueSeconds = updatedPruningValue * ONE_DAY_SECONDS;
        Integer deletedLogs = pruneCheckingDAO.getAmountOfPotentiallyDeletedLogs(updatedPruningValueSeconds);
        List<WorkflowExecution> deletedWorkflowExecutions = pruneCheckingDAO.getPotentiallyDeletedWorkflows(updatedPruningValueSeconds);
        Integer deletedWorkflowLogs =
                deletedWorkflowExecutions.isEmpty()
                        ?
                        new Integer(0)
                        :
                        pruneCheckingDAO.getAmountOfPotentiallyDeletedWorkflowLogs(updatedPruningValueSeconds, deletedWorkflowExecutions);

        SetExpiryCheckResponseDTO setExpiryCheckResponseDTO
                = new SetExpiryCheckResponseDTO(deletedLogs, deletedWorkflowExecutions, deletedWorkflowLogs);

        return setExpiryCheckResponseDTO;
    }

    @Override
    @Transactional
    public void updatePruningDelayProperty(double updatedPruningValue) {
        Properties props = new Properties();

        try {
            FileInputStream fileInputStream = new FileInputStream(propertiesFile);
            props.load(fileInputStream);
            fileInputStream.close();

            String previousPruningValue = (String) props.get(PRUNE_PROPERTY);

            FileOutputStream fileOutputStream = new FileOutputStream(propertiesFile);
            props.setProperty(PRUNE_PROPERTY, String.valueOf(updatedPruningValue));
            props.store(fileOutputStream, null);
            fileOutputStream.close();

            pruneService.updateScheduledPruning(updatedPruningValue);

            logger.info("Successfully updated pruning delay from {} to {}.", previousPruningValue, updatedPruningValue);
        } catch (FileNotFoundException e) {
            logger.log(Categories.repositoryErrors().pruningConfigurationNotFound().catching(e));
        } catch (IOException e) {
            logger.log(Categories.repositoryErrors().pruningConfigurationNotFound().catching(e));
        }
    }

}
