package eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.impl;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;

import org.apache.commons.compress.archivers.zip.ZipArchiveEntry;
import org.apache.commons.compress.archivers.zip.ZipArchiveInputStream;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.type.MapType;
import com.fasterxml.jackson.databind.type.TypeFactory;

import eu.profinit.manta.bachelor.thesis.log.viewer.errors.Categories;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.ErrorDefinition;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.ErrorMetadata;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.PackageMetadata;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.UploadedPackage;
import eu.profinit.manta.bachelor.thesis.log.viewer.model.logs.UploadedLog;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.ErrorDefinitionDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.ErrorMetadataDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.dao.PackageUploadDAO;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.UploadPackageService;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.exceptions.InvalidPackageFileException;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.exceptions.PackageAlreadyExistsException;
import eu.profinit.manta.bachelor.thesis.log.viewer.repository.services.exceptions.UnableToDeleteTemporaryFiles;
import eu.profinit.manta.bachelor.thesis.logging.api.logging.Logger;

/**
 * Upload package service implementations
 * @author jkovac
 */
@Service
public class UploadPackageServiceImpl implements UploadPackageService {

    /**
     * Logger
     */
    private Logger logger = new Logger(getClass());

    /**
     * Package upload DAO
     */
    private PackageUploadDAO packageUploadDAO;

    /**
     * Error Definition DAO
     */
    private ErrorDefinitionDAO errorDefinitionDAO;

    /**
     * Error Metadata DAO
     */
    private ErrorMetadataDAO errorMetadataDAO;

    /**
     * Jackson object mapper
     */
    private ObjectMapper objectMapper;

    /**
     * Base upload directory, pointing to 'logs/temp' folder in Tomcat
     */
    private String uploadBaseDirectory;

    /**
     * Common suffix for all metadata files
     */
    private static final String METADATA_FILE_SUFFIX = "_metadata.json";

    /**
     * Common suffix for all log dump files
     */
    private static final String DUMP_FILE_SUFFIX = "_logs.csv";

    /**
     * Common suffix for all error dump files
     */
    private static final String ERROR_FILE_SUFFIX = "_errors.csv";

    /**
     * Common name for all application metadata files
     */
    private static final String APPLICATION_METADATA_FILE = "application.json";

    /**
     * JSON key in application metadata
     */
    private static final String APPLICATION_KEY = "APPLICATION";

    /**
     * Error type
     */
    private static final String ERROR_TYPE = "ERROR";

    /**
     * Instantiates a new Upload package service.
     *
     * @param packageUploadDAO    the package upload dao
     * @param objectMapper        the object mapper
     * @param uploadBaseDirectory the upload base directory
     * @param errorDefinitionDAO  the error definition dao
     * @param errorMetadataDAO    the error metadata dao
     */
    @Autowired
    public UploadPackageServiceImpl(PackageUploadDAO packageUploadDAO,
                                    ObjectMapper objectMapper,
                                    @Value("${packaging.upload.directory}") String uploadBaseDirectory,
                                    ErrorDefinitionDAO errorDefinitionDAO,
                                    ErrorMetadataDAO errorMetadataDAO) {
        this.packageUploadDAO = packageUploadDAO;
        this.uploadBaseDirectory = uploadBaseDirectory;
        this.objectMapper = objectMapper;
        this.errorDefinitionDAO = errorDefinitionDAO;
        this.errorMetadataDAO = errorMetadataDAO;
    }

    @Override
    public void uploadPackage(MultipartFile file, String packageName) {
        if (packageName == null){
            throw new IllegalArgumentException("Package name must not be null.");
        }

        if (file == null){
            throw new InvalidPackageFileException("Uploaded package is null.");
        }

        File tempDirectory = new File(uploadBaseDirectory);

        try {
            unzip(tempDirectory, file);
        } catch (IOException e) {
            throw new InvalidPackageFileException("Unable to get input stream for the uploaded package.", e);
        }

        Optional<File> metadataFileOptional = getFile(tempDirectory, METADATA_FILE_SUFFIX);
        Optional<File> applicationMetadataFileOptional = getFile(tempDirectory, APPLICATION_METADATA_FILE);

        if (metadataFileOptional.isPresent()){
            uploadConnectionLogs(tempDirectory, metadataFileOptional.get(), packageName);
        } else if (applicationMetadataFileOptional.isPresent()){
            uploadApplicationLogs(tempDirectory, applicationMetadataFileOptional.get(), packageName);
        } else {
            throw new InvalidPackageFileException("Package file is corrupted.");
        }

        try {
            FileUtils.deleteDirectory(tempDirectory);
        } catch (IOException e) {
            throw new UnableToDeleteTemporaryFiles("Upload was successful, but temporary files were not deleted due to error.", e);
        }
    }

    /**
     * Upload metadata for connection and then upload the logs
     * @param tempDirectory paht to the temp directory
     * @param metadata metadata file
     * @param packageName new package name
     */
    private void uploadConnectionLogs(File tempDirectory, File metadata, String packageName){
        PackageMetadata packageMetadata;
        try {
            packageMetadata = objectMapper.readValue(metadata, PackageMetadata.class);
        } catch (IOException e) {
            logger.log(Categories.repositoryErrors().unableToCreateDump().catching(e));
            throw new InvalidPackageFileException("Metadata file is corrupted.", e);
        }

        UploadedPackage uploadedPackage = new UploadedPackage(
                packageName,
                packageMetadata.getTechnology(),
                packageMetadata.getPhase(),
                packageMetadata.getScenarioName(),
                packageMetadata.getConnectionId()
        );

        doUpload(tempDirectory, packageName, uploadedPackage);
    }

    /**
     * Upload metadata for application logs
     * @param tempDirectory path to the temp directory
     * @param applicationMetadataFile application metadata file
     * @param packageName new package name
     */
    private void uploadApplicationLogs(File tempDirectory, File applicationMetadataFile, String packageName){
        String applicationName = "";

        try {
            String metadata = FileUtils.readFileToString(applicationMetadataFile);
            TypeFactory typeFactory = objectMapper.getTypeFactory();
            MapType mapType = typeFactory.constructMapType(HashMap.class, String.class, String.class);
            Map<String, String> map = objectMapper.readValue(metadata, mapType);
            applicationName = map.get(APPLICATION_KEY) == null ? "" : map.get(APPLICATION_KEY);
        } catch (IOException e) {
            e.printStackTrace();
        }

        doUpload(tempDirectory, packageName, new UploadedPackage(packageName, applicationName));
    }

    /**
     * Upload logs
     * @param tempDirectory path to the temp directory
     * @param packageName package name
     * @param uploadedPackage package metadata
     */
    private void doUpload(File tempDirectory, String packageName, UploadedPackage uploadedPackage){
        try {
            packageUploadDAO.save(uploadedPackage);
        } catch (DuplicateKeyException e){
            throw new PackageAlreadyExistsException("Package with name " + packageName + " already exists.");
        }

        Optional<File> dumpOpt = getFile(tempDirectory, DUMP_FILE_SUFFIX);
        String dumpPath;

        if (dumpOpt.isPresent()){
            dumpPath = dumpOpt.get().getAbsolutePath();
        } else {
            throw new InvalidPackageFileException("Database dump with logs was not found in the package.");
        }

        Optional<File> errorFileOpt = getFile(tempDirectory, ERROR_FILE_SUFFIX);
        File errorFile =
                errorFileOpt.orElseThrow(() -> new InvalidPackageFileException("Error metadata file was not found in the package."));

        List<ErrorDefinition> errorDefinitionsList =  packageUploadDAO.readErrorDefinitions(errorFile.getAbsolutePath());
        Map<Long, Long> errorMappings = mapErrorIdsToLocalIds(errorDefinitionsList);

        errorDefinitionsList.forEach(error -> {
            if (!errorMappings.containsKey(error.getErrorDefinitionId())){
                Long uploadedId = error.getErrorDefinitionId();
                errorDefinitionDAO.save(error);
                errorMappings.put(uploadedId, error.getErrorDefinitionId());
            }
        });

        List<UploadedLog> logs = packageUploadDAO.readLogs(dumpPath);

        Map<Long, Long> uploadedDefinitionToLocalErrorMetadataMapping = new HashMap<>();

        for (UploadedLog log : logs){
            // If it is a log error
            if (log.getType().equals(ERROR_TYPE)){
                Long uploadedLogErrDefinitionId = log.getErrorDefinitionId();
                if (!uploadedDefinitionToLocalErrorMetadataMapping.containsKey(uploadedLogErrDefinitionId)){
                    checkErrorMetadata(log, errorMappings, packageName, uploadedDefinitionToLocalErrorMetadataMapping);
                }
                Long errorMetadataId = uploadedDefinitionToLocalErrorMetadataMapping.get(uploadedLogErrDefinitionId);
                packageUploadDAO.upload(log, errorMetadataId, packageName);
            } else {
                // if debug log, just store it
                packageUploadDAO.upload(log, packageName);
            }
        }
    }

    /**
     * Check if metadata for this error is already stored
     * @param uploadedLog log to be checked
     * @param errorMappings mappings of (packaged error definition id) -> (locally stored error definition)
     * @param packageName new package name
     * @param definitionToMetadataMapping mapping of (package error definition id) -> (locally stored error metadata)
     */
    private void checkErrorMetadata(UploadedLog uploadedLog, Map<Long, Long> errorMappings, String packageName, Map<Long, Long> definitionToMetadataMapping){
        Long errorDefinitionId = uploadedLog.getErrorDefinitionId();
        ErrorMetadata errorMetadata = new ErrorMetadata(
                errorMappings.get(errorDefinitionId),
                packageName,
                uploadedLog.getApplication()
        );

        Optional<Long> errorId = Optional.ofNullable(errorMetadataDAO.getIdByData(errorMetadata));

        // MyBatis stores ID of the inserted object automatically
        Long errorMetadataId = errorId.orElseGet(() -> {
            errorMetadataDAO.save(errorMetadata);
            return errorMetadata.getErrorId();
        });

        definitionToMetadataMapping.put(errorDefinitionId, errorMetadataId);
    }

    /**
     * Map error definition ids from the uploaded dump to locally stored definitions
     * @param uploadedErrors uploaded errors
     * @return mappings
     */
    private Map<Long, Long> mapErrorIdsToLocalIds(List<ErrorDefinition> uploadedErrors){
        Map<Long, Long> mapping = new HashMap<>();

        uploadedErrors.forEach(error -> {
            Optional<Long> locallyStoredError = Optional.ofNullable(errorDefinitionDAO.getIdByData(error));
            locallyStoredError.ifPresent(id -> mapping.put(error.getErrorDefinitionId(), id));
        });

        return mapping;
    }

    /**
     * Unzip package
     * @param tempDirectory temp directory in 'logs' directory of Tomcat
     * @param file uploaded file
     * @throws IOException io exception
     */
    private void unzip(File tempDirectory, MultipartFile file) throws IOException {
        ZipArchiveInputStream zipArchiveInputStream = new ZipArchiveInputStream(file.getInputStream());
        FileUtils.forceMkdir(tempDirectory);

        ZipArchiveEntry entry = null;
        FileOutputStream fos = null;
        while((entry = zipArchiveInputStream.getNextZipEntry()) != null){
            File outputFile = new File(tempDirectory, entry.getName());
            fos = new FileOutputStream(outputFile);
            IOUtils.copy(zipArchiveInputStream, fos);
            IOUtils.closeQuietly(fos);
        }

        IOUtils.closeQuietly(zipArchiveInputStream);
    }

    /**
     * Get file by suffix
     * @param tempDirectory temp directory
     * @param suffix suffix of the wanted file
     * @return optional of wanted file if present, otherwise empty optional
     */
    private Optional<File> getFile(File tempDirectory, String suffix){
        File[] files = tempDirectory.listFiles();
        if (files == null){
            throw new InvalidPackageFileException("Package is empty.");
        }

        for (File file : files){
            if (file.getName().endsWith(suffix)){
                return Optional.of(file);
            }
        }
        return Optional.empty();
    }

}
