package eu.profinit.manta.bachelor.thesis.logging.api.categorization;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * Annotation that transforms a method to an error declaration. The method's name should be in camel-case and should
 * return a builder, whose name is always in the format of method name with capitalized first letter and the
 * suffix "Builder". E.g. "testError" should return "TestErrorBuilder".
 *
 * @author jkovac
 */
@Retention(RetentionPolicy.RUNTIME)
@Target(ElementType.METHOD)
@Documented
public @interface Error {

    /**
     * Parameter that overrides name of the error that is generated by default from the error name. Optional parameter.
     *
     * @return specified name of the error.
     */
    String errorName() default "";

    /**
     * Attribute of an error providing a user friendly message without technical details.
     *
     * @return user message of the error.
     */
    String userMessage();

    /**
     * Attribute of an error providing a message that has necessary technical details that are required for fixing
     * the error, e.g. by developers.
     *
     * @return technical message of the error.
     */
    String technicalMessage();

    /**
     * Attribute of an error providing information that may help in resolving the error.
     *
     * @return solution of the error.
     */
    String solution();

    /**
     * Attribute of an error providing information that the error may have had on the resulting data lineage or an
     * application run (e.g. scenario failed). Default value is UNDEFINED, parameter is optional.
     *
     * @return impact of the error on data lineage.
     */
    Impact lineageImpact() default Impact.UNDEFINED;

    /**
     * Attribute of an error describing its severity.
     * Allowed values are WARN, ERROR an FATAL.<br><br>
     * WARN - Error that may lead to more severe errors.<br>
     * ERROR - General error that does not lead to application failure.<br>
     * FATAL - Error leading to application failure.
     *
     * @return severity of the error.
     */
    Severity severity();
}
