package eu.profinit.manta.bachelor.thesis.logging.api.logging;

import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.annotation.processing.AbstractProcessor;
import javax.annotation.processing.Filer;
import javax.annotation.processing.ProcessingEnvironment;
import javax.annotation.processing.RoundEnvironment;
import javax.annotation.processing.SupportedAnnotationTypes;
import javax.lang.model.element.Element;
import javax.lang.model.element.ExecutableElement;
import javax.lang.model.element.Modifier;
import javax.lang.model.element.PackageElement;
import javax.lang.model.element.TypeElement;
import javax.lang.model.type.TypeMirror;
import javax.tools.Diagnostic;

import com.squareup.javapoet.ClassName;
import com.squareup.javapoet.JavaFile;
import com.squareup.javapoet.MethodSpec;
import com.squareup.javapoet.TypeSpec;

import eu.profinit.manta.bachelor.thesis.logging.api.categorization.BaseCategory;
import eu.profinit.manta.bachelor.thesis.logging.api.categorization.Error;
import eu.profinit.manta.bachelor.thesis.logging.api.categorization.Impact;
import eu.profinit.manta.bachelor.thesis.logging.api.categorization.Severity;
import eu.profinit.manta.bachelor.thesis.logging.api.utils.MessageParser;

/**
 * Annotation processor that processes annotated error methods and generates builders for them.
 * @author jkovac
 */
@SupportedAnnotationTypes("eu.profinit.manta.bachelor.thesis.logging.api.categorization.Error")
public class BuilderProcessor extends AbstractProcessor {
    private Filer filer;

    /**
     * Suffix of the name of the generated builder.
     */
    private static final String BUILDER = "Builder";

    @Override
    public synchronized void init(ProcessingEnvironment processingEnv) {
        super.init(processingEnv);
        filer = processingEnv.getFiler();
    }

    @Override
    public boolean process(Set<? extends TypeElement> annotations, RoundEnvironment roundEnv) {

        Map<String, String> failedElements = new HashMap<>();

        for (Element annotatedElement : roundEnv.getElementsAnnotatedWith(Error.class)) {
            Error error = annotatedElement.getAnnotation(Error.class);
            ErrorDataHolder builderData = new ErrorDataHolder(annotatedElement);

            /*
                Generated builder is in the same package as the category class.
             */
            PackageElement packageElement = processingEnv.getElementUtils().getPackageOf(annotatedElement);
            String packageName = packageElement.getQualifiedName().toString();

            // Parse name of the builder class
            String className = getBuilderName(annotatedElement);

            TypeMirror builderReturnType = ((ExecutableElement) annotatedElement).getReturnType();
            String[] packageClasses = builderReturnType.toString().split("\\.");

            if (packageClasses.length == 0){
                processingEnv.getMessager().printMessage(Diagnostic.Kind.ERROR, "Unknown error, please report it.");
                continue;
            }

            String returnTypeName = packageClasses[packageClasses.length - 1];
            if (!returnTypeName.equals(className)){
                failedElements.put(annotatedElement.getSimpleName().toString(), className);
                continue;
            }

            // Parse placeholders from the messages
            List<String> placeholders = MessageParser.parsePlaceholders(error);

            // Initialize builder of the generated builder
            TypeSpec.Builder generatedClassBuilder = TypeSpec.classBuilder(className)
                    .addModifiers(Modifier.PUBLIC)
                    .superclass(GenericBuilder.class);

            // needed for "return this"
            ClassName returnType = ClassName.get(packageName, className);

            /*
                Method for setting the throwable.
             */
            MethodSpec throwableMethod = MethodSpec.methodBuilder("catching")
                    .addJavadoc("Setter for Throwable.")
                    .addModifiers(Modifier.PUBLIC)
                    .addParameter(Throwable.class, "t")
                    .returns(returnType)
                    .addStatement("setThrowable(t)")
                    .addStatement("return $L", "this")
                    .build();
            generatedClassBuilder.addMethod(throwableMethod);

            /*
                Generating method for each placeholder and adding it to the builder.
             */
            placeholders.forEach(placeholder -> {
                MethodSpec placeholderMethod = MethodSpec.methodBuilder(placeholder)
                        .addJavadoc("Setter for '" + placeholder + "' property.")
                        .addModifiers(Modifier.PUBLIC)
                        .addParameter(Object.class, "value")
                        .returns(returnType)
                        .addStatement("putArg(\"$L\", value)", placeholder)
                        .addStatement("return $L", "this")
                        .build();
                generatedClassBuilder.addMethod(placeholderMethod);
            });

            /*
                Adding constructor of the super class.
             */
            MethodSpec constructor = MethodSpec.constructorBuilder()
                    .addJavadoc("Ctr for the generated builder.")
                    .addParameter(BaseCategory.class, "category")
                    .addStatement("super(category)")
                    .addStatement("setData($S, $S, $S, $S, $T.$L, $T.$L)",
                            builderData.getErrorName(),
                            builderData.getUserMessage(),
                            builderData.getTechnicalMessage(),
                            builderData.getSolution(),
                            Impact.class,
                            builderData.getImpact(),
                            Severity.class,
                            builderData.getSeverity())
                    .build();
            generatedClassBuilder.addMethod(constructor);

            // Add javadoc
            generatedClassBuilder.addJavadoc(
                    "Generated builder for " + getNiceErrorName(annotatedElement)
            );

            TypeSpec builder = generatedClassBuilder.build();

            // create a new source file
            JavaFile javaFile = JavaFile.builder(packageName, builder).build();
            try {
                javaFile.writeTo(filer);
            } catch (IOException e) {
                processingEnv.getMessager().printMessage(Diagnostic.Kind.ERROR,
                        "IOException thrown while trying to create a new source file for " + className);
            }
        }

        failedElements.forEach((key, value) -> {
            processingEnv.getMessager().printMessage(Diagnostic.Kind.ERROR,
                    "Unable to generate builder for error '" + key +"'. Did you mean '" + value +"'?");
        });

        return true;
    }

    private String getNiceErrorName(Element annotatedElement){
        String methodName = annotatedElement.getSimpleName().toString();

        String firstLetterUppercase = methodName.substring(0, 1).toUpperCase();
        String rest = methodName.substring(1);

        StringBuilder sb = new StringBuilder();
        sb.append(firstLetterUppercase);
        sb.append(rest);

        return sb.toString();
    }

    private String getBuilderName(Element annotatedElement) {
        String methodName = annotatedElement.getSimpleName().toString();

        String firstLetterUppercase = methodName.substring(0, 1).toUpperCase();
        String rest = methodName.substring(1);

        StringBuilder sb = new StringBuilder();
        sb.append(firstLetterUppercase);
        sb.append(rest);
        sb.append(BUILDER);

        return sb.toString();
    }
}
