package eu.profinit.manta.bachelor.thesis.logging.api.logging;

import java.util.HashMap;
import java.util.Map;

import javax.lang.model.element.Element;

import eu.profinit.manta.bachelor.thesis.logging.api.categorization.Error;
import eu.profinit.manta.bachelor.thesis.logging.api.categorization.Impact;
import eu.profinit.manta.bachelor.thesis.logging.api.categorization.Severity;
import eu.profinit.manta.bachelor.thesis.logging.api.utils.MessageFormatter;
import eu.profinit.manta.bachelor.thesis.logging.api.utils.NameConverter;

/**
 * Holder of data for the generated builder.
 * @author jkovac
 */
public class ErrorDataHolder {

    /**
     * Fields have the same meaning as in Error annotation.
     */

    private final String errorName;
    private final Map<String, String> args = new HashMap<>();
    private String userMessage;
    private String technicalMessage;
    private String solution;
    private final Impact impact;
    private final Severity severity;

    /**
     * Holder of error data.
     *
     * @param errorName name of the error.
     * @param userMessage user message of the error.
     * @param technicalMessage technical message of the error.
     * @param solution solution of the error.
     * @param impact impact of the error on data lineage.
     * @param severity severity (warn, error, fatal) of the error.
     */
    protected ErrorDataHolder(String errorName,
                           String userMessage,
                           String technicalMessage,
                           String solution,
                           Impact impact,
                           Severity severity) {
        this.errorName = errorName;
        this.userMessage = userMessage;
        this.technicalMessage = technicalMessage;
        this.solution = solution;
        this.impact = impact;
        this.severity = severity;
    }

    /**
     * Setting up data during the generation of the builder in annotation processor.
     * @param annotatedElement The annotated error method.
     */
    protected ErrorDataHolder(Element annotatedElement) {
        Error error = annotatedElement.getAnnotation(Error.class);

        userMessage = error.userMessage();
        technicalMessage = error.technicalMessage();
        solution = error.solution();

        if (error.errorName().equals("")) {
            String methodName = annotatedElement.getSimpleName().toString();
            errorName = NameConverter.normalizeName(methodName);
        } else {
            errorName = error.errorName();
        }

        impact = error.lineageImpact();
        severity = error.severity();
    }

    /**
     * Puts placeholder and its value into a HashMap.
     * @param placeholder Name of the placeholder.
     * @param value Value of the placeholder.
     */
    protected void put(String placeholder, Object value) {
        args.put(placeholder, value.toString());
    }

    /**
     * Replaces placeholders with their values.
     */
    protected void insertPlaceholders() {
        if (args.isEmpty()){
            return;
        }

        userMessage = MessageFormatter.format(userMessage, args);
        technicalMessage = MessageFormatter.format(technicalMessage, args);
        solution = MessageFormatter.format(solution, args);
    }

    /**
     * ---------------------------------------------- GETTERS ----------------------------------------------
     */

    /**
     * Returns the generated error name.
     *
     * @return name of the error
     */
    protected String getErrorName() {
        return errorName;
    }

    /**
     * Returns the user message specified in the annotation.
     *
     * @return user message of the error.
     */
    protected String getUserMessage() {
        return userMessage;
    }

    /**
     * Returns the technical message specified in the annotation.
     *
     * @return technical message of the error.
     */
    protected String getTechnicalMessage() {
        return technicalMessage;
    }

    /**
     * Returns the solution specified in the annotation.
     *
     * @return solution of the error.
     */
    protected String getSolution() {
        return solution;
    }

    /**
     * Returns the impact specified in the annotation.
     *
     * @return impact of the error.
     */
    protected Impact getImpact() {
        return impact;
    }

    /**
     * Returns the severity specified in the annotation.
     *
     * @return severity of the error.
     */
    protected Severity getSeverity() {
        return severity;
    }
}
