package eu.profinit.manta.bachelor.thesis.logging.api.logging;

import org.apache.logging.log4j.Level;

import eu.profinit.manta.bachelor.thesis.logging.api.categorization.BaseCategory;
import eu.profinit.manta.bachelor.thesis.logging.api.categorization.Impact;
import eu.profinit.manta.bachelor.thesis.logging.api.categorization.Severity;
import eu.profinit.manta.bachelor.thesis.logging.api.utils.NameConverter;

/**
 * Class implementing functionality that is common for all generated builders.
 */
public abstract class GenericBuilder {

    /**
     * Throwable when logging exception.
     */
    private Throwable throwable = null;

    /**
     * Normalized name of the category.
     */
    private final String categoryName;

    /**
     * POJO (more or less) of error data.
     */
    private ErrorDataHolder dataHolder;

    /**
     *
     * @param category Category of the error that should be logged.
     */
    public GenericBuilder(BaseCategory category) {
        categoryName = NameConverter.normalizeName(category.getCategoryName());
    }

    /**
     * Logs the built error.
     *
     * @param logger log4j logger that executes the logging.
     * @param ignoreContext flag whether context node should be ignored.
     */
    protected void log(org.apache.logging.log4j.Logger logger, boolean ignoreContext) {
        dataHolder.insertPlaceholders();
        Severity severity = dataHolder.getSeverity();

        switch (severity) {
            case WARN:
                LoggingExecutor.log(logger, Level.WARN, dataHolder, categoryName, throwable, ignoreContext);
                break;
            case ERROR:
                LoggingExecutor.log(logger, Level.ERROR, dataHolder, categoryName, throwable, ignoreContext);
                break;
            case FATAL:
                LoggingExecutor.log(logger, Level.FATAL, dataHolder, categoryName, throwable, ignoreContext);
                break;
            default:
                break;
        }
    }

    /**
     * Sets the basic error data by creating a holder for it.
     *
     * @param errorName name of the error.
     * @param userMessage user message of the error.
     * @param technicalMessage technical message of the error.
     * @param solution solution of the error.
     * @param impact impact of the error.
     * @param severity severity of the error.
     */
    protected void setData(String errorName,
                           String userMessage,
                           String technicalMessage,
                           String solution,
                           Impact impact,
                           Severity severity) {
        dataHolder = new ErrorDataHolder(errorName, userMessage, technicalMessage, solution, impact, severity);
    }

    /**
     * Sets throwable, if there is one, that should be logged. Should be used when logging exceptions.
     * @param throwable Exception that should be logged.
     */
    protected void setThrowable(Throwable throwable) {
        this.throwable = throwable;
    }

    /**
     * Stores the placeholder value in a HashMap.
     * @param placeholder Name of the placeholder in a message. Set up in a message in a format %{placeholderName}
     * @param value Value that should later replace the placeholder.
     */
    protected void putArg(String placeholder, Object value) {
        dataHolder.put(placeholder, value);
    }

    /**
     * Convert built log error to String, as it would be loggeed to a file.
     * @return error in string
     */
    public String toString(){
        dataHolder.insertPlaceholders();

        StringBuilder sb = new StringBuilder();

        sb.append(categoryName);
        sb.append(" ");
        sb.append(dataHolder.getErrorName());
        sb.append(System.lineSeparator());
        appendMessage(sb, "User message: ", dataHolder.getUserMessage());
        appendMessage(sb, "Technical message: ", dataHolder.getTechnicalMessage());
        appendMessage(sb, "Solution: ", dataHolder.getSolution());
        appendMessage(sb, "Impact: ", dataHolder.getImpact().name());

        return sb.toString();
    }

    private void appendMessage(StringBuilder sb, String messageType, String message){
        sb.append(messageType);
        sb.append(message);
        sb.append(System.lineSeparator());
    }

}
