package eu.profinit.manta.bachelor.thesis.logging.api.logging;

import org.apache.logging.log4j.Level;
import org.apache.logging.log4j.Logger;
import org.apache.logging.log4j.Marker;
import org.apache.logging.log4j.MarkerManager;
import org.apache.logging.log4j.ThreadContext;

/**
 * Calls the Log4j logger and logs the error.
 * @author jkovac
 */
public final class LoggingExecutor {

    /**
     * Keys of the error attributes used in Log4j configuration.
     */

    private static final String CATEGORY_KEY = "CATEGORY";
    private static final String ERROR_KEY = "ERROR_TYPE";

    private static final String USER_MESSAGE_KEY = "USER_MESSAGE";
    private static final String TECHNICAL_MESSAGE_KEY = "TECHNICAL_MESSAGE";
    private static final String SOLUTION_KEY = "SOLUTION";
    private static final String IMPACT_KEY = "IMPACT";

    /**
     * Logging marker for separating error logs from new API without context.
     */
    private static final Marker ERROR_MARKER = MarkerManager.getMarker("ERROR");

    /**
     * Private constructor.
     */
    private LoggingExecutor() {
    }

    /**
     * Logs all of the error data using the log4j logger.
     *
     * @param logger log4j logger that logs the messages.
     * @param level severity of the message specified in the annotation.
     * @param dataHolder holder of the error data (name, messages, ...).
     * @param category normalized name of the category.
     * @param t exception that is logged, may be null.
     * @param ignoreContext flag whether context node should be ignored when logging
     */
    protected static void log(Logger logger, Level level, ErrorDataHolder dataHolder, String category, Throwable t, boolean ignoreContext) {
        ThreadContext.put(CATEGORY_KEY, category);
        ThreadContext.put(ERROR_KEY, dataHolder.getErrorName());
        ThreadContext.put(USER_MESSAGE_KEY, dataHolder.getUserMessage());
        ThreadContext.put(TECHNICAL_MESSAGE_KEY, dataHolder.getTechnicalMessage());
        ThreadContext.put(SOLUTION_KEY, dataHolder.getSolution());
        ThreadContext.put(IMPACT_KEY, dataHolder.getImpact().name());

        if (ignoreContext && ThreadContext.containsKey("context-set")){
            ThreadContext.put("context-set", "false");
        }

        if (t == null) {
            logger.log(level, ERROR_MARKER, "");
        } else {
            logger.log(level, ERROR_MARKER, "", t);
        }

        if (ignoreContext){
            ThreadContext.put("context-set", "true");
        }

        ThreadContext.remove(CATEGORY_KEY);
        ThreadContext.remove(ERROR_KEY);
        ThreadContext.remove(USER_MESSAGE_KEY);
        ThreadContext.remove(TECHNICAL_MESSAGE_KEY);
        ThreadContext.remove(SOLUTION_KEY);
        ThreadContext.remove(IMPACT_KEY);
    }
}
