package eu.profinit.manta.bachelor.thesis.logging.proof.of.concept;

import eu.profinit.manta.bachelor.thesis.logging.api.logging.Logger;
import eu.profinit.manta.bachelor.thesis.logging.api.logging.LoggingContext;
import eu.profinit.manta.bachelor.thesis.logging.proof.of.concept.Errors.Categories;

/**
 * Logging proof of concept
 *
 * IMPORTANT
 * To be able to run this in Intellij IDEA the necessary settings are:
 * 1. Type 'mvn clean compile' in the terminal and execute.
 * 2. When using Intellij IDEA: File - Project structure - Modules - Click on 'manta-bachelor-thesis-proof-of-concept'
 * 3. Navigate to ./target/generated-sources/annotations folder.
 * 4. Mark the 'annotations' folder as a 'Source'.
 *
 * Doing this will enable IDEA to detect the generated source files and will therefore prevent outputting errors regarding missing classes.
 *
 * @author jkovac
 */
public class Main {

    /**
     * Logger
     */
    private static Logger logger = new Logger(Main.class);

    /**
     * Simulate couple of workflow executions and scenario executions which will be then
     * stored in the repository and able to be queried from the FE.
     * @param args args
     */
    public static void main(String[] args) {

        // same for all executions
        LoggingContext.setTechnology("Oracle");
        LoggingContext.setConnectionID("OracleTestDB");


        // simulate three workflow executions
        for (int i = 1; i < 4; ++i){

            // set a workflow execution context
            LoggingContext.setWorkflowExecutionId("WE" + i);

            // simulate three scenario executions, 9 in total
            for (int k = 1; k < 4; ++k){

                // set scenario execution id
                LoggingContext.setScenarioExecutionId("SE" + i + k);

                switch (k){
                    case 1:
                        LoggingContext.setScenario("Oracle Extraction");
                        LoggingContext.setPhase("Extraction");
                        break;
                    case 2:
                        LoggingContext.setScenario("Oracle DDL Dataflow Analysis");
                        LoggingContext.setPhase("Analysis");
                        break;
                    case 3:
                        LoggingContext.setScenario("Oracle Upload Metadata");
                        LoggingContext.setPhase("Upload");
                        break;
                }

                // loop three times for different context files
                for(int m = 1; m < 4; ++m){
                    String fileName = "script0" + m + ".sql";

                    LoggingContext.setContext(fileName);

                    logger.info("Trying to read the file {}...", fileName);
                    logger.info("Opening input stream...");
                    logger.log(Categories.ioErrors().fileLocked());
                    logger.info("Closing input stream...");

                    logger.info("Trying to read the file {}...", fileName);
                    logger.log(Categories.ioErrors().fileNotFound()
                            .operation("deletion")
                            .path("C:/Windows/System32"));
                    logger.info("Closing input stream...");

                    String encoding = "CODE-1234";
                    logger.info("Trying to read {} using {} encoding...", fileName, encoding);
                    logger.log(Categories.ioErrors().invalidEncoding()
                            .encoding(encoding)
                            .website("www.fel.cvut.cz"));
                    logger.info("Reading {} using UTF-8.");

                    String fileFormat = ".sql";
                    logger.info("Trying to generate a PDF file from {}...", fileName);
                    logger.log(Categories.ioErrors().invalidFileFormat()
                            .allowedFormats(".doc, .txt")
                            .format(fileFormat));

                    logger.info("Trying to get server resources...");
                    logger.log(Categories.httpErrors().badResponse()
                            .code(404)
                            .message("Bad request: missing parameter 'subject'.")
                            .website("cw.felk.cvut.cz/brute/student/index.php"));

                    logger.info("Trying to parse binary tree...");
                    logger.log(Categories.systemErrors().stackOverflowError().operation("parsing binary tree"));

                    logger.info("Adding OPT exam dates to KOS...");
                    logger.log(Categories.systemErrors().outOfMemoryError()
                            .catching(new OutOfMemoryError()),
                            true); // context file not used here

                    logger.info("Trying to log in to BRUTE...");
                    logger.log(Categories.authorizationErrors().badPassword()
                            .userName("kovacj"),
                            true);

                    logger.info("Trying to install Windows 10...");
                    logger.log(Categories.authorizationErrors().invalidLicense()
                            .userName("kovacj")
                            .licenseHash("0xa45ads8754da858"), true);

                    logger.info("Trying to adjust grades in BRUTE...");
                    logger.log(Categories.authorizationErrors().missingUserPrivilege()
                            .userName("novakj")
                            .userRole("STUDENT")
                            .action("change grades")
                            .requiredRole("TEACHER"),
                            true);

                }
            }
        }

    }

}
